@@ -29,13 +29,15 @@
 #include "iw_ndis.h"
 #include "wrapndis.h"
 
+#ifdef CONFIG_WIRELESS_EXT
+
 static int freq_chan[] = { 2412, 2417, 2422, 2427, 2432, 2437, 2442,
 			   2447, 2452, 2457, 2462, 2467, 2472, 2484 };
 
 static const char *network_names[] = {"IEEE 802.11FH", "IEEE 802.11b",
 				      "IEEE 802.11a", "IEEE 802.11g", "Auto"};
 
-int set_essid(struct ndis_device *wnd, const char *ssid, int ssid_len)
+static int set_essid(struct ndis_device *wnd, const char *ssid, int ssid_len)
 {
 	NDIS_STATUS res;
 	struct ndis_essid req;
@@ -57,6 +59,81 @@ int set_essid(struct ndis_device *wnd, const char *ssid, int ssid_len)
 	EXIT2(return 0);
 }
 
+static int set_iw_auth_mode(struct ndis_device *wnd, int wpa_version,
+			    int auth_80211_alg)
+{
+	NDIS_STATUS res;
+	ULONG auth_mode;
+
+	ENTER2("%d, %d", wpa_version, auth_80211_alg);
+	if (wpa_version & IW_AUTH_WPA_VERSION_WPA2) {
+		if (wnd->iw_auth_key_mgmt & IW_AUTH_KEY_MGMT_802_1X)
+			auth_mode = Ndis802_11AuthModeWPA2;
+		else
+			auth_mode = Ndis802_11AuthModeWPA2PSK;
+	} else if (wpa_version & IW_AUTH_WPA_VERSION_WPA) {
+		if (wnd->iw_auth_key_mgmt & IW_AUTH_KEY_MGMT_802_1X)
+			auth_mode = Ndis802_11AuthModeWPA;
+		else if (wnd->iw_auth_key_mgmt & IW_AUTH_KEY_MGMT_PSK)
+			auth_mode = Ndis802_11AuthModeWPAPSK;
+		else
+			auth_mode = Ndis802_11AuthModeWPANone;
+	} else if (auth_80211_alg & IW_AUTH_ALG_SHARED_KEY) {
+		if (auth_80211_alg & IW_AUTH_ALG_OPEN_SYSTEM)
+			auth_mode = Ndis802_11AuthModeAutoSwitch;
+		else
+			auth_mode = Ndis802_11AuthModeShared;
+	} else
+		auth_mode = Ndis802_11AuthModeOpen;
+
+	res = mp_set_int(wnd, OID_802_11_AUTHENTICATION_MODE, auth_mode);
+	if (res) {
+		WARNING("setting auth mode to %u failed (%08X)",
+			auth_mode, res);
+		if (res == NDIS_STATUS_INVALID_DATA)
+			EXIT2(return -EINVAL);
+		return -EOPNOTSUPP;
+	}
+	wnd->iw_auth_wpa_version = wpa_version;
+	wnd->iw_auth_80211_alg = auth_80211_alg;
+	EXIT2(return 0);
+}
+
+static int set_auth_mode(struct ndis_device *wnd)
+{
+	return set_iw_auth_mode(wnd, wnd->iw_auth_wpa_version,
+				wnd->iw_auth_80211_alg);
+}
+
+static enum ndis_priv_filter ndis_priv_mode(struct ndis_device *wnd)
+{
+	if (wnd->iw_auth_wpa_version & IW_AUTH_WPA_VERSION_WPA2 ||
+	    wnd->iw_auth_wpa_version & IW_AUTH_WPA_VERSION_WPA)
+		return Ndis802_11PrivFilter8021xWEP;
+	else
+		return Ndis802_11PrivFilterAcceptAll;
+}
+
+static int set_priv_filter(struct ndis_device *wnd)
+{
+	NDIS_STATUS res;
+	ULONG flags;
+
+	flags = ndis_priv_mode(wnd);
+	ENTER2("filter: %d", flags);
+	res = mp_set_int(wnd, OID_802_11_PRIVACY_FILTER, flags);
+	if (res)
+		TRACE2("setting privacy filter to %d failed (%08X)",
+		       flags, res);
+	EXIT2(return 0);
+}
+
+static int set_encr_mode(struct ndis_device *wnd)
+{
+	return set_iw_encr_mode(wnd, wnd->iw_auth_cipher_pairwise,
+				wnd->iw_auth_cipher_group);
+}
+
 static int set_assoc_params(struct ndis_device *wnd)
 {
 	TRACE2("wpa_version=0x%x auth_alg=0x%x key_mgmt=0x%x "
@@ -74,21 +151,16 @@ static int iw_set_essid(struct net_device *dev, struct iw_request_info *info,
 			union iwreq_data *wrqu, char *extra)
 {
 	struct ndis_device *wnd = netdev_priv(dev);
-	char ssid[NDIS_ESSID_MAX_SIZE];
 	int length;
 
 	ENTER2("");
-	memset(ssid, 0, sizeof(ssid));
 	/* there is no way to turn off essid other than to set to
 	 * random bytes; instead, we use off to mean any */
 	if (wrqu->essid.flags) {
-		/* wireless-tools prior to version 20 add extra 1, and
-		 * later than 20 don't! Deal with that mess */
-		length = wrqu->essid.length - 1;
-		if (length > 0)
+		length = wrqu->essid.length;
+		/* Strip '\0' appended by wireless extensions 19 and older */
+		if (length > 0 && extra[length - 1] == '\0')
 			length--;
-		while (length < wrqu->essid.length && extra[length])
-			length++;
 		TRACE2("%d", length);
 		if (length <= 0 || length > NDIS_ESSID_MAX_SIZE)
 			EXIT2(return -EINVAL);
@@ -97,8 +169,7 @@ static int iw_set_essid(struct net_device *dev, struct iw_request_info *info,
 
 	set_assoc_params(wnd);
 
-	memcpy(ssid, extra, length);
-	if (set_essid(wnd, ssid, length))
+	if (set_essid(wnd, extra, length))
 		EXIT2(return -EINVAL);
 
 	EXIT2(return 0);
@@ -120,15 +191,59 @@ static int iw_get_essid(struct net_device *dev, struct iw_request_info *info,
 	}
 	memcpy(extra, req.essid, req.length);
 	if (req.length > 0)
-		wrqu->essid.flags  = 1;
+		wrqu->essid.flags = 1;
 	else
 		wrqu->essid.flags = 0;
 	wrqu->essid.length = req.length;
 	EXIT2(return 0);
 }
 
-int set_infra_mode(struct ndis_device *wnd,
-		   enum ndis_infrastructure_mode mode)
+/* index must be 0 - N, as per NDIS */
+static int add_wep_key(struct ndis_device *wnd, char *key, int key_len,
+		       int index)
+{
+	struct ndis_encr_key ndis_key;
+	NDIS_STATUS res;
+
+	ENTER2("key index: %d, length: %d", index, key_len);
+	if (key_len <= 0 || key_len > NDIS_ENCODING_TOKEN_MAX) {
+		WARNING("invalid key length (%d)", key_len);
+		EXIT2(return -EINVAL);
+	}
+	if (index < 0 || index >= MAX_ENCR_KEYS) {
+		WARNING("invalid key index (%d)", index);
+		EXIT2(return -EINVAL);
+	}
+	ndis_key.struct_size = sizeof(ndis_key);
+	ndis_key.length = key_len;
+	memcpy(&ndis_key.key, key, key_len);
+	ndis_key.index = index;
+
+	if (index == wnd->encr_info.tx_key_index) {
+		ndis_key.index |= (1 << 31);
+		res = set_iw_encr_mode(wnd, IW_AUTH_CIPHER_WEP104,
+				       IW_AUTH_CIPHER_NONE);
+		if (res)
+			WARNING("encryption couldn't be enabled (%08X)", res);
+	}
+	TRACE2("key %d: " MACSTRSEP, index, MAC2STR(key));
+	res = mp_set(wnd, OID_802_11_ADD_WEP, &ndis_key, sizeof(ndis_key));
+	if (res) {
+		WARNING("adding encryption key %d failed (%08X)",
+			index+1, res);
+		EXIT2(return -EINVAL);
+	}
+
+	/* Atheros driver messes up ndis_key during ADD_WEP, so
+	 * don't rely on that; instead use info in key and key_len */
+	wnd->encr_info.keys[index].length = key_len;
+	memcpy(&wnd->encr_info.keys[index].key, key, key_len);
+
+	EXIT2(return 0);
+}
+
+static int set_infra_mode(struct ndis_device *wnd,
+			  enum ndis_infrastructure_mode mode)
 {
 	NDIS_STATUS res;
 	unsigned int i;
@@ -137,7 +252,7 @@ int set_infra_mode(struct ndis_device *wnd,
 	res = mp_query_int(wnd, OID_802_11_INFRASTRUCTURE_MODE,
 			   &wnd->infrastructure_mode);
 	if (res != NDIS_STATUS_SUCCESS) {
-		WARNING("getting operating mode to failed (%08X)", res);
+		WARNING("getting operating mode failed (%08X)", res);
 		EXIT2(return -EINVAL);
 	}
 	if (wnd->infrastructure_mode == mode)
@@ -206,7 +321,7 @@ static int iw_get_infra_mode(struct net_device *dev,
 		EXIT2(return -EOPNOTSUPP);
 	}
 
-	switch(ndis_mode) {
+	switch (ndis_mode) {
 	case Ndis802_11IBSS:
 		iw_mode = IW_MODE_ADHOC;
 		break;
@@ -226,12 +341,10 @@ static int iw_get_infra_mode(struct net_device *dev,
 
 static const char *network_type_to_name(int net_type)
 {
-	if (net_type >= 0 &&
-	    net_type < (sizeof(network_names)/sizeof(network_names[0])))
+	if (net_type >= 0 && net_type < ARRAY_SIZE(network_names))
 		return network_names[net_type];
 	else
-		return network_names[sizeof(network_names) /
-				     sizeof(network_names[0]) - 1];
+		return network_names[ARRAY_SIZE(network_names) - 1];
 }
 
 static int iw_get_network_type(struct net_device *dev,
@@ -250,7 +363,7 @@ static int iw_get_network_type(struct net_device *dev,
 		network_type = -1;
 	}
 	strncpy(wrqu->name, network_type_to_name(network_type),
-	        sizeof(wrqu->name) - 1);
+		sizeof(wrqu->name) - 1);
 	wrqu->name[sizeof(wrqu->name)-1] = 0;
 	return 0;
 }
@@ -308,8 +421,7 @@ static int iw_set_freq(struct net_device *dev, struct iw_request_info *info,
 	}
 
 	if (wrqu->freq.m < 1000 && wrqu->freq.e == 0) {
-		if (wrqu->freq.m >= 1 &&
-		    wrqu->freq.m <= (sizeof(freq_chan) / sizeof(freq_chan[0])))
+		if (wrqu->freq.m >= 1 && wrqu->freq.m <= ARRAY_SIZE(freq_chan))
 			req.ds_config = freq_chan[wrqu->freq.m - 1] * 1000;
 		else
 			return -EINVAL;
@@ -406,7 +518,7 @@ static int iw_set_bitrate(struct net_device *dev, struct iw_request_info *info,
 	struct ndis_device *wnd = netdev_priv(dev);
 	int i, n;
 	NDIS_STATUS res;
-	ndis_rates_ex rates;
+	UCHAR rates[NDIS_MAX_RATES_EX];
 
 	ENTER2("");
 	if (wrqu->bitrate.fixed == 0)
@@ -568,46 +680,6 @@ static int iw_set_ap_address(struct net_device *dev,
 	EXIT2(return 0);
 }
 
-int set_iw_auth_mode(struct ndis_device *wnd, int wpa_version,
-		     int auth_80211_alg)
-{
-	NDIS_STATUS res;
-	ULONG auth_mode;
-
-	ENTER2("%d, %d", wpa_version, auth_80211_alg);
-	if (wpa_version & IW_AUTH_WPA_VERSION_WPA2) {
-		if (wnd->iw_auth_key_mgmt & IW_AUTH_KEY_MGMT_802_1X)
-			auth_mode = Ndis802_11AuthModeWPA2;
-		else
-			auth_mode = Ndis802_11AuthModeWPA2PSK;
-	} else if (wpa_version & IW_AUTH_WPA_VERSION_WPA) {
-		if (wnd->iw_auth_key_mgmt & IW_AUTH_KEY_MGMT_802_1X)
-			auth_mode = Ndis802_11AuthModeWPA;
-		else if (wnd->iw_auth_key_mgmt & IW_AUTH_KEY_MGMT_PSK)
-			auth_mode = Ndis802_11AuthModeWPAPSK;
-		else
-			auth_mode = Ndis802_11AuthModeWPANone;
-	} else if (auth_80211_alg & IW_AUTH_ALG_SHARED_KEY) {
-		if (auth_80211_alg & IW_AUTH_ALG_OPEN_SYSTEM)
-			auth_mode = Ndis802_11AuthModeAutoSwitch;
-		else
-			auth_mode = Ndis802_11AuthModeShared;
-	} else
-		auth_mode = Ndis802_11AuthModeOpen;
-
-	res = mp_set_int(wnd, OID_802_11_AUTHENTICATION_MODE, auth_mode);
-	if (res) {
-		WARNING("setting auth mode to %u failed (%08X)",
-			auth_mode, res);
-		if (res == NDIS_STATUS_INVALID_DATA)
-			EXIT2(return -EINVAL);
-		return -EOPNOTSUPP;
-	}
-	wnd->iw_auth_wpa_version = wpa_version;
-	wnd->iw_auth_80211_alg = auth_80211_alg;
-	EXIT2(return 0);
-}
-
 int set_ndis_auth_mode(struct ndis_device *wnd, ULONG auth_mode)
 {
 	NDIS_STATUS res;
@@ -661,12 +733,6 @@ int set_ndis_auth_mode(struct ndis_device *wnd, ULONG auth_mode)
 	EXIT2(return 0);
 }
 
-int set_auth_mode(struct ndis_device *wnd)
-{
-	return set_iw_auth_mode(wnd, wnd->iw_auth_wpa_version,
-				wnd->iw_auth_80211_alg);
-}
-
 int get_ndis_auth_mode(struct ndis_device *wnd)
 {
 	ULONG mode;
@@ -715,12 +781,6 @@ int set_iw_encr_mode(struct ndis_device *wnd, int cipher_pairwise,
 	EXIT2(return 0);
 }
 
-int set_encr_mode(struct ndis_device *wnd)
-{
-	return set_iw_encr_mode(wnd, wnd->iw_auth_cipher_pairwise,
-				wnd->iw_auth_cipher_group);
-}
-
 int get_ndis_encr_mode(struct ndis_device *wnd)
 {
 	ULONG mode;
@@ -806,50 +866,6 @@ static int iw_get_encr(struct net_device *dev, struct iw_request_info *info,
 	EXIT2(return 0);
 }
 
-/* index must be 0 - N, as per NDIS  */
-int add_wep_key(struct ndis_device *wnd, char *key, int key_len,
-		int index)
-{
-	struct ndis_encr_key ndis_key;
-	NDIS_STATUS res;
-
-	ENTER2("key index: %d, length: %d", index, key_len);
-	if (key_len <= 0 || key_len > NDIS_ENCODING_TOKEN_MAX) {
-		WARNING("invalid key length (%d)", key_len);
-		EXIT2(return -EINVAL);
-	}
-	if (index < 0 || index >= MAX_ENCR_KEYS) {
-		WARNING("invalid key index (%d)", index);
-		EXIT2(return -EINVAL);
-	}
-	ndis_key.struct_size = sizeof(ndis_key);
-	ndis_key.length = key_len;
-	memcpy(&ndis_key.key, key, key_len);
-	ndis_key.index = index;
-
-	if (index == wnd->encr_info.tx_key_index) {
-		ndis_key.index |= (1 << 31);
-		res = set_iw_encr_mode(wnd, IW_AUTH_CIPHER_WEP104,
-				       IW_AUTH_CIPHER_NONE);
-		if (res)
-			WARNING("encryption couldn't be enabled (%08X)", res);
-	}
-	TRACE2("key %d: " MACSTRSEP, index, MAC2STR(key));
-	res = mp_set(wnd, OID_802_11_ADD_WEP, &ndis_key, sizeof(ndis_key));
-	if (res) {
-		WARNING("adding encryption key %d failed (%08X)",
-			index+1, res);
-		EXIT2(return -EINVAL);
-	}
-
-	/* Atheros driver messes up ndis_key during ADD_WEP, so
-	 * don't rely on that; instead use info in key and key_len */
-	wnd->encr_info.keys[index].length = key_len;
-	memcpy(&wnd->encr_info.keys[index].key, key, key_len);
-
-	EXIT2(return 0);
-}
-
 /* remove_key is for both wep and wpa */
 static int remove_key(struct ndis_device *wnd, int index,
 		      mac_address bssid)
@@ -864,21 +880,18 @@ static int remove_key(struct ndis_device *wnd, int index,
 	    wnd->iw_auth_cipher_pairwise == IW_AUTH_CIPHER_CCMP ||
 	    wnd->iw_auth_cipher_group == IW_AUTH_CIPHER_TKIP ||
 	    wnd->iw_auth_cipher_group == IW_AUTH_CIPHER_CCMP) {
-		struct ndis_remove_key remove_key;
-		remove_key.struct_size = sizeof(remove_key);
-		remove_key.index = index;
+		struct ndis_remove_key rmkey;
+		rmkey.struct_size = sizeof(rmkey);
+		rmkey.index = index;
 		if (bssid) {
 			/* pairwise key */
 			if (memcmp(bssid, "\xff\xff\xff\xff\xff\xff",
 				   ETH_ALEN) != 0)
-				remove_key.index |= (1 << 30);
-			memcpy(remove_key.bssid, bssid,
-			       sizeof(remove_key.bssid));
+				rmkey.index |= (1 << 30);
+			memcpy(rmkey.bssid, bssid, sizeof(rmkey.bssid));
 		} else
-			memset(remove_key.bssid, 0xff,
-			       sizeof(remove_key.bssid));
-		if (mp_set(wnd, OID_802_11_REMOVE_KEY,
-			   &remove_key, sizeof(remove_key)))
+			memset(rmkey.bssid, 0xff, sizeof(rmkey.bssid));
+		if (mp_set(wnd, OID_802_11_REMOVE_KEY, &rmkey, sizeof(rmkey)))
 			EXIT2(return -EINVAL);
 	} else {
 		ndis_key_index keyindex = index;
@@ -919,7 +932,7 @@ static int iw_set_wep(struct net_device *dev, struct iw_request_info *info,
 	else
 		index = encr_info->tx_key_index;
 
-	if (index < 0 || index >= MAX_ENCR_KEYS) {
+	if (index >= MAX_ENCR_KEYS) {
 		WARNING("encryption index out of range (%u)", index);
 		EXIT2(return -EINVAL);
 	}
@@ -991,7 +1004,7 @@ static int iw_set_nick(struct net_device *dev, struct iw_request_info *info,
 {
 	struct ndis_device *wnd = netdev_priv(dev);
 
-	if (wrqu->data.length >= IW_ESSID_MAX_SIZE || wrqu->data.length <= 0)
+	if (wrqu->data.length >= sizeof(wnd->nick))
 		return -EINVAL;
 	memcpy(wnd->nick, extra, wrqu->data.length);
 	wnd->nick[wrqu->data.length] = 0;
@@ -1022,22 +1035,28 @@ static char *ndis_translate_scan(struct net_device *dev,
 {
 	struct iw_event iwe;
 	char *current_val;
+	char *ret;
 	int i, nrates;
-	unsigned char buf[MAX_WPA_IE_LEN * 2 + 30];
+	unsigned char custom_str[64];
 	struct ndis_wlan_bssid *bssid;
 	struct ndis_wlan_bssid_ex *bssid_ex;
+	int extended;
 
 	ENTER2("%p, %p", event, item);
 	bssid = item;
 	bssid_ex = item;
+	extended = (bssid->length > offsetof(struct ndis_wlan_bssid_ex, var));
+
 	/* add mac address */
 	memset(&iwe, 0, sizeof(iwe));
 	iwe.cmd = SIOCGIWAP;
 	iwe.u.ap_addr.sa_family = ARPHRD_ETHER;
 	iwe.len = IW_EV_ADDR_LEN;
 	memcpy(iwe.u.ap_addr.sa_data, bssid->mac, ETH_ALEN);
-	event = iwe_stream_add_event(info, event, end_buf, &iwe,
-				     IW_EV_ADDR_LEN);
+	ret = iwe_stream_add_event(info, event, end_buf, &iwe, IW_EV_ADDR_LEN);
+	if (ret == event)
+		return NULL;
+	event = ret;
 
 	/* add essid */
 	memset(&iwe, 0, sizeof(iwe));
@@ -1047,15 +1066,20 @@ static char *ndis_translate_scan(struct net_device *dev,
 		iwe.u.data.length = IW_ESSID_MAX_SIZE;
 	iwe.u.data.flags = 1;
 	iwe.len = IW_EV_POINT_LEN + iwe.u.data.length;
-	event = iwe_stream_add_point(info, event, end_buf, &iwe,
-				     bssid->ssid.essid);
+	ret = iwe_stream_add_point(info, event, end_buf, &iwe,
+				   bssid->ssid.essid);
+	if (ret == event)
+		return NULL;
+	event = ret;
 
 	/* add protocol name */
 	memset(&iwe, 0, sizeof(iwe));
 	iwe.cmd = SIOCGIWNAME;
 	strncpy(iwe.u.name, network_type_to_name(bssid->net_type), IFNAMSIZ);
-	event = iwe_stream_add_event(info, event, end_buf, &iwe,
-				     IW_EV_CHAR_LEN);
+	ret = iwe_stream_add_event(info, event, end_buf, &iwe, IW_EV_CHAR_LEN);
+	if (ret == event)
+		return NULL;
+	event = ret;
 
 	/* add mode */
 	memset(&iwe, 0, sizeof(iwe));
@@ -1063,11 +1087,13 @@ static char *ndis_translate_scan(struct net_device *dev,
 	if (bssid->mode == Ndis802_11IBSS)
 		iwe.u.mode = IW_MODE_ADHOC;
 	else if (bssid->mode == Ndis802_11Infrastructure)
-		iwe.u.mode = IW_MODE_INFRA;
+		iwe.u.mode = IW_MODE_MASTER;
 	else // if (bssid->mode == Ndis802_11AutoUnknown)
 		iwe.u.mode = IW_MODE_AUTO;
-	event = iwe_stream_add_event(info, event, end_buf, &iwe,
-				     IW_EV_UINT_LEN);
+	ret = iwe_stream_add_event(info, event, end_buf, &iwe, IW_EV_UINT_LEN);
+	if (ret == event)
+		return NULL;
+	event = ret;
 
 	/* add freq */
 	memset(&iwe, 0, sizeof(iwe));
@@ -1082,8 +1108,10 @@ static char *ndis_translate_scan(struct net_device *dev,
 	/* convert from kHz to Hz */
 	iwe.u.freq.e += 3;
 	iwe.len = IW_EV_FREQ_LEN;
-	event = iwe_stream_add_event(info, event, end_buf, &iwe,
-				     IW_EV_FREQ_LEN);
+	ret = iwe_stream_add_event(info, event, end_buf, &iwe, IW_EV_FREQ_LEN);
+	if (ret == event)
+		return NULL;
+	event = ret;
 
 	/* add qual */
 	memset(&iwe, 0, sizeof(iwe));
@@ -1095,10 +1123,12 @@ static char *ndis_translate_scan(struct net_device *dev,
 		i = 100;
 	iwe.u.qual.level = bssid->rssi;
 	iwe.u.qual.noise = WL_NOISE;
-	iwe.u.qual.qual  = i;
+	iwe.u.qual.qual = i;
 	iwe.len = IW_EV_QUAL_LEN;
-	event = iwe_stream_add_event(info, event, end_buf, &iwe,
-				     IW_EV_QUAL_LEN);
+	ret = iwe_stream_add_event(info, event, end_buf, &iwe, IW_EV_QUAL_LEN);
+	if (ret == event)
+		return NULL;
+	event = ret;
 
 	/* add key info */
 	memset(&iwe, 0, sizeof(iwe));
@@ -1109,25 +1139,30 @@ static char *ndis_translate_scan(struct net_device *dev,
 		iwe.u.data.flags = IW_ENCODE_ENABLED | IW_ENCODE_NOKEY;
 	iwe.u.data.length = 0;
 	iwe.len = IW_EV_POINT_LEN;
-	event = iwe_stream_add_point(info, event, end_buf, &iwe,
-				     bssid->ssid.essid);
+	ret = iwe_stream_add_point(info, event, end_buf, &iwe,
+				   bssid->ssid.essid);
+	if (ret == event)
+		return NULL;
+	event = ret;
 
 	/* add rate */
 	memset(&iwe, 0, sizeof(iwe));
 	current_val = event + iwe_stream_lcp_len(info);
 	iwe.cmd = SIOCGIWRATE;
-	if (bssid->length > sizeof(*bssid))
-		nrates = NDIS_MAX_RATES_EX;
+	if (extended)
+		nrates = ARRAY_SIZE(bssid->rates);
 	else
-		nrates = NDIS_MAX_RATES;
-	for (i = 0 ; i < nrates ; i++) {
-		if (bssid->rates[i] & 0x7f) {
+		nrates = ARRAY_SIZE(bssid_ex->rates_ex);
+	for (i = 0; i < nrates; i++) {
+		if (bssid_ex->rates_ex[i] & 0x7f) {
 			iwe.u.bitrate.value = ((bssid->rates[i] & 0x7f) *
 					       500000);
-			current_val = iwe_stream_add_value(info, event,
-							   current_val,
-							   end_buf, &iwe,
-							   IW_EV_PARAM_LEN);
+			ret = iwe_stream_add_value(info, event, current_val,
+						   end_buf, &iwe,
+						   IW_EV_PARAM_LEN);
+			if (ret == current_val)
+				return NULL;
+			current_val = ret;
 		}
 	}
 
@@ -1136,47 +1171,47 @@ static char *ndis_translate_scan(struct net_device *dev,
 
 	memset(&iwe, 0, sizeof(iwe));
 	iwe.cmd = IWEVCUSTOM;
-	sprintf(buf, "bcn_int=%d", bssid->config.beacon_period);
-	iwe.u.data.length = strlen(buf);
-	event = iwe_stream_add_point(info, event, end_buf, &iwe, buf);
+	sprintf(custom_str, "bcn_int=%d", bssid->config.beacon_period);
+	iwe.u.data.length = strlen(custom_str);
+	ret = iwe_stream_add_point(info, event, end_buf, &iwe, custom_str);
+	if (ret == event)
+		return NULL;
+	event = ret;
 
 	memset(&iwe, 0, sizeof(iwe));
 	iwe.cmd = IWEVCUSTOM;
-	sprintf(buf, "atim=%u", bssid->config.atim_window);
-	iwe.u.data.length = strlen(buf);
-	event = iwe_stream_add_point(info, event, end_buf, &iwe, buf);
+	sprintf(custom_str, "atim=%u", bssid->config.atim_window);
+	iwe.u.data.length = strlen(custom_str);
+	ret = iwe_stream_add_point(info, event, end_buf, &iwe, custom_str);
+	if (ret == event)
+		return NULL;
+	event = ret;
 
-	TRACE2("%d, %u", bssid->length, (unsigned int)sizeof(*bssid));
-	if (bssid->length > sizeof(*bssid)) {
-		unsigned char *iep = (unsigned char *)bssid_ex->ies +
-			sizeof(struct ndis_fixed_ies);
-		no_warn_unused unsigned char *end = iep + bssid_ex->ie_length;
+	TRACE2("%d, %zu", bssid->length, sizeof(*bssid));
+	if (extended) {
+		struct ndis_variable_ies *iep = bssid_ex->var;
+		unsigned char *end = (unsigned char *)&bssid_ex->fixed +
+			bssid_ex->ie_length;
 
-		while (iep + 1 < end && iep + 2 + iep[1] <= end) {
-			unsigned char ielen = 2 + iep[1];
+		while (&iep->length < end && &iep->data[iep->length] <= end) {
+			unsigned char ielen = iep->length + 2;
 
-			if (ielen > SSID_MAX_WPA_IE_LEN) {
-				iep += ielen;
-				continue;
-			}
-			if ((iep[0] == WLAN_EID_GENERIC && iep[1] >= 4 &&
-			     memcmp(iep + 2, "\x00\x50\xf2\x01", 4) == 0) ||
-			    iep[0] == RSN_INFO_ELEM) {
-				memset(&iwe, 0, sizeof(iwe));
-				iwe.cmd = IWEVGENIE;
-				iwe.u.data.length = ielen;
-				event = iwe_stream_add_point(info, event,
-							     end_buf, &iwe,
-							     iep);
-			}
-			iep += ielen;
+			memset(&iwe, 0, sizeof(iwe));
+			iwe.cmd = IWEVGENIE;
+			iwe.u.data.length = ielen;
+			ret = iwe_stream_add_point(info, event, end_buf, &iwe,
+						   (char *)iep);
+			if (ret == event)
+				return NULL;
+			event = ret;
+			iep = (typeof(iep))&iep->data[iep->length];
 		}
 	}
 	TRACE2("event = %p, current_val = %p", event, current_val);
 	EXIT2(return event);
 }
 
-int set_scan(struct ndis_device *wnd)
+static int set_scan(struct ndis_device *wnd)
 {
 	NDIS_STATUS res;
 
@@ -1201,54 +1236,62 @@ static int iw_get_scan(struct net_device *dev, struct iw_request_info *info,
 		       union iwreq_data *wrqu, char *extra)
 {
 	struct ndis_device *wnd = netdev_priv(dev);
-	unsigned int i, list_len, needed;
+	unsigned int i, buf_len, needed, data_len;
 	NDIS_STATUS res;
 	struct ndis_bssid_list *bssid_list = NULL;
 	char *event = extra;
-	struct ndis_wlan_bssid *cur_item ;
+	struct ndis_wlan_bssid *cur_item;
 
 	ENTER2("");
 	if (time_before(jiffies, wnd->scan_timestamp + 3 * HZ))
 		return -EAGAIN;
 	/* try with space for a few scan items */
-	list_len = sizeof(ULONG) + sizeof(struct ndis_wlan_bssid_ex) * 8;
-	bssid_list = kmalloc(list_len, GFP_KERNEL);
-	if (!bssid_list) {
-		ERROR("couldn't allocate memory");
-		return -ENOMEM;
-	}
-	/* some drivers don't set bssid_list->num_items to 0 if
-	   OID_802_11_BSSID_LIST returns no items (prism54 driver, e.g.,) */
-	memset(bssid_list, 0, list_len);
+	buf_len = sizeof(ULONG) + offsetof(struct ndis_wlan_bssid_ex, var) * 8;
 
-	needed = 0;
-	res = mp_query_info(wnd, OID_802_11_BSSID_LIST,
-			    bssid_list, list_len, NULL, &needed);
-	if (res == NDIS_STATUS_INVALID_LENGTH ||
-	    res == NDIS_STATUS_BUFFER_TOO_SHORT) {
-		/* now try with required space */
-		kfree(bssid_list);
-		list_len = needed;
-		bssid_list = kmalloc(list_len, GFP_KERNEL);
+	/* Try many times, as the needed space may grow between queries */
+	for (i = 0; i < 10; i++) {
+		bssid_list = kzalloc(buf_len, GFP_KERNEL);
 		if (!bssid_list) {
-			ERROR("couldn't allocate memory");
+			ERROR("couldn't allocate %u bytes for scan results",
+			      buf_len);
 			return -ENOMEM;
 		}
-		memset(bssid_list, 0, list_len);
 
-		res = mp_query(wnd, OID_802_11_BSSID_LIST,
-			       bssid_list, list_len);
+		needed = 0;
+		data_len = 0;
+		res = mp_query_info(wnd, OID_802_11_BSSID_LIST, bssid_list,
+				    buf_len, &data_len, &needed);
+		TRACE2("try %d: given %d bytes, needed %d, written %d",
+		       i, buf_len, needed, data_len);
+		if (needed <= buf_len)
+			break;
+		kfree(bssid_list);
+		buf_len = needed;
 	}
 	if (res) {
 		WARNING("getting BSSID list failed (%08X)", res);
 		kfree(bssid_list);
 		EXIT2(return -EOPNOTSUPP);
 	}
-	TRACE2("%d", bssid_list->num_items);
+
+	/* some drivers don't set bssid_list->num_items to 0 if
+	   OID_802_11_BSSID_LIST returns no items (prism54 driver, e.g.,) */
+	TRACE2("items: %d", bssid_list->num_items);
 	cur_item = &bssid_list->bssid[0];
 	for (i = 0; i < bssid_list->num_items; i++) {
+		TRACE2("item %d: len %d, remaining data %d",
+		       i, cur_item->length, data_len);
+		/* drop truncated items */
+		if (cur_item->length > data_len)
+			break;
 		event = ndis_translate_scan(dev, info, event,
-					    extra + IW_SCAN_MAX_DATA, cur_item);
+					    extra + wrqu->data.length,
+					    cur_item);
+		if (!event) {
+			kfree(bssid_list);
+			return -E2BIG;
+		}
+		data_len -= cur_item->length;
 		cur_item = (struct ndis_wlan_bssid *)((char *)cur_item +
 						      cur_item->length);
 	}
@@ -1264,7 +1307,7 @@ static int iw_set_power_mode(struct net_device *dev,
 {
 	struct ndis_device *wnd = netdev_priv(dev);
 	NDIS_STATUS res;
-	ULONG power_mode;
+	enum ndis_power power_mode;
 
 	if (wrqu->power.disabled == 1)
 		power_mode = NDIS_POWER_OFF;
@@ -1287,7 +1330,7 @@ static int iw_get_power_mode(struct net_device *dev,
 {
 	struct ndis_device *wnd = netdev_priv(dev);
 	NDIS_STATUS res;
-	ULONG power_mode;
+	enum ndis_power power_mode;
 
 	ENTER2("");
 	res = mp_query(wnd, OID_802_11_POWER_MODE,
@@ -1372,7 +1415,7 @@ static int iw_get_range(struct net_device *dev, struct iw_request_info *info,
 	struct ndis_device *wnd = netdev_priv(dev);
 	unsigned int i, n;
 	NDIS_STATUS res;
-	ndis_rates_ex rates;
+	UCHAR rates[NDIS_MAX_RATES_EX];
 	ndis_tx_power_level tx_power;
 
 	ENTER2("");
@@ -1390,7 +1433,7 @@ static int iw_get_range(struct net_device *dev, struct iw_request_info *info,
 	}
 
 	range->we_version_compiled = WIRELESS_EXT;
-	range->we_version_source = 18;
+	range->we_version_source = 19;
 
 	range->retry_capa = IW_RETRY_LIMIT;
 	range->retry_flags = IW_RETRY_LIMIT;
@@ -1426,10 +1469,9 @@ static int iw_get_range(struct net_device *dev, struct iw_request_info *info,
 			}
 	}
 
-	range->num_channels = (sizeof(freq_chan)/sizeof(freq_chan[0]));
+	range->num_channels = ARRAY_SIZE(freq_chan);
 
-	for (i = 0; i < (sizeof(freq_chan)/sizeof(freq_chan[0])) &&
-		    i < IW_MAX_FREQUENCIES; i++) {
+	for (i = 0; i < ARRAY_SIZE(freq_chan) && i < IW_MAX_FREQUENCIES; i++) {
 		range->freq[i].i = i + 1;
 		range->freq[i].m = freq_chan[i] * 100000;
 		range->freq[i].e = 1;
@@ -1508,29 +1550,6 @@ NDIS_STATUS disassociate(struct ndis_device *wnd, int reset_ssid)
 	return res;
 }
 
-static ULONG ndis_priv_mode(struct ndis_device *wnd)
-{
-	if (wnd->iw_auth_wpa_version & IW_AUTH_WPA_VERSION_WPA2 ||
-	    wnd->iw_auth_wpa_version & IW_AUTH_WPA_VERSION_WPA)
-		return Ndis802_11PrivFilter8021xWEP;
-	else
-		return Ndis802_11PrivFilterAcceptAll;
-}
-
-int set_priv_filter(struct ndis_device *wnd)
-{
-	NDIS_STATUS res;
-	ULONG flags;
-
-	flags = ndis_priv_mode(wnd);
-	ENTER2("filter: %d", flags);
-	res = mp_set_int(wnd, OID_802_11_PRIVACY_FILTER, flags);
-	if (res)
-		TRACE2("setting privacy filter to %d failed (%08X)",
-		       flags, res);
-	EXIT2(return 0);
-}
-
 static int iw_set_mlme(struct net_device *dev, struct iw_request_info *info,
 		       union iwreq_data *wrqu, char *extra)
 {
@@ -1593,9 +1612,9 @@ static int iw_set_auth(struct net_device *dev,
 		break;
 #ifdef IW_AUTH_MFP
 	case IW_AUTH_MFP:
-	        if (wrqu->param.value == IW_AUTH_MFP_DISABLED ||
-                    wrqu->param.value == IW_AUTH_MFP_OPTIONAL)
-		        break;
+		if (wrqu->param.value == IW_AUTH_MFP_DISABLED ||
+		    wrqu->param.value == IW_AUTH_MFP_OPTIONAL)
+			break;
 		WARNING("MFP not implemented");
 		return -EOPNOTSUPP;
 #endif
@@ -1691,9 +1710,9 @@ static int iw_set_encodeext(struct net_device *dev,
 	ndis_key.index = keyidx;
 
 	if (ext->ext_flags & IW_ENCODE_EXT_RX_SEQ_VALID) {
-		for (i = 0; i < 6 ; i++)
+		for (i = 0; i < 6; i++)
 			ndis_key.rsc |= (((u64)ext->rx_seq[i]) << (i * 8));
-		TRACE2("0x%Lx", ndis_key.rsc);
+		TRACE2("0x%llx", ndis_key.rsc);
 		ndis_key.index |= 1 << 29;
 	}
 
@@ -1787,43 +1806,43 @@ static int iw_set_pmksa(struct net_device *dev, struct iw_request_info *info,
 #define WEXT(id) [id - SIOCIWFIRST]
 
 static const iw_handler	ndis_handler[] = {
-	WEXT(SIOCGIWNAME)	 = iw_get_network_type,
-	WEXT(SIOCSIWESSID)	 = iw_set_essid,
-	WEXT(SIOCGIWESSID)	 = iw_get_essid,
-	WEXT(SIOCSIWMODE)	 = iw_set_infra_mode,
-	WEXT(SIOCGIWMODE)	 = iw_get_infra_mode,
-	WEXT(SIOCGIWFREQ)	 = iw_get_freq,
-	WEXT(SIOCSIWFREQ)	 = iw_set_freq,
-	WEXT(SIOCGIWTXPOW)	 = iw_get_tx_power,
-	WEXT(SIOCSIWTXPOW)	 = iw_set_tx_power,
-	WEXT(SIOCGIWRATE)	 = iw_get_bitrate,
-	WEXT(SIOCSIWRATE)	 = iw_set_bitrate,
-	WEXT(SIOCGIWRTS)	 = iw_get_rts_threshold,
-	WEXT(SIOCSIWRTS)	 = iw_set_rts_threshold,
-	WEXT(SIOCGIWFRAG)	 = iw_get_frag_threshold,
-	WEXT(SIOCSIWFRAG)	 = iw_set_frag_threshold,
-	WEXT(SIOCGIWAP)		 = iw_get_ap_address,
-	WEXT(SIOCSIWAP)		 = iw_set_ap_address,
-	WEXT(SIOCSIWENCODE)	 = iw_set_wep,
-	WEXT(SIOCGIWENCODE)	 = iw_get_encr,
-	WEXT(SIOCSIWSCAN)	 = iw_set_scan,
-	WEXT(SIOCGIWSCAN)	 = iw_get_scan,
-	WEXT(SIOCGIWPOWER)	 = iw_get_power_mode,
-	WEXT(SIOCSIWPOWER)	 = iw_set_power_mode,
-	WEXT(SIOCGIWRANGE)	 = iw_get_range,
-	WEXT(SIOCGIWSTATS)	 = iw_get_ndis_stats,
-	WEXT(SIOCGIWSENS)	 = iw_get_sensitivity,
-	WEXT(SIOCSIWSENS)	 = iw_set_sensitivity,
-	WEXT(SIOCGIWNICKN)	 = iw_get_nick,
-	WEXT(SIOCSIWNICKN)	 = iw_set_nick,
-	WEXT(SIOCSIWCOMMIT)	 = iw_set_dummy,
-	WEXT(SIOCSIWMLME)	 = iw_set_mlme,
-	WEXT(SIOCSIWGENIE)	 = iw_set_genie,
-	WEXT(SIOCSIWAUTH)	 = iw_set_auth,
-	WEXT(SIOCGIWAUTH)	 = iw_get_auth,
-	WEXT(SIOCSIWENCODEEXT)	 = iw_set_encodeext,
-	WEXT(SIOCGIWENCODEEXT)	 = iw_get_encodeext,
-	WEXT(SIOCSIWPMKSA)	 = iw_set_pmksa,
+	WEXT(SIOCGIWNAME)	= iw_get_network_type,
+	WEXT(SIOCSIWESSID)	= iw_set_essid,
+	WEXT(SIOCGIWESSID)	= iw_get_essid,
+	WEXT(SIOCSIWMODE)	= iw_set_infra_mode,
+	WEXT(SIOCGIWMODE)	= iw_get_infra_mode,
+	WEXT(SIOCGIWFREQ)	= iw_get_freq,
+	WEXT(SIOCSIWFREQ)	= iw_set_freq,
+	WEXT(SIOCGIWTXPOW)	= iw_get_tx_power,
+	WEXT(SIOCSIWTXPOW)	= iw_set_tx_power,
+	WEXT(SIOCGIWRATE)	= iw_get_bitrate,
+	WEXT(SIOCSIWRATE)	= iw_set_bitrate,
+	WEXT(SIOCGIWRTS)	= iw_get_rts_threshold,
+	WEXT(SIOCSIWRTS)	= iw_set_rts_threshold,
+	WEXT(SIOCGIWFRAG)	= iw_get_frag_threshold,
+	WEXT(SIOCSIWFRAG)	= iw_set_frag_threshold,
+	WEXT(SIOCGIWAP)		= iw_get_ap_address,
+	WEXT(SIOCSIWAP)		= iw_set_ap_address,
+	WEXT(SIOCSIWENCODE)	= iw_set_wep,
+	WEXT(SIOCGIWENCODE)	= iw_get_encr,
+	WEXT(SIOCSIWSCAN)	= iw_set_scan,
+	WEXT(SIOCGIWSCAN)	= iw_get_scan,
+	WEXT(SIOCGIWPOWER)	= iw_get_power_mode,
+	WEXT(SIOCSIWPOWER)	= iw_set_power_mode,
+	WEXT(SIOCGIWRANGE)	= iw_get_range,
+	WEXT(SIOCGIWSTATS)	= iw_get_ndis_stats,
+	WEXT(SIOCGIWSENS)	= iw_get_sensitivity,
+	WEXT(SIOCSIWSENS)	= iw_set_sensitivity,
+	WEXT(SIOCGIWNICKN)	= iw_get_nick,
+	WEXT(SIOCSIWNICKN)	= iw_set_nick,
+	WEXT(SIOCSIWCOMMIT)	= iw_set_dummy,
+	WEXT(SIOCSIWMLME)	= iw_set_mlme,
+	WEXT(SIOCSIWGENIE)	= iw_set_genie,
+	WEXT(SIOCSIWAUTH)	= iw_set_auth,
+	WEXT(SIOCGIWAUTH)	= iw_get_auth,
+	WEXT(SIOCSIWENCODEEXT)	= iw_set_encodeext,
+	WEXT(SIOCGIWENCODEEXT)	= iw_get_encodeext,
+	WEXT(SIOCSIWPMKSA)	= iw_set_pmksa,
 };
 
 /* private ioctl's */
@@ -1961,21 +1980,23 @@ static const struct iw_priv_args priv_args[] = {
 #define WEPRIV(id) [id - SIOCIWFIRSTPRIV]
 
 static const iw_handler priv_handler[] = {
-	WEPRIV(PRIV_RESET)		 = priv_reset,
-	WEPRIV(PRIV_POWER_PROFILE)	 = priv_power_profile,
-	WEPRIV(PRIV_DEAUTHENTICATE)	 = priv_deauthenticate,
-	WEPRIV(PRIV_NETWORK_TYPE)	 = priv_network_type,
-	WEPRIV(PRIV_MEDIA_STREAM_MODE)	 = priv_media_stream_mode,
-	WEPRIV(PRIV_RELOAD_DEFAULTS)	 = priv_reload_defaults,
+	WEPRIV(PRIV_RESET)		= priv_reset,
+	WEPRIV(PRIV_POWER_PROFILE)	= priv_power_profile,
+	WEPRIV(PRIV_DEAUTHENTICATE)	= priv_deauthenticate,
+	WEPRIV(PRIV_NETWORK_TYPE)	= priv_network_type,
+	WEPRIV(PRIV_MEDIA_STREAM_MODE)	= priv_media_stream_mode,
+	WEPRIV(PRIV_RELOAD_DEFAULTS)	= priv_reload_defaults,
 };
 
 const struct iw_handler_def ndis_handler_def = {
-	.num_standard	= sizeof(ndis_handler) / sizeof(ndis_handler[0]),
-	.num_private	= sizeof(priv_handler) / sizeof(priv_handler[0]),
-	.num_private_args = sizeof(priv_args) / sizeof(priv_args[0]),
+	.num_standard	= ARRAY_SIZE(ndis_handler),
+	.num_private	= ARRAY_SIZE(priv_handler),
+	.num_private_args = ARRAY_SIZE(priv_args),
 
 	.standard	= (iw_handler *)ndis_handler,
 	.private	= (iw_handler *)priv_handler,
 	.private_args	= (struct iw_priv_args *)priv_args,
 	.get_wireless_stats = get_iw_stats,
 };
+
+#endif
