@@ -33,17 +33,17 @@
 /* the indices used here must match macros WRAP_NDIS_DEVICE etc. */
 static struct guid class_guids[] = {
 	/* Network */
-	{0x4d36e972, 0xe325, 0x11ce, },
+	{ .data1 = 0x4d36e972, .data2 = 0xe325, .data3 = 0x11ce },
 	/* USB WDM */
-	{0x36fc9e60, 0xc465, 0x11cf, },
+	{ .data1 = 0x36fc9e60, .data2 = 0xc465, .data3 = 0x11cf },
 	/* Bluetooth */
-	{0xe0cbf06c, 0xcd8b, 0x4647, },
-	/* ivtcorporatino.com's bluetooth device claims this is
+	{ .data1 = 0xe0cbf06c, .data2 = 0xcd8b, .data3 = 0x4647 },
+	/* ivtcorporation.com's bluetooth device claims this is
 	 * bluetooth guid */
-	{0xf12d3cf8, 0xb11d, 0x457e, },
+	{ .data1 = 0xf12d3cf8, .data2 = 0xb11d, .data3 = 0x457e},
 };
 
-struct semaphore loader_mutex;
+struct mutex loader_mutex;
 static struct completion loader_complete;
 
 static struct nt_list wrap_devices;
@@ -52,7 +52,7 @@ static struct nt_list wrap_drivers;
 static int wrap_device_type(int data1)
 {
 	int i;
-	for (i = 0; i < sizeof(class_guids) / sizeof(class_guids[0]); i++)
+	for (i = 0; i < ARRAY_SIZE(class_guids); i++)
 		if (data1 == class_guids[i].data1)
 			return i;
 	ERROR("unknown device: 0x%x\n", data1);
@@ -68,10 +68,7 @@ struct wrap_driver *load_wrap_driver(struct wrap_device *wd)
 
 	ENTER1("device: %04X:%04X:%04X:%04X", wd->vendor, wd->device,
 	       wd->subvendor, wd->subdevice);
-	if (down_interruptible(&loader_mutex)) {
-		WARNING("couldn't obtain loader_mutex");
-		EXIT1(return NULL);
-	}
+	mutex_lock(&loader_mutex);
 	wrap_driver = NULL;
 	nt_list_for_each(cur, &wrap_drivers) {
 		wrap_driver = container_of(cur, struct wrap_driver, list);
@@ -81,11 +78,11 @@ struct wrap_driver *load_wrap_driver(struct wrap_device *wd)
 		} else
 			wrap_driver = NULL;
 	}
-	up(&loader_mutex);
+	mutex_unlock(&loader_mutex);
 
 	if (!wrap_driver) {
 		char *argv[] = {"loadndisdriver", WRAP_CMD_LOAD_DRIVER,
-#if defined(DEBUG) && DEBUG >= 1
+#if DEBUG >= 1
 				"1",
 #else
 				"0",
@@ -95,14 +92,11 @@ struct wrap_driver *load_wrap_driver(struct wrap_device *wd)
 		char *env[] = {NULL};
 
 		TRACE1("loading driver %s", wd->driver_name);
-		if (down_interruptible(&loader_mutex)) {
-			WARNING("couldn't obtain loader_mutex");
-			EXIT1(return NULL);
-		}
+		mutex_lock(&loader_mutex);
 		INIT_COMPLETION(loader_complete);
 		ret = call_usermodehelper("/sbin/loadndisdriver", argv, env, 1);
 		if (ret) {
-			up(&loader_mutex);
+			mutex_unlock(&loader_mutex);
 			ERROR("couldn't load driver %s; check system log "
 			      "for messages from 'loadndisdriver'",
 			      wd->driver_name);
@@ -120,7 +114,7 @@ struct wrap_driver *load_wrap_driver(struct wrap_device *wd)
 			} else
 				wrap_driver = NULL;
 		}
-		up(&loader_mutex);
+		mutex_unlock(&loader_mutex);
 		if (wrap_driver)
 			TRACE1("driver %s is loaded", wrap_driver->name);
 		else
@@ -149,8 +143,7 @@ static int load_sys_files(struct wrap_driver *driver,
 		strncpy(pe_image->name, load_driver->sys_files[i].name,
 			sizeof(pe_image->name));
 		pe_image->name[sizeof(pe_image->name)-1] = 0;
-		TRACE1("image size: %lu bytes",
-		       (unsigned long)load_driver->sys_files[i].size);
+		TRACE1("image size: %zu bytes", load_driver->sys_files[i].size);
 
 #ifdef CONFIG_X86_64
 #ifdef PAGE_KERNEL_EXECUTABLE
@@ -223,10 +216,7 @@ struct wrap_bin_file *get_bin_file(char *bin_file_name)
 	struct wrap_driver *driver, *cur;
 
 	ENTER1("%s", bin_file_name);
-	if (down_interruptible(&loader_mutex)) {
-		WARNING("couldn't obtain loader_mutex");
-		EXIT1(return NULL);
-	}
+	mutex_lock(&loader_mutex);
 	driver = NULL;
 	nt_list_for_each_entry(cur, &wrap_drivers, list) {
 		for (i = 0; i < cur->num_bin_files; i++)
@@ -237,15 +227,15 @@ struct wrap_bin_file *get_bin_file(char *bin_file_name)
 		if (driver)
 			break;
 	}
-	up(&loader_mutex);
+	mutex_unlock(&loader_mutex);
 	if (!driver) {
-		TRACE1("coudln't find bin file '%s'", bin_file_name);
+		TRACE1("couldn't find bin file '%s'", bin_file_name);
 		return NULL;
 	}
 
 	if (!driver->bin_files[i].data) {
 		char *argv[] = {"loadndisdriver", WRAP_CMD_LOAD_BIN_FILE,
-#if defined(DEBUG) && DEBUG >= 1
+#if DEBUG >= 1
 				"1",
 #else
 				"0",
@@ -257,21 +247,18 @@ struct wrap_bin_file *get_bin_file(char *bin_file_name)
 
 		TRACE1("loading bin file %s/%s", driver->name,
 		       driver->bin_files[i].name);
-		if (down_interruptible(&loader_mutex)) {
-			WARNING("couldn't obtain loader_mutex");
-			EXIT1(return NULL);
-		}
+		mutex_lock(&loader_mutex);
 		INIT_COMPLETION(loader_complete);
 		ret = call_usermodehelper("/sbin/loadndisdriver", argv, env, 1);
 		if (ret) {
-			up(&loader_mutex);
+			mutex_unlock(&loader_mutex);
 			ERROR("couldn't load file %s/%s; check system log "
 			      "for messages from 'loadndisdriver' (%d)",
 			      driver->name, driver->bin_files[i].name, ret);
 			EXIT1(return NULL);
 		}
 		wait_for_completion(&loader_complete);
-		up(&loader_mutex);
+		mutex_unlock(&loader_mutex);
 		if (!driver->bin_files[i].data) {
 			WARNING("couldn't load binary file %s",
 				driver->bin_files[i].name);
@@ -360,7 +347,7 @@ static int load_bin_files_info(struct wrap_driver *driver,
 	EXIT1(return 0);
 }
 
-/* load settnigs for a device. called with loader_mutex down */
+/* load settings for a device. called with loader_mutex down */
 static int load_settings(struct wrap_driver *wrap_driver,
 			 struct load_driver *load_driver)
 {
@@ -416,15 +403,14 @@ void unload_wrap_device(struct wrap_device *wd)
 	ENTER1("unloading device %p (%04X:%04X:%04X:%04X), driver %s", wd,
 	       wd->vendor, wd->device, wd->subvendor, wd->subdevice,
 	       wd->driver_name);
-	if (down_interruptible(&loader_mutex))
-		WARNING("couldn't obtain loader_mutex");
+	mutex_lock(&loader_mutex);
 	while ((cur = RemoveHeadList(&wd->settings))) {
 		struct wrap_device_setting *setting;
 		setting = container_of(cur, struct wrap_device_setting, list);
 		kfree(setting);
 	}
 	RemoveEntryList(&wd->list);
-	up(&loader_mutex);
+	mutex_unlock(&loader_mutex);
 	kfree(wd);
 	EXIT1(return);
 }
@@ -452,8 +438,7 @@ void unload_wrap_driver(struct wrap_driver *driver)
 		if (driver->bin_files[i].data)
 			vfree(driver->bin_files[i].data);
 	}
-	if (driver->bin_files)
-		kfree(driver->bin_files);
+	kfree(driver->bin_files);
 	RtlFreeUnicodeString(&drv_obj->name);
 	RemoveEntryList(&driver->list);
 	nt_list_for_each_safe(cur, next, &driver->settings) {
@@ -566,7 +551,6 @@ static int load_user_space_driver(struct load_driver *load_driver)
 	memset(wrap_driver, 0, sizeof(*wrap_driver));
 	InitializeListHead(&wrap_driver->list);
 	InitializeListHead(&wrap_driver->settings);
-	InitializeListHead(&wrap_driver->wrap_devices);
 	wrap_driver->drv_obj = drv_obj;
 	RtlInitAnsiString(&ansi_reg, "/tmp");
 	if (RtlAnsiStringToUnicodeString(&drv_obj->name, &ansi_reg, TRUE) !=
@@ -586,6 +570,7 @@ static int load_user_space_driver(struct load_driver *load_driver)
 	    start_wrap_driver(wrap_driver) ||
 	    add_wrap_driver(wrap_driver)) {
 		unload_wrap_driver(wrap_driver);
+		ObDereferenceObject(drv_obj);
 		EXIT1(return -EINVAL);
 	} else {
 		printk(KERN_INFO "%s: driver %s (%s) loaded\n",
@@ -596,7 +581,15 @@ static int load_user_space_driver(struct load_driver *load_driver)
 }
 
 static struct pci_device_id wrap_pci_id_table[] = {
-	{PCI_ANY_ID, PCI_ANY_ID, PCI_ANY_ID, PCI_ANY_ID},
+	{
+		.vendor = PCI_ANY_ID,
+		.device = PCI_ANY_ID,
+		.subvendor = PCI_ANY_ID,
+		.subdevice = PCI_ANY_ID,
+		.class = 0,
+		.class_mask = 0,
+		.driver_data = 0
+	}
 };
 
 static struct pci_driver wrap_pci_driver = {
@@ -650,14 +643,13 @@ static void unregister_devices(void)
 {
 	struct nt_list *cur, *next;
 
-	if (down_interruptible(&loader_mutex))
-		WARNING("couldn't obtain loader_mutex");
+	mutex_lock(&loader_mutex);
 	nt_list_for_each_safe(cur, next, &wrap_devices) {
 		struct wrap_device *wd;
 		wd = container_of(cur, struct wrap_device, list);
-		set_bit(HW_PRESENT, &wd->hw_status);
+		set_bit(HW_DISABLED, &wd->hw_status);
 	}
-	up(&loader_mutex);
+	mutex_unlock(&loader_mutex);
 
 	if (wrap_pci_driver.name)
 		pci_unregister_driver(&wrap_pci_driver);
@@ -683,7 +675,7 @@ struct wrap_device *load_wrap_device(struct load_device *load_device)
 	    sprintf(subdevice, "%04x", load_device->subdevice) == 4 &&
 	    sprintf(bus, "%04x", load_device->bus) == 4) {
 		char *argv[] = {"loadndisdriver", WRAP_CMD_LOAD_DEVICE,
-#if defined(DEBUG) && DEBUG >= 1
+#if DEBUG >= 1
 				"1",
 #else
 				"0",
@@ -693,14 +685,11 @@ struct wrap_device *load_wrap_device(struct load_device *load_device)
 		char *env[] = {NULL};
 		TRACE2("%s, %s, %s, %s, %s", vendor, device,
 		       subvendor, subdevice, bus);
-		if (down_interruptible(&loader_mutex)) {
-			WARNING("couldn't obtain loader_mutex");
-			EXIT1(return NULL);
-		}
+		mutex_lock(&loader_mutex);
 		INIT_COMPLETION(loader_complete);
 		ret = call_usermodehelper("/sbin/loadndisdriver", argv, env, 1);
 		if (ret) {
-			up(&loader_mutex);
+			mutex_unlock(&loader_mutex);
 			TRACE1("couldn't load device %04x:%04x; check system "
 			       "log for messages from 'loadndisdriver'",
 			       load_device->vendor, load_device->device);
@@ -718,7 +707,7 @@ struct wrap_device *load_wrap_device(struct load_device *load_device)
 			else
 				wd = NULL;
 		}
-		up(&loader_mutex);
+		mutex_unlock(&loader_mutex);
 	} else
 		wd = NULL;
 	EXIT1(return wd);
@@ -729,10 +718,7 @@ struct wrap_device *get_wrap_device(void *dev, int bus)
 	struct nt_list *cur;
 	struct wrap_device *wd;
 
-	if (down_interruptible(&loader_mutex)) {
-		WARNING("couldn't obtain loader_mutex");
-		return NULL;
-	}
+	mutex_lock(&loader_mutex);
 	wd = NULL;
 	nt_list_for_each(cur, &wrap_devices) {
 		wd = container_of(cur, struct wrap_device, list);
@@ -745,7 +731,7 @@ struct wrap_device *get_wrap_device(void *dev, int bus)
 		else
 			wd = NULL;
 	}
-	up(&loader_mutex);
+	mutex_unlock(&loader_mutex);
 	return wd;
 }
 
@@ -835,8 +821,7 @@ static int wrapper_ioctl_release(struct inode *inode, struct file *file)
 }
 
 static struct file_operations wrapper_fops = {
-	.owner          = THIS_MODULE,
-
+	.owner		= THIS_MODULE,
 #ifdef HAVE_UNLOCKED_IOCTL
 	.unlocked_ioctl	= wrapper_ioctl,
 #else
@@ -846,9 +831,9 @@ static struct file_operations wrapper_fops = {
 };
 
 static struct miscdevice wrapper_misc = {
-	.name   = DRIVER_NAME,
+	.name	= DRIVER_NAME,
 	.minor	= MISC_DYNAMIC_MINOR,
-	.fops   = &wrapper_fops
+	.fops	= &wrapper_fops
 };
 
 int loader_init(void)
@@ -857,9 +842,9 @@ int loader_init(void)
 
 	InitializeListHead(&wrap_drivers);
 	InitializeListHead(&wrap_devices);
-	sema_init(&loader_mutex, 1);
+	mutex_init(&loader_mutex);
 	init_completion(&loader_complete);
-	if ((err = misc_register(&wrapper_misc)) < 0 ) {
+	if ((err = misc_register(&wrapper_misc)) < 0) {
 		ERROR("couldn't register module (%d)", err);
 		unregister_devices();
 		EXIT1(return err);
@@ -875,13 +860,12 @@ void loader_exit(void)
 	ENTER1("");
 	misc_deregister(&wrapper_misc);
 	unregister_devices();
-	if (down_interruptible(&loader_mutex))
-		WARNING("couldn't obtain loader_mutex");
+	mutex_lock(&loader_mutex);
 	nt_list_for_each_safe(cur, next, &wrap_drivers) {
 		struct wrap_driver *driver;
 		driver = container_of(cur, struct wrap_driver, list);
 		unload_wrap_driver(driver);
 	}
-	up(&loader_mutex);
+	mutex_unlock(&loader_mutex);
 	EXIT1(return);
 }
