@@ -18,11 +18,15 @@
 
 #include "ndiswrapper.h"
 
+#ifndef __KERNEL__
+#define __user
+#endif
+
 struct load_driver_file {
 	char driver_name[MAX_DRIVER_NAME_LEN];
 	char name[MAX_DRIVER_NAME_LEN];
 	size_t size;
-	void *data;
+	void __user *data;
 };
 
 struct load_device_setting {
@@ -48,11 +52,11 @@ struct load_devices {
 struct load_driver {
 	char name[MAX_DRIVER_NAME_LEN];
 	char conf_file_name[MAX_DRIVER_NAME_LEN];
-	unsigned int nr_sys_files;
+	unsigned int num_sys_files;
 	struct load_driver_file sys_files[MAX_DRIVER_PE_IMAGES];
-	unsigned int nr_settings;
+	unsigned int num_settings;
 	struct load_device_setting settings[MAX_DEVICE_SETTINGS];
-	unsigned int nr_bin_files;
+	unsigned int num_bin_files;
 	struct load_driver_file bin_files[MAX_DRIVER_BIN_FILES];
 };
 
@@ -78,7 +82,8 @@ void free_bin_file(struct wrap_bin_file *bin_file);
 void unload_wrap_driver(struct wrap_driver *driver);
 void unload_wrap_device(struct wrap_device *wd);
 struct wrap_device *get_wrap_device(void *dev, int bus_type);
+
+extern struct mutex loader_mutex;
 #endif
 
 #endif /* LOADER_H */
-
