@@ -25,13 +25,11 @@
 #define MAX_ALLOCATED_NDIS_PACKETS TX_RING_SIZE
 #define MAX_ALLOCATED_NDIS_BUFFERS TX_RING_SIZE
 
-static void ndis_worker(worker_param_t dummy);
-static work_struct_t ndis_work;
+static struct work_struct ndis_work;
 static struct nt_list ndis_work_list;
 static spinlock_t ndis_work_list_lock;
 
-workqueue_struct_t *ndis_wq;
-static struct nt_thread *ndis_worker_thread;
+struct workqueue_struct *ndis_wq;
 
 static void *ndis_get_routine_address(char *name);
 
@@ -101,8 +99,8 @@ wstdcall NDIS_STATUS WIN_FUNC(NdisMRegisterMiniport,3)
 			"pnp_event_notify", "shutdown",
 		};
 		func = (void **)&ndis_driver->mp.queryinfo;
-		for (i = 0; i < (sizeof(mp_funcs) / sizeof(mp_funcs[0])); i++)
-			TRACE2("function '%s' is at %p", mp_funcs[i], func[i]);
+		for (i = 0; i < ARRAY_SIZE(mp_funcs); i++)
+			TRACE0("function '%s' is at %p", mp_funcs[i], func[i]);
 	}
 	EXIT1(return NDIS_STATUS_SUCCESS);
 }
@@ -359,14 +357,14 @@ ndis_encode_setting(struct wrap_device_setting *setting,
 		ERROR("couldn't allocate memory");
 		return NULL;
 	}
-	switch(type) {
+	switch (type) {
 	case NdisParameterInteger:
 		param->data.integer = simple_strtol(setting->value, NULL, 0);
-		TRACE2("0x%x", (ULONG)param->data.integer);
+		TRACE2("0x%x", param->data.integer);
 		break;
 	case NdisParameterHexInteger:
 		param->data.integer = simple_strtol(setting->value, NULL, 16);
-		TRACE2("0x%x", (ULONG)param->data.integer);
+		TRACE2("0x%x", param->data.integer);
 		break;
 	case NdisParameterString:
 		RtlInitAnsiString(&ansi, setting->value);
@@ -379,7 +377,7 @@ ndis_encode_setting(struct wrap_device_setting *setting,
 		break;
 	case NdisParameterBinary:
 		param->data.integer = simple_strtol(setting->value, NULL, 2);
-		TRACE2("0x%x", (ULONG)param->data.integer);
+		TRACE2("0x%x", param->data.integer);
 		break;
 	default:
 		ERROR("unknown type: %d", type);
@@ -403,16 +401,14 @@ static int ndis_decode_setting(struct wrap_device_setting *setting,
 		RtlFreeUnicodeString(&prev->data.string);
 		setting->encoded = NULL;
 	}
-	switch(param->type) {
+	switch (param->type) {
 	case NdisParameterInteger:
-		snprintf(setting->value, sizeof(u32), "%u",
+		snprintf(setting->value, MAX_SETTING_VALUE_LEN, "%u",
 			 param->data.integer);
-		setting->value[sizeof(ULONG)] = 0;
 		break;
 	case NdisParameterHexInteger:
-		snprintf(setting->value, sizeof(u32), "%x",
+		snprintf(setting->value, MAX_SETTING_VALUE_LEN, "%x",
 			 param->data.integer);
-		setting->value[sizeof(ULONG)] = 0;
 		break;
 	case NdisParameterString:
 		ansi.buf = setting->value;
@@ -426,11 +422,6 @@ static int ndis_decode_setting(struct wrap_device_setting *setting,
 			ansi.length--;
 		setting->value[ansi.length] = 0;
 		break;
-	case NdisParameterBinary:
-		snprintf(setting->value, sizeof(u32), "%u",
-			 param->data.integer);
-		setting->value[sizeof(ULONG)] = 0;
-		break;
 	default:
 		TRACE2("unknown setting type: %d", param->type);
 		return -1;
@@ -445,12 +436,11 @@ static int read_setting(struct nt_list *setting_list, char *keyname, int length,
 			enum ndis_parameter_type type)
 {
 	struct wrap_device_setting *setting;
-	if (down_interruptible(&loader_mutex))
-		WARNING("couldn't obtain loader_mutex");
+	mutex_lock(&loader_mutex);
 	nt_list_for_each_entry(setting, setting_list, list) {
 		if (strnicmp(keyname, setting->name, length) == 0) {
 			TRACE2("setting %s='%s'", keyname, setting->value);
-			up(&loader_mutex);
+			mutex_unlock(&loader_mutex);
 			*param = ndis_encode_setting(setting, type);
 			if (*param)
 				EXIT2(return 0);
@@ -458,7 +448,7 @@ static int read_setting(struct nt_list *setting_list, char *keyname, int length,
 				EXIT2(return -1);
 		}
 	}
-	up(&loader_mutex);
+	mutex_unlock(&loader_mutex);
 	EXIT2(return -1);
 }
 
@@ -510,11 +500,10 @@ wstdcall void WIN_FUNC(NdisWriteConfiguration,4)
 	keyname = ansi.buf;
 	TRACE2("%s", keyname);
 
-	if (down_interruptible(&loader_mutex))
-		WARNING("couldn't obtain loader_mutex");
+	mutex_lock(&loader_mutex);
 	nt_list_for_each_entry(setting, &nmb->wnd->wd->settings, list) {
 		if (strnicmp(keyname, setting->name, ansi.length) == 0) {
-			up(&loader_mutex);
+			mutex_unlock(&loader_mutex);
 			if (ndis_decode_setting(setting, param))
 				*status = NDIS_STATUS_FAILURE;
 			else
@@ -523,7 +512,7 @@ wstdcall void WIN_FUNC(NdisWriteConfiguration,4)
 			EXIT2(return);
 		}
 	}
-	up(&loader_mutex);
+	mutex_unlock(&loader_mutex);
 	setting = kzalloc(sizeof(*setting), GFP_KERNEL);
 	if (setting) {
 		if (ansi.length == ansi.max_length)
@@ -534,10 +523,9 @@ wstdcall void WIN_FUNC(NdisWriteConfiguration,4)
 			*status = NDIS_STATUS_FAILURE;
 		else {
 			*status = NDIS_STATUS_SUCCESS;
-			if (down_interruptible(&loader_mutex))
-				WARNING("couldn't obtain loader_mutex");
+			mutex_lock(&loader_mutex);
 			InsertTailList(&nmb->wnd->wd->settings, &setting->list);
-			up(&loader_mutex);
+			mutex_unlock(&loader_mutex);
 		}
 	} else
 		*status = NDIS_STATUS_RESOURCES;
@@ -687,6 +675,10 @@ wstdcall ULONG WIN_FUNC(NdisReadPciSlotInformation,5)
 {
 	struct wrap_device *wd = nmb->wnd->wd;
 	ULONG i;
+	if (!wrap_is_pci_bus(wd->dev_bus)) {
+		ERROR("used on a non-PCI device");
+		return 0;
+	}
 	for (i = 0; i < len; i++)
 		if (pci_read_config_byte(wd->pci.pdev, offset + i, &buf[i]) !=
 		    PCIBIOS_SUCCESSFUL)
@@ -711,6 +703,10 @@ wstdcall ULONG WIN_FUNC(NdisWritePciSlotInformation,5)
 {
 	struct wrap_device *wd = nmb->wnd->wd;
 	ULONG i;
+	if (!wrap_is_pci_bus(wd->dev_bus)) {
+		ERROR("used on a non-PCI device");
+		return 0;
+	}
 	for (i = 0; i < len; i++)
 		if (pci_write_config_byte(wd->pci.pdev, offset + i, buf[i]) !=
 		    PCIBIOS_SUCCESSFUL)
@@ -815,7 +811,7 @@ wstdcall NDIS_STATUS WIN_FUNC(NdisMMapIoSpace,4)
 {
 	struct ndis_device *wnd = nmb->wnd;
 
-	ENTER2("%Lx, %d", phy_addr, len);
+	ENTER2("%llx, %d", phy_addr, len);
 	*virt = MmMapIoSpace(phy_addr, len, MmCached);
 	if (*virt == NULL) {
 		ERROR("ioremap failed");
@@ -896,7 +892,7 @@ wstdcall void WIN_FUNC(NdisInitializeReadWriteLock,1)
 	return;
 }
 
-/* read/write locks are implemented in a rather simplisitic way - we
+/* read/write locks are implemented in a rather simplistic way - we
  * should probably use Linux's rw_lock implementation */
 
 wstdcall void WIN_FUNC(NdisAcquireReadWriteLock,3)
@@ -939,29 +935,36 @@ wstdcall NDIS_STATUS WIN_FUNC(NdisMAllocateMapRegisters,5)
 	struct ndis_device *wnd = nmb->wnd;
 
 	ENTER2("%p, %d %d %d %d", wnd, dmachan, dmasize, basemap, max_buf_size);
+	if (!wrap_is_pci_bus(wnd->wd->dev_bus)) {
+		ERROR("used on a non-PCI device");
+		return NDIS_STATUS_NOT_SUPPORTED;
+	}
 	if (wnd->dma_map_count > 0) {
 		WARNING("%s: map registers already allocated: %u",
 			wnd->net_dev->name, wnd->dma_map_count);
 		EXIT2(return NDIS_STATUS_RESOURCES);
 	}
 	if (dmasize == NDIS_DMA_24BITS) {
-		if (pci_set_dma_mask(wnd->wd->pci.pdev, DMA_24BIT_MASK) ||
+		if (pci_set_dma_mask(wnd->wd->pci.pdev, DMA_BIT_MASK(24)) ||
 		    pci_set_consistent_dma_mask(wnd->wd->pci.pdev,
-						DMA_24BIT_MASK))
+						DMA_BIT_MASK(24)))
 			WARNING("setting dma mask failed");
 	} else if (dmasize == NDIS_DMA_32BITS) {
 		/* consistent dma is in low 32-bits by default */
-		if (pci_set_dma_mask(wnd->wd->pci.pdev, DMA_32BIT_MASK))
+		if (pci_set_dma_mask(wnd->wd->pci.pdev, DMA_BIT_MASK(32)))
 			WARNING("setting dma mask failed");
 #ifdef CONFIG_X86_64
 	} else if (dmasize == NDIS_DMA_64BITS) {
-		if (pci_set_dma_mask(wnd->wd->pci.pdev, DMA_64BIT_MASK) ||
+		if (pci_set_dma_mask(wnd->wd->pci.pdev, DMA_BIT_MASK(64)) ||
 		    pci_set_consistent_dma_mask(wnd->wd->pci.pdev,
-						DMA_64BIT_MASK))
+						DMA_BIT_MASK(64)))
 			WARNING("setting dma mask failed");
 		else
 			wnd->net_dev->features |= NETIF_F_HIGHDMA;
 #endif
+	} else {
+		ERROR("dmasize %d not supported", dmasize);
+		EXIT2(return NDIS_STATUS_NOT_SUPPORTED);
 	}
 	/* since memory for buffer is allocated with kmalloc, buffer
 	 * is physically contiguous, so entire map will fit in one
@@ -975,11 +978,10 @@ wstdcall NDIS_STATUS WIN_FUNC(NdisMAllocateMapRegisters,5)
 //		EXIT2(return NDIS_STATUS_RESOURCES);
 	}
 
-	wnd->dma_map_addr = kmalloc(basemap * sizeof(*(wnd->dma_map_addr)),
+	wnd->dma_map_addr = kzalloc(basemap * sizeof(*(wnd->dma_map_addr)),
 				    GFP_KERNEL);
 	if (!wnd->dma_map_addr)
 		EXIT2(return NDIS_STATUS_RESOURCES);
-	memset(wnd->dma_map_addr, 0, basemap * sizeof(*(wnd->dma_map_addr)));
 	wnd->dma_map_count = basemap;
 	TRACE2("%u", wnd->dma_map_count);
 	EXIT2(return NDIS_STATUS_SUCCESS);
@@ -995,9 +997,9 @@ wstdcall void WIN_FUNC(NdisMFreeMapRegisters,1)
 	if (wnd->dma_map_addr) {
 		for (i = 0; i < wnd->dma_map_count; i++) {
 			if (wnd->dma_map_addr[i])
-				WARNING("%s: dma addr %p not freed by "
+				WARNING("%s: dma addr 0x%llx not freed by "
 					"Windows driver", wnd->net_dev->name,
-					(void *)wnd->dma_map_addr[i]);
+					(unsigned long long)wnd->dma_map_addr[i]);
 		}
 		kfree(wnd->dma_map_addr);
 		wnd->dma_map_addr = NULL;
@@ -1015,6 +1017,10 @@ wstdcall void WIN_FUNC(NdisMStartBufferPhysicalMapping,6)
 	struct ndis_device *wnd = nmb->wnd;
 
 	ENTER3("%p, %p, %u, %u", wnd, buf, index, wnd->dma_map_count);
+	if (!wrap_is_pci_bus(wnd->wd->dev_bus)) {
+		ERROR("used on a non-PCI device");
+		return;
+	}
 	if (unlikely(wnd->sg_dma_size || !write_to_dev ||
 		     index >= wnd->dma_map_count)) {
 		WARNING("invalid request: %d, %d, %d, %d", wnd->sg_dma_size,
@@ -1025,24 +1031,24 @@ wstdcall void WIN_FUNC(NdisMStartBufferPhysicalMapping,6)
 		return;
 	}
 	if (wnd->dma_map_addr[index]) {
-		TRACE2("buffer %p at %d is already mapped: %lx", buf, index,
-		       (unsigned long)wnd->dma_map_addr[index]);
+		TRACE2("buffer %p at %d is already mapped: %llx", buf, index,
+		       (unsigned long long)wnd->dma_map_addr[index]);
 //		*array_size = 1;
 		return;
 	}
 	TRACE3("%p, %p, %u", buf, MmGetSystemAddressForMdl(buf),
 	       MmGetMdlByteCount(buf));
 	DBG_BLOCK(4) {
-		dump_bytes(__FUNCTION__, MmGetSystemAddressForMdl(buf),
+		dump_bytes(__func__, MmGetSystemAddressForMdl(buf),
 			   MmGetMdlByteCount(buf));
 	}
-	wnd->dma_map_addr[index] = 
+	wnd->dma_map_addr[index] =
 		PCI_DMA_MAP_SINGLE(wnd->wd->pci.pdev,
 				   MmGetSystemAddressForMdl(buf),
 				   MmGetMdlByteCount(buf), PCI_DMA_TODEVICE);
 	phy_addr_array[0].phy_addr = wnd->dma_map_addr[index];
 	phy_addr_array[0].length = MmGetMdlByteCount(buf);
-	TRACE4("%Lx, %d, %d", phy_addr_array[0].phy_addr,
+	TRACE4("%llx, %d, %d", phy_addr_array[0].phy_addr,
 	       phy_addr_array[0].length, index);
 	*array_size = 1;
 }
@@ -1054,6 +1060,10 @@ wstdcall void WIN_FUNC(NdisMCompleteBufferPhysicalMapping,3)
 
 	ENTER3("%p, %p %u (%u)", wnd, buf, index, wnd->dma_map_count);
 
+	if (!wrap_is_pci_bus(wnd->wd->dev_bus)) {
+		ERROR("used on a non-PCI device");
+		return;
+	}
 	if (unlikely(wnd->sg_dma_size))
 		WARNING("buffer %p may have been unmapped already", buf);
 	if (index >= wnd->dma_map_count) {
@@ -1061,7 +1071,7 @@ wstdcall void WIN_FUNC(NdisMCompleteBufferPhysicalMapping,3)
 		      index, wnd->dma_map_count);
 		return;
 	}
-	TRACE4("%lx", (unsigned long)wnd->dma_map_addr[index]);
+	TRACE4("%llx", (unsigned long long)wnd->dma_map_addr[index]);
 	if (wnd->dma_map_addr[index]) {
 		PCI_DMA_UNMAP_SINGLE(wnd->wd->pci.pdev, wnd->dma_map_addr[index],
 				     MmGetMdlByteCount(buf), PCI_DMA_TODEVICE);
@@ -1078,6 +1088,10 @@ wstdcall void WIN_FUNC(NdisMAllocateSharedMemory,5)
 	struct wrap_device *wd = nmb->wnd->wd;
 
 	ENTER3("size: %u, cached: %d", size, cached);
+	if (!wrap_is_pci_bus(wd->dev_bus)) {
+		ERROR("used on a non-PCI device");
+		return;
+	}
 	*virt = PCI_DMA_ALLOC_COHERENT(wd->pci.pdev, size, &dma_addr);
 	if (*virt)
 		*phys = dma_addr;
@@ -1092,7 +1106,11 @@ wstdcall void WIN_FUNC(NdisMFreeSharedMemory,5)
 	 void *virt, NDIS_PHY_ADDRESS addr)
 {
 	struct wrap_device *wd = nmb->wnd->wd;
-	ENTER3("%p, %Lx, %u", virt, addr, size);
+	ENTER3("%p, %llx, %u", virt, addr, size);
+	if (!wrap_is_pci_bus(wd->dev_bus)) {
+		ERROR("used on a non-PCI device");
+		return;
+	}
 	PCI_DMA_FREE_COHERENT(wd->pci.pdev, size, virt, addr);
 	EXIT3(return);
 }
@@ -1300,7 +1318,7 @@ wstdcall void WIN_FUNC(NdisQueryBuffer,3)
 	if (virt)
 		*virt = MmGetSystemAddressForMdl(buffer);
 	*length = MmGetMdlByteCount(buffer);
-	TRACE4("%p, %u", virt? *virt : NULL, *length);
+	TRACE4("%p, %u", virt ? *virt : NULL, *length);
 	return;
 }
 
@@ -1312,7 +1330,7 @@ wstdcall void WIN_FUNC(NdisQueryBufferSafe,4)
 	if (virt)
 		*virt = MmGetSystemAddressForMdlSafe(buffer, priority);
 	*length = MmGetMdlByteCount(buffer);
-	TRACE4("%p, %u", virt? *virt : NULL, *length);
+	TRACE4("%p, %u", virt ? *virt : NULL, *length);
 }
 
 wstdcall void *WIN_FUNC(NdisBufferVirtualAddress,1)
@@ -1893,7 +1911,7 @@ wstdcall BOOLEAN ndis_isr(struct kinterrupt *kinterrupt, void *ctx)
 {
 	struct ndis_mp_interrupt *mp_interrupt = ctx;
 	struct ndis_device *wnd = mp_interrupt->nmb->wnd;
-	BOOLEAN recognized, queue_handler;
+	BOOLEAN recognized = TRUE, queue_handler = TRUE;
 
 	TRACE6("%p", wnd);
 	/* kernel may call ISR when registering interrupt, in
@@ -2148,7 +2166,7 @@ wstdcall void WIN_FUNC(NdisMIndicateStatusComplete,1)
 	struct ndis_device *wnd = nmb->wnd;
 	ENTER2("%p", wnd);
 	if (wnd->tx_ok)
-		schedule_wrapndis_work(&wnd->tx_work);
+		queue_work(wrapndis_wq, &wnd->tx_work);
 }
 
 /* called via function pointer */
@@ -2182,7 +2200,7 @@ wstdcall void NdisMSendComplete(struct ndis_mp_block *nmb,
 		 */
 		if (xchg(&wnd->tx_ok, 1) == 0) {
 			TRACE3("%d, %d", wnd->tx_ring_start, wnd->tx_ring_end);
-			schedule_wrapndis_work(&wnd->tx_work);
+			queue_work(wrapndis_wq, &wnd->tx_work);
 		}
 	}
 	EXIT3(return);
@@ -2194,7 +2212,7 @@ wstdcall void NdisMSendResourcesAvailable(struct ndis_mp_block *nmb)
 	struct ndis_device *wnd = nmb->wnd;
 	ENTER3("%d, %d", wnd->tx_ring_start, wnd->tx_ring_end);
 	wnd->tx_ok = 1;
-	schedule_wrapndis_work(&wnd->tx_work);
+	queue_work(wrapndis_wq, &wnd->tx_work);
 	EXIT3(return);
 }
 
@@ -2247,7 +2265,7 @@ wstdcall void NdisMIndicateReceivePacket(struct ndis_mp_block *nmb,
 						 NormalPagePriority);
 		TRACE3("%d, %d", length, total_length);
 		oob_data = NDIS_PACKET_OOB_DATA(packet);
-		TRACE3("0x%x, 0x%x, %Lu", packet->private.flags,
+		TRACE3("0x%x, 0x%x, %llu", packet->private.flags,
 		       packet->private.packet_flags, oob_data->time_rxed);
 		skb = dev_alloc_skb(total_length);
 		if (skb) {
@@ -2557,7 +2575,7 @@ wstdcall void WIN_FUNC(NdisGetCurrentSystemTime,1)
 	(LARGE_INTEGER *time)
 {
 	*time = ticks_1601();
-	TRACE5("%Lu, %lu", *time, jiffies);
+	TRACE5("%llu, %lu", *time, jiffies);
 }
 
 wstdcall LONG WIN_FUNC(NdisInterlockedDecrement,1)
@@ -2593,16 +2611,21 @@ wstdcall struct nt_list *WIN_FUNC(NdisInterlockedRemoveHeadList,2)
 }
 
 wstdcall NDIS_STATUS WIN_FUNC(NdisMInitializeScatterGatherDma,3)
-	(struct ndis_mp_block *nmb, BOOLEAN dma_size, ULONG max_phy_map)
+	(struct ndis_mp_block *nmb, BOOLEAN dma64_supported, ULONG max_phy_map)
 {
 	struct ndis_device *wnd = nmb->wnd;
-	ENTER2("dma_size=%d, maxtransfer=%u", dma_size, max_phy_map);
+	ENTER2("dma64_supported=%d, maxtransfer=%u", dma64_supported,
+	       max_phy_map);
+	if (!wrap_is_pci_bus(wnd->wd->dev_bus)) {
+		ERROR("used on a non-PCI device");
+		return NDIS_STATUS_NOT_SUPPORTED;
+	}
 #ifdef CONFIG_X86_64
-	if (dma_size != NDIS_DMA_64BITS) {
-		TRACE1("DMA size is not 64-bits");
-		if (pci_set_dma_mask(wnd->wd->pci.pdev, DMA_32BIT_MASK) ||
+	if (!dma64_supported) {
+		TRACE1("64-bit DMA size is not supported");
+		if (pci_set_dma_mask(wnd->wd->pci.pdev, DMA_BIT_MASK(32)) ||
 		    pci_set_consistent_dma_mask(wnd->wd->pci.pdev,
-						DMA_32BIT_MASK))
+						DMA_BIT_MASK(32)))
 			WARNING("setting dma mask failed");
 	}
 #endif
@@ -2624,7 +2647,7 @@ wstdcall ULONG WIN_FUNC(NdisMGetDmaAlignment,1)
 wstdcall CHAR WIN_FUNC(NdisSystemProcessorCount,0)
 	(void)
 {
-	return NR_CPUS;
+	return num_online_cpus();
 }
 
 wstdcall void WIN_FUNC(NdisGetCurrentProcessorCounts,3)
@@ -2674,7 +2697,7 @@ wstdcall void WIN_FUNC(NdisResetEvent,1)
 	KeResetEvent(&ndis_event->nt_event);
 }
 
-static void ndis_worker(worker_param_t dummy)
+static void ndis_worker(struct work_struct *dummy)
 {
 	struct nt_list *ent;
 	struct ndis_work_item *ndis_work_item;
@@ -2704,7 +2727,7 @@ wstdcall NDIS_STATUS WIN_FUNC(NdisScheduleWorkItem,1)
 	InsertTailList(&ndis_work_list, &ndis_work_item->list);
 	spin_unlock_bh(&ndis_work_list_lock);
 	WORKTRACE("scheduling %p", ndis_work_item);
-	schedule_ndis_work(&ndis_work);
+	queue_work(ndis_wq, &ndis_work);
 	EXIT3(return NDIS_STATUS_SUCCESS);
 }
 
@@ -2837,7 +2860,7 @@ wstdcall NDIS_STATUS WIN_FUNC(NdisIMNotifiyPnPEvent,2)
 	EXIT2(return NDIS_STATUS_SUCCESS);
 }
 
-wstdcall void WIN_FUNC(NdisCompletePnPEvent,2)
+wstdcall void WIN_FUNC(NdisCompletePnPEvent,3)
 	(NDIS_STATUS status, void *handle, struct net_pnp_event *event)
 {
 	ENTER2("%d, %p, %d", status, handle, event->code);
@@ -2884,7 +2907,7 @@ static void *ndis_get_routine_address(char *name)
 {
 	int i;
 	ENTER2("%p", name);
-	for (i = 0; i < sizeof(ndis_exports) / sizeof(ndis_exports[0]); i++) {
+	for (i = 0; i < ARRAY_SIZE(ndis_exports); i++) {
 		if (strcmp(name, ndis_exports[i].name) == 0) {
 			TRACE2("%p", ndis_exports[i].func);
 			return ndis_exports[i].func;
@@ -2928,8 +2951,7 @@ void ndis_exit_device(struct ndis_device *wnd)
 {
 	struct wrap_device_setting *setting;
 	ENTER2("%p", wnd);
-	if (down_interruptible(&loader_mutex))
-		WARNING("couldn't obtain loader_mutex");
+	mutex_lock(&loader_mutex);
 	nt_list_for_each_entry(setting, &wnd->wd->settings, list) {
 		struct ndis_configuration_parameter *param;
 		param = setting->encoded;
@@ -2940,7 +2962,7 @@ void ndis_exit_device(struct ndis_device *wnd)
 			setting->encoded = NULL;
 		}
 	}
-	up(&loader_mutex);
+	mutex_unlock(&loader_mutex);
 }
 
 /* ndis_init is called once when module is loaded */
@@ -2948,7 +2970,7 @@ int ndis_init(void)
 {
 	InitializeListHead(&ndis_work_list);
 	spin_lock_init(&ndis_work_list_lock);
-	initialize_work(&ndis_work, ndis_worker, NULL);
+	INIT_WORK(&ndis_work, ndis_worker);
 
 	ndis_wq = create_singlethread_workqueue("ndis_wq");
 	if (!ndis_wq) {
@@ -2956,8 +2978,7 @@ int ndis_init(void)
 		EXIT1(return -ENOMEM);
 	}
 
-	ndis_worker_thread = wrap_worker_init(ndis_wq);
-	TRACE1("%p", ndis_worker_thread);
+	TRACE1("ndis_wq: %p", ndis_wq);
 	return 0;
 }
 
@@ -2967,8 +2988,5 @@ void ndis_exit(void)
 	ENTER1("");
 	if (ndis_wq)
 		destroy_workqueue(ndis_wq);
-	TRACE1("%p", ndis_worker_thread);
-	if (ndis_worker_thread)
-		ObDereferenceObject(ndis_worker_thread);
 	EXIT1(return);
 }
