@@ -16,22 +16,18 @@
 #ifndef _NDISWRAPPER_H_
 #define _NDISWRAPPER_H_
 
-#define DRIVER_VERSION "1.54"
+#define DRIVER_VERSION "1.57"
 #define UTILS_VERSION "1.9"
 
 #define DRIVER_NAME "ndiswrapper"
 #define DRIVER_CONFIG_DIR "/etc/ndiswrapper"
 
-#define SSID_MAX_WPA_IE_LEN 40
 #define NDIS_ESSID_MAX_SIZE 32
 #define NDIS_ENCODING_TOKEN_MAX 32
 #define MAX_ENCR_KEYS 4
 #define TX_RING_SIZE 16
 #define NDIS_MAX_RATES 8
 #define NDIS_MAX_RATES_EX 16
-#define WLAN_EID_GENERIC 221
-#define MAX_WPA_IE_LEN 64
-#define MAX_STR_LEN 512
 
 #define WRAP_PCI_BUS 5
 #define WRAP_PCMCIA_BUS 8
@@ -41,7 +37,7 @@
  * vendors say 15 is USB; which is correct? */
 #define WRAP_USB_BUS 15
 
-/* NDIS device must be 0, for compatability with old versions of
+/* NDIS device must be 0, for compatibility with old versions of
  * ndiswrapper where device type for NDIS drivers is 0 */
 #define WRAP_NDIS_DEVICE 0
 #define WRAP_USB_DEVICE 1
@@ -75,69 +71,72 @@
 /* DEBUG macros */
 
 #define MSG(level, fmt, ...)				\
-	printk(level "ndiswrapper (%s:%d): " fmt "\n",	\
-	       __FUNCTION__, __LINE__ , ## __VA_ARGS__)
+	printk(level DRIVER_NAME " (%s:%d): " fmt "\n",	\
+	       __func__, __LINE__ , ## __VA_ARGS__)
 
 #define WARNING(fmt, ...) MSG(KERN_WARNING, fmt, ## __VA_ARGS__)
 #define ERROR(fmt, ...) MSG(KERN_ERR, fmt , ## __VA_ARGS__)
 #define INFO(fmt, ...) MSG(KERN_INFO, fmt , ## __VA_ARGS__)
 #define TODO() WARNING("not fully implemented (yet)")
 
-#define TRACE(fmt, ...) do { } while (0)
-#define TRACE1(fmt, ...) do { } while (0)
-#define TRACE2(fmt, ...) do { } while (0)
-#define TRACE3(fmt, ...) do { }  while (0)
-#define TRACE4(fmt, ...) do { } while (0)
-#define TRACE5(fmt, ...) do { } while (0)
-#define TRACE6(fmt, ...) do { } while (0)
-
-/* for a block of code */
-#define DBG_BLOCK(level) while (0)
+#define TRACE(level, fmt, ...)						\
+do {									\
+	if (debug >= level)						\
+		printk(KERN_INFO "%s (%s:%d): " fmt "\n", DRIVER_NAME,	\
+		       __func__, __LINE__ , ## __VA_ARGS__);		\
+} while (0)
+#define TRACE0(fmt, ...) TRACE(0, fmt , ## __VA_ARGS__)
 
 extern int debug;
 
-#if defined DEBUG
-#undef TRACE
-#define TRACE(level, fmt, ...)				       \
-do {								       \
-	if (debug >= level)					       \
-		printk(KERN_INFO "%s (%s:%d): " fmt "\n", DRIVER_NAME, \
-		       __FUNCTION__, __LINE__ , ## __VA_ARGS__);       \
-} while (0)
-#undef DBG_BLOCK
+#ifndef DEBUG
+#define DEBUG 0
+#endif
+
+/* for a block of code */
+#if DEBUG >= 1
 #define DBG_BLOCK(level) if (debug >= level)
+#else
+#define DBG_BLOCK(level) while (0)
 #endif
 
-#if defined(DEBUG) && DEBUG >= 1
-#undef TRACE1
+#if DEBUG >= 1
 #define TRACE1(fmt, ...) TRACE(1, fmt , ## __VA_ARGS__)
+#else
+#define TRACE1(fmt, ...) do { } while (0)
 #endif
 
-#if defined(DEBUG) && DEBUG >= 2
-#undef TRACE2
+#if DEBUG >= 2
 #define TRACE2(fmt, ...) TRACE(2, fmt , ## __VA_ARGS__)
+#else
+#define TRACE2(fmt, ...) do { } while (0)
 #endif
 
-#if defined(DEBUG) && DEBUG >= 3
-#undef TRACE3
+#if DEBUG >= 3
 #define TRACE3(fmt, ...) TRACE(3, fmt , ## __VA_ARGS__)
+#else
+#define TRACE3(fmt, ...) do { } while (0)
 #endif
 
-#if defined(DEBUG) && DEBUG >= 4
-#undef TRACE4
+#if DEBUG >= 4
 #define TRACE4(fmt, ...) TRACE(4, fmt , ## __VA_ARGS__)
+#else
+#define TRACE4(fmt, ...) do { } while (0)
 #endif
 
-#if defined(DEBUG) && DEBUG >= 5
-#undef TRACE5
+#if DEBUG >= 5
 #define TRACE5(fmt, ...) TRACE(5, fmt , ## __VA_ARGS__)
+#else
+#define TRACE5(fmt, ...) do { } while (0)
 #endif
 
-#if defined(DEBUG) && DEBUG >= 6
-#undef TRACE6
+#if DEBUG >= 6
 #define TRACE6(fmt, ...) TRACE(6, fmt , ## __VA_ARGS__)
+#else
+#define TRACE6(fmt, ...) do { } while (0)
 #endif
 
+#define ENTER0(fmt, ...) TRACE0("Enter " fmt , ## __VA_ARGS__)
 #define ENTER1(fmt, ...) TRACE1("Enter " fmt , ## __VA_ARGS__)
 #define ENTER2(fmt, ...) TRACE2("Enter " fmt , ## __VA_ARGS__)
 #define ENTER3(fmt, ...) TRACE3("Enter " fmt , ## __VA_ARGS__)
@@ -145,64 +144,65 @@ do {								       \
 #define ENTER5(fmt, ...) TRACE5("Enter " fmt , ## __VA_ARGS__)
 #define ENTER6(fmt, ...) TRACE6("Enter " fmt , ## __VA_ARGS__)
 
-#define EXIT1(stmt) do { TRACE1("Exit"); stmt; } while(0)
-#define EXIT2(stmt) do { TRACE2("Exit"); stmt; } while(0)
-#define EXIT3(stmt) do { TRACE3("Exit"); stmt; } while(0)
-#define EXIT4(stmt) do { TRACE4("Exit"); stmt; } while(0)
-#define EXIT5(stmt) do { TRACE5("Exit"); stmt; } while(0)
-#define EXIT6(stmt) do { TRACE6("Exit"); stmt; } while(0)
+#define EXIT0(stmt) do { TRACE0("Exit"); stmt; } while (0)
+#define EXIT1(stmt) do { TRACE1("Exit"); stmt; } while (0)
+#define EXIT2(stmt) do { TRACE2("Exit"); stmt; } while (0)
+#define EXIT3(stmt) do { TRACE3("Exit"); stmt; } while (0)
+#define EXIT4(stmt) do { TRACE4("Exit"); stmt; } while (0)
+#define EXIT5(stmt) do { TRACE5("Exit"); stmt; } while (0)
+#define EXIT6(stmt) do { TRACE6("Exit"); stmt; } while (0)
 
 #if defined(USB_DEBUG)
-#define USBTRACE TRACE1
-#define USBENTER ENTER1
-#define USBEXIT EXIT1
+#define USBTRACE TRACE0
+#define USBENTER ENTER0
+#define USBEXIT EXIT0
 #else
-#define USBTRACE(fmt, ...)
+#define USBTRACE(fmt, ...) do { } while (0)
 #define USBENTER(fmt, ...)
 #define USBEXIT(stmt) stmt
 #endif
 
 #if defined(EVENT_DEBUG)
-#define EVENTTRACE TRACE1
-#define EVENTENTER ENTER1
-#define EVENTEXIT EXIT1
+#define EVENTTRACE TRACE0
+#define EVENTENTER ENTER0
+#define EVENTEXIT EXIT0
 #else
-#define EVENTTRACE(fmt, ...)
+#define EVENTTRACE(fmt, ...) do { } while (0)
 #define EVENTENTER(fmt, ...)
 #define EVENTEXIT(stmt) stmt
 #endif
 
 #if defined(TIMER_DEBUG)
-#define TIMERTRACE TRACE1
-#define TIMERENTER ENTER1
-#define TIMEREXIT EXIT1
+#define TIMERTRACE TRACE0
+#define TIMERENTER ENTER0
+#define TIMEREXIT EXIT0
 #else
-#define TIMERTRACE(fmt, ...)
+#define TIMERTRACE(fmt, ...) do { } while (0)
 #define TIMERENTER(fmt, ...)
 #define TIMEREXIT(stmt) stmt
 #endif
 
 #if defined(IO_DEBUG)
-#define IOTRACE TRACE1
-#define IOENTER ENTER1
-#define IOEXIT EXIT1
+#define IOTRACE TRACE0
+#define IOENTER ENTER0
+#define IOEXIT EXIT0
 #else
-#define IOTRACE(fmt, ...)
+#define IOTRACE(fmt, ...) do { } while (0)
 #define IOENTER(fmt, ...)
 #define IOEXIT(stmt) stmt
 #endif
 
 #if defined(WORK_DEBUG)
-#define WORKTRACE TRACE1
-#define WORKENTER ENTER1
-#define WORKEXIT EXIT1
+#define WORKTRACE TRACE0
+#define WORKENTER ENTER0
+#define WORKEXIT EXIT0
 #else
-#define WORKTRACE(fmt, ...)
+#define WORKTRACE(fmt, ...) do { } while (0)
 #define WORKENTER(fmt, ...)
 #define WORKEXIT(stmt) stmt
 #endif
 
-#ifdef DEBUG
+#if DEBUG >= 1
 #define assert(expr)							\
 do {									\
 	if (!(expr)) {							\
