@@ -61,6 +61,41 @@
 #endif
 #endif /* Linux < 2.6.29 */
 
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,16)
+#ifndef mutex_init
+#define mutex semaphore
+#define mutex_init(m) sema_init(m, 1)
+#define mutex_lock(m) down(m)
+#define mutex_trylock(m) (!down_trylock(m))
+#define mutex_unlock(m) up(m)
+#define mutex_is_locked(m) (atomic_read(m.count) == 0)
+#endif
+#endif /* Linux < 2.6.16 */
+
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,26)
+#define set_cpus_allowed_ptr(task, mask) set_cpus_allowed(task, *mask)
+#endif /* Linux < 2.6.26 */
+
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,28)
+#define cpumask_copy(dst, src) do { *dst = *src; } while (0)
+#define cpumask_equal(mask1, mask2) cpus_equal(*mask1, *mask2)
+#define cpumask_setall(mask) cpus_setall(*mask)
+static cpumask_t cpumasks[NR_CPUS];
+#define cpumask_of(cpu) 			\
+({						\
+	cpumasks[cpu] = cpumask_of_cpu(cpu);	\
+	&cpumasks[cpu];				\
+})
+#endif /* Linux < 2.6.28 */
+
+#ifndef tsk_cpus_allowed
+#define tsk_cpus_allowed(tsk) (&(tsk)->cpus_allowed)
+#endif
+
+#ifndef __packed
+#define __packed __attribute__((packed))
+#endif
+
 /* pci functions in 2.6 kernels have problems allocating dma buffers,
  * but seem to work fine with dma functions
  */
@@ -130,171 +165,70 @@ do {							\
 
 #ifdef WRAP_WQ
 
-struct workqueue_struct;
+struct wrap_workqueue_struct;
 
-struct workqueue_thread {
-	spinlock_t lock;
-	struct task_struct *task;
-	struct completion *completion;
-	char name[16];
-	int pid;
-	/* whether any work_structs pending? <0 implies quit */
-	s8 pending;
-	/* list of work_structs pending */
-	struct list_head work_list;
-};
-
-typedef struct workqueue_struct {
-	u8 singlethread;
-	u8 qon;
-	int num_cpus;
-	struct workqueue_thread threads[0];
-} workqueue_struct_t;
-
-typedef struct {
+struct wrap_work_struct {
 	struct list_head list;
-	void (*func)(void *data);
+	void (*func)(struct wrap_work_struct *data);
 	void *data;
 	/* whether/on which thread scheduled */
 	struct workqueue_thread *thread;
-} work_struct_t;
+};
 
-#define initialize_work(work, pfunc, pdata)			\
+#define work_struct wrap_work_struct
+#define workqueue_struct wrap_workqueue_struct
+
+#undef INIT_WORK
+#define INIT_WORK(work, pfunc)					\
 	do {							\
 		(work)->func = (pfunc);				\
-		(work)->data = (pdata);				\
+		(work)->data = (work);				\
 		(work)->thread = NULL;				\
 	} while (0)
 
 #undef create_singlethread_workqueue
-#define create_singlethread_workqueue(name) wrap_create_wq(name, 1, 0)
+#define create_singlethread_workqueue(wq) wrap_create_wq(wq, 1, 0)
 #undef create_workqueue
-#define create_workqueue(name) wrap_create_wq(name, 0, 0)
+#define create_workqueue(wq) wrap_create_wq(wq, 0, 0)
 #undef destroy_workqueue
-#define destroy_workqueue wrap_destroy_wq
+#define destroy_workqueue(wq) wrap_destroy_wq(wq)
 #undef queue_work
-#define queue_work wrap_queue_work
+#define queue_work(wq, work) wrap_queue_work(wq, work)
 #undef flush_workqueue
-#define flush_workqueue wrap_flush_wq
+#define flush_workqueue(wq) wrap_flush_wq(wq)
 
-workqueue_struct_t *wrap_create_wq(const char *name, u8 singlethread, u8 freeze);
-void wrap_destroy_wq_on(workqueue_struct_t *workq, int cpu);
-void wrap_destroy_wq(workqueue_struct_t *workq);
-int wrap_queue_work_on(workqueue_struct_t *workq, work_struct_t *work,
-		       int cpu);
-int wrap_queue_work(workqueue_struct_t *workq, work_struct_t *work);
-void wrap_cancel_work(work_struct_t *work);
-void wrap_flush_wq_on(workqueue_struct_t *workq, int cpu);
-void wrap_flush_wq(workqueue_struct_t *workq);
-typedef void *worker_param_t;
-#define worker_param_data(param, type, member) param
+struct workqueue_struct *wrap_create_wq(const char *name, u8 singlethread,
+					u8 freeze);
+void wrap_destroy_wq(struct workqueue_struct *workq);
+int wrap_queue_work(struct workqueue_struct *workq, struct work_struct *work);
+void wrap_cancel_work(struct work_struct *work);
+void wrap_flush_wq(struct workqueue_struct *workq);
 
 #else // WRAP_WQ
 
-typedef struct workqueue_struct workqueue_struct_t;
-typedef struct work_struct work_struct_t;
-
-#if defined(INIT_WORK_NAR) || defined(INIT_DELAYED_WORK_DEFERRABLE)
-#define initialize_work(work, func, data) INIT_WORK(work, func)
-typedef struct work_struct *worker_param_t;
-#define worker_param_data(param, type, member)	\
-	container_of(param, type, member)
-#else
-#define initialize_work(work, func, data) INIT_WORK(work, func, data)
-typedef void *worker_param_t;
-#define worker_param_data(param, type, member) param
-#endif // INIT_WORK_NAR
+/* Compatibility for Linux before 2.6.20 where INIT_WORK takes 3 arguments */
+#if !defined(INIT_WORK_NAR) && !defined(INIT_DELAYED_WORK_DEFERRABLE)
+typedef void (*compat_work_func_t)(void *work);
+typedef void (*work_func_t)(struct work_struct *work);
+static inline void (INIT_WORK)(struct work_struct *work, work_func_t func)
+{
+	INIT_WORK(work, (compat_work_func_t)func, work);
+}
+#undef INIT_WORK
+#endif
 
 #endif // WRAP_WQ
 
-struct nt_thread *wrap_worker_init(workqueue_struct_t *wq);
-
-#ifdef module_param
-#define WRAP_MODULE_PARM_INT(name, perm) module_param(name, int, perm)
-#define WRAP_MODULE_PARM_STRING(name, perm) module_param(name, charp, perm)
-#else
-#define WRAP_MODULE_PARM_INT(name, perm) MODULE_PARM(name, "i")
-#define WRAP_MODULE_PARM_STRING(name, perm) MODULE_PARM(name, "s")
-#endif
-
-#ifndef LOCK_PREFIX
-#ifdef LOCK
-#define LOCK_PREFIX LOCK
-#else
-#ifdef CONFIG_SMP
-#define LOCK_PREFIX "lock ; "
-#else
-#define LOCK_PREFIX ""
-#endif
-#endif
-#endif
-
-#ifndef NETDEV_TX_OK
-#define NETDEV_TX_OK 0
-#endif
-
-#ifndef NETDEV_TX_BUSY
-#define NETDEV_TX_BUSY 1
-#endif
-
-#ifndef CHECKSUM_HW
-#define CHECKSUM_HW CHECKSUM_PARTIAL
-#endif
-
-#ifndef offset_in_page
-#define offset_in_page(p) ((unsigned long)(p) & ~PAGE_MASK)
-#endif
-
-#ifndef PMSG_SUSPEND
-#ifdef PM_SUSPEND
-/* this is not correct - the value of PM_SUSPEND is different from
- * PMSG_SUSPEND, but ndiswrapper doesn't care about the value when
- * suspending */
-#define PMSG_SUSPEND PM_SUSPEND
-#define PSMG_ON PM_ON
-#else
-typedef u32 pm_message_t;
-#define PMSG_SUSPEND 2
-#define PMSG_ON 0
-#endif
-#endif
-
-#ifndef PCI_D0
-#define PCI_D0 0
-#endif
-
-#ifndef PCI_D3hot
-#define PCI_D3hot 3
-#endif
-
-#ifndef PCI_D3cold
-#define PCI_D3cold 3
-#endif
-
-#ifndef PM_EVENT_SUSPEND
-#define PM_EVENT_SUSPEND 2
-#endif
-
-#if !defined(HAVE_NETDEV_PRIV)
-#define netdev_priv(dev)  ((dev)->priv)
-#endif
-
 #if LINUX_VERSION_CODE > KERNEL_VERSION(2,6,18)
 #define ISR_PT_REGS_PARAM_DECL
-#define ISR_PT_REGS_ARG
 #else
 #define ISR_PT_REGS_PARAM_DECL , struct pt_regs *regs
-#define ISR_PT_REGS_ARG , NULL
 #endif
 
 #if LINUX_VERSION_CODE <= KERNEL_VERSION(2,6,16)
 #define for_each_possible_cpu(_cpu) for_each_cpu(_cpu)
 #endif
 
-#ifndef flush_icache_range
-#define flush_icache_range(start, end) do { } while (0)
-#endif
-
 #ifndef CHECKSUM_PARTIAL
 #define CHECKSUM_PARTIAL CHECKSUM_HW
 #endif
@@ -371,6 +305,14 @@ static inline void netif_poll_disable(struct net_device *dev)
 #define proc_net_root proc_net
 #endif
 
+#if ((LINUX_VERSION_CODE < KERNEL_VERSION(3,2,0)) && \
+     (LINUX_VERSION_CODE >= KERNEL_VERSION(3,0,0))) || \
+    (LINUX_VERSION_CODE < KERNEL_VERSION(2,6,42))
+#ifndef skb_frag_page
+#define skb_frag_page(frag) ((frag)->page)
+#endif
+#endif
+
 /* TICK is 100ns */
 #define TICKSPERSEC		10000000
 #define TICKSPERMSEC		10000
@@ -426,7 +368,7 @@ struct wrap_export {
 
 #endif
 
-#define WIN_FUNC(name, argc) name
+#define WIN_FUNC(name, argc) (name)
 /* map name s to f - if f is different from s */
 #define WIN_SYMBOL_MAP(s, f)
 
@@ -487,14 +429,13 @@ struct wrap_driver {
 	struct pe_image pe_images[MAX_DRIVER_PE_IMAGES];
 	unsigned short num_bin_files;
 	struct wrap_bin_file *bin_files;
-	struct nt_list wrap_devices;
 	struct nt_list settings;
 	int dev_type;
 	struct ndis_driver *ndis_driver;
 };
 
 enum hw_status {
-	HW_INITIALIZED = 1, HW_SUSPENDED, HW_HALTED, HW_PRESENT,
+	HW_INITIALIZED = 1, HW_SUSPENDED, HW_HALTED, HW_DISABLED,
 };
 
 struct wrap_device {
@@ -527,9 +468,6 @@ struct wrap_device {
 			struct nt_list wrap_urb_list;
 		} usb;
 	};
-	union {
-		struct ndis_device *wnd;
-	};
 };
 
 #define wrap_is_pci_bus(dev_bus)			\
@@ -547,15 +485,9 @@ struct wrap_device {
 	(WRAP_DEVICE(dev_bus) == WRAP_BLUETOOTH_DEVICE1 ||	\
 	 WRAP_DEVICE(dev_bus) == WRAP_BLUETOOTH_DEVICE2)
 
-extern workqueue_struct_t *ntos_wq;
-#define schedule_ntos_work(work_struct) queue_work(ntos_wq, work_struct)
-#define schedule_work(work_struct) queue_work(ntos_wq, work_struct)
-
-extern workqueue_struct_t *ndis_wq;
-#define schedule_ndis_work(work_struct) queue_work(ndis_wq, work_struct)
-
-extern workqueue_struct_t *wrapndis_wq;
-#define schedule_wrapndis_work(work_struct) queue_work(wrapndis_wq, work_struct)
+extern struct workqueue_struct *ntos_wq;
+extern struct workqueue_struct *ndis_wq;
+extern struct workqueue_struct *wrapndis_wq;
 
 #define atomic_unary_op(var, size, oper)				\
 do {									\
@@ -597,14 +529,6 @@ do {									\
 
 #define post_atomic_add(var, i) (pre_atomic_add(var, i) + i)
 
-#ifndef in_atomic
-#define in_atomic() in_interrupt()
-#endif
-
-#ifndef preempt_enable_no_resched
-#define preempt_enable_no_resched() preempt_enable()
-#endif
-
 //#define DEBUG_IRQL 1
 
 #ifdef DEBUG_IRQL
@@ -642,33 +566,29 @@ do {									\
 
 #ifdef WRAP_PREEMPT
 
-typedef struct {
+struct irql_info {
 	int count;
 	struct mutex lock;
 #ifdef CONFIG_SMP
-	typeof(current->cpus_allowed) cpus_allowed;
+	cpumask_t cpus_allowed;
 #endif
 	struct task_struct *task;
-} irql_info_t;
+};
 
-DECLARE_PER_CPU(irql_info_t, irql_info);
+DECLARE_PER_CPU(struct irql_info, irql_info);
 
 static inline KIRQL raise_irql(KIRQL newirql)
 {
-	irql_info_t *info;
+	struct irql_info *info;
 
 	assert(newirql == DISPATCH_LEVEL);
 	info = &get_cpu_var(irql_info);
 	if (info->task == current) {
 		assert(info->count > 0);
 		assert(mutex_is_locked(&info->lock));
-#if defined(CONFIG_SMP) && defined(DEBUG)
-		do {
-			cpumask_t cpumask;
-			cpumask = cpumask_of_cpu(smp_processor_id());
-			cpus_xor(cpumask, cpumask, current->cpus_allowed);
-			assert(cpus_empty(cpumask));
-		} while (0);
+#if defined(CONFIG_SMP) && DEBUG >= 1
+		assert(cpumask_equal(tsk_cpus_allowed(current),
+				     cpumask_of(smp_processor_id())));
 #endif
 		info->count++;
 		put_cpu_var(irql_info);
@@ -677,8 +597,8 @@ static inline KIRQL raise_irql(KIRQL newirql)
 	/* TODO: is this enough to pin down to current cpu? */
 #ifdef CONFIG_SMP
 	assert(task_cpu(current) == smp_processor_id());
-	info->cpus_allowed = current->cpus_allowed;
-	current->cpus_allowed = cpumask_of_cpu(smp_processor_id());
+	cpumask_copy(&info->cpus_allowed, tsk_cpus_allowed(current));
+	set_cpus_allowed_ptr(current, cpumask_of(smp_processor_id()));
 #endif
 	put_cpu_var(irql_info);
 	mutex_lock(&info->lock);
@@ -691,7 +611,7 @@ static inline KIRQL raise_irql(KIRQL newirql)
 
 static inline void lower_irql(KIRQL oldirql)
 {
-	irql_info_t *info;
+	struct irql_info *info;
 
 	assert(oldirql <= DISPATCH_LEVEL);
 	info = &get_cpu_var(irql_info);
@@ -701,7 +621,7 @@ static inline void lower_irql(KIRQL oldirql)
 	if (--info->count == 0) {
 		info->task = NULL;
 #ifdef CONFIG_SMP
-		current->cpus_allowed = info->cpus_allowed;
+		set_cpus_allowed_ptr(current, &info->cpus_allowed);
 #endif
 		mutex_unlock(&info->lock);
 	}
@@ -767,7 +687,7 @@ static inline void lower_irql(KIRQL oldirql)
 #define NT_SPIN_LOCK_UNLOCKED 0
 #define NT_SPIN_LOCK_LOCKED 1
 
-static inline void  nt_spin_lock_init(NT_SPIN_LOCK *lock)
+static inline void nt_spin_lock_init(NT_SPIN_LOCK *lock)
 {
 	*lock = NT_SPIN_LOCK_UNLOCKED;
 }
@@ -776,31 +696,34 @@ static inline void  nt_spin_lock_init(NT_SPIN_LOCK *lock)
 
 static inline void nt_spin_lock(NT_SPIN_LOCK *lock)
 {
-	__asm__ __volatile__(
-		"1:\t"
-		"  xchgl %1, %0\n\t"
-		"  testl %1, %1\n\t"
-		"  jz 3f\n"
-		"2:\t"
-		"  rep; nop\n\t"
-		"  cmpl %2, %0\n\t"
-		"  je 1b\n\t"
-		"  jmp 2b\n"
-		"3:\n\t"
-		: "+m" (*lock)
-		: "r" (NT_SPIN_LOCK_LOCKED), "i" (NT_SPIN_LOCK_UNLOCKED));
+	while (1) {
+		unsigned long lockval = xchg(lock, NT_SPIN_LOCK_LOCKED);
+
+		if (likely(lockval == NT_SPIN_LOCK_UNLOCKED))
+			break;
+		if (unlikely(lockval > NT_SPIN_LOCK_LOCKED)) {
+			ERROR("bad spinlock: 0x%lx at %p", lockval, lock);
+			return;
+		}
+		/* "rep; nop" doesn't change cx register, it's a "pause" */
+		__asm__ __volatile__("rep; nop");
+	}
 }
 
 static inline void nt_spin_unlock(NT_SPIN_LOCK *lock)
 {
-	*lock = NT_SPIN_LOCK_UNLOCKED;
+	unsigned long lockval = xchg(lock, NT_SPIN_LOCK_UNLOCKED);
+
+	if (likely(lockval == NT_SPIN_LOCK_LOCKED))
+		return;
+	WARNING("unlocking unlocked spinlock: 0x%lx at %p", lockval, lock);
 }
 
 #else // CONFIG_SMP
 
 #define nt_spin_lock(lock) do { } while (0)
 
-#define nt_spin_unlock(lock)  do { } while (0)
+#define nt_spin_unlock(lock) do { } while (0)
 
 #endif // CONFIG_SMP
 
@@ -942,18 +865,17 @@ int ntoskernel_init_device(struct wrap_device *wd);
 void ntoskernel_exit_device(struct wrap_device *wd);
 void *allocate_object(ULONG size, enum common_object_type type,
 		      struct unicode_string *name);
-void free_object(void *object);
 
+#ifdef ENABLE_USB
 int usb_init(void);
 void usb_exit(void);
+#else
+static inline int usb_init(void) { return 0; }
+static inline void usb_exit(void) {}
+#endif
 int usb_init_device(struct wrap_device *wd);
 void usb_exit_device(struct wrap_device *wd);
-void usb_cancel_pending_urbs(void);
 
-int crt_init(void);
-void crt_exit(void);
-int rtl_init(void);
-void rtl_exit(void);
 int wrap_procfs_init(void);
 void wrap_procfs_remove(void);
 
@@ -997,16 +919,9 @@ void ObfDereferenceObject(void *object) wfastcall;
 #define ObReferenceObject(object) ObfReferenceObject(object)
 #define ObDereferenceObject(object) ObfDereferenceObject(object)
 
-void WRITE_PORT_UCHAR(ULONG_PTR port, UCHAR value) wstdcall;
-UCHAR READ_PORT_UCHAR(ULONG_PTR port) wstdcall;
-
-#undef ExAllocatePoolWithTag
-void *ExAllocatePoolWithTag(enum pool_type pool_type, SIZE_T size,
-			    ULONG tag) wstdcall;
-#if defined(ALLOC_DEBUG) && ALLOC_DEBUG > 1
-#define ExAllocatePoolWithTag(pool_type, size, tag)			\
-	wrap_ExAllocatePoolWithTag(pool_type, size, tag, __FILE__, __LINE__)
-#endif
+/* prevent expansion of ExAllocatePoolWithTag macro */
+void *(ExAllocatePoolWithTag)(enum pool_type pool_type, SIZE_T size,
+			      ULONG tag) wstdcall;
 
 void ExFreePool(void *p) wstdcall;
 ULONG MmSizeOfMdl(void *base, ULONG length) wstdcall;
@@ -1021,12 +936,9 @@ void KeInitializeEvent(struct nt_event *nt_event,
 LONG KeSetEvent(struct nt_event *nt_event, KPRIORITY incr,
 		BOOLEAN wait) wstdcall;
 LONG KeResetEvent(struct nt_event *nt_event) wstdcall;
-void KeClearEvent(struct nt_event *nt_event) wstdcall;
-void KeInitializeDpc(struct kdpc *kdpc, void *func, void *ctx) wstdcall;
 BOOLEAN queue_kdpc(struct kdpc *kdpc);
 BOOLEAN dequeue_kdpc(struct kdpc *kdpc);
 
-void KeFlushQueuedDpcs(void) wstdcall;
 NTSTATUS IoConnectInterrupt(struct kinterrupt **kinterrupt,
 			    PKSERVICE_ROUTINE service_routine,
 			    void *service_context, NT_SPIN_LOCK *lock,
@@ -1042,10 +954,7 @@ BOOLEAN KeSynchronizeExecution(struct kinterrupt *interrupt,
 NTSTATUS KeWaitForSingleObject(void *object, KWAIT_REASON reason,
 			       KPROCESSOR_MODE waitmode, BOOLEAN alertable,
 			       LARGE_INTEGER *timeout) wstdcall;
-struct mdl *IoAllocateMdl(void *virt, ULONG length, BOOLEAN second_buf,
-			  BOOLEAN charge_quota, struct irp *irp) wstdcall;
 void MmBuildMdlForNonPagedPool(struct mdl *mdl) wstdcall;
-void IoFreeMdl(struct mdl *mdl) wstdcall;
 NTSTATUS IoCreateDevice(struct driver_object *driver, ULONG dev_ext_length,
 			struct unicode_string *dev_name, DEVICE_TYPE dev_type,
 			ULONG dev_chars, BOOLEAN exclusive,
@@ -1064,20 +973,11 @@ void *IoGetDriverObjectExtension(struct driver_object *drv,
 				 void *client_id) wstdcall;
 struct device_object *IoAttachDeviceToDeviceStack
 	(struct device_object *src, struct device_object *dst) wstdcall;
-void KeInitializeEvent(struct nt_event *nt_event, enum event_type type,
-		       BOOLEAN state) wstdcall;
-struct irp *IoAllocateIrp(char stack_count, BOOLEAN charge_quota) wstdcall;
-void IoFreeIrp(struct irp *irp) wstdcall;
 BOOLEAN IoCancelIrp(struct irp *irp) wstdcall;
 struct irp *IoBuildSynchronousFsdRequest
 	(ULONG major_func, struct device_object *dev_obj, void *buf,
 	 ULONG length, LARGE_INTEGER *offset, struct nt_event *event,
 	 struct io_status_block *status) wstdcall;
-struct irp *IoBuildAsynchronousFsdRequest
-	(ULONG major_func, struct device_object *dev_obj, void *buf,
-	 ULONG length, LARGE_INTEGER *offset,
-	 struct io_status_block *status) wstdcall;
-NTSTATUS PoCallDriver(struct device_object *dev_obj, struct irp *irp) wstdcall;
 
 NTSTATUS IoPassIrpDown(struct device_object *dev_obj, struct irp *irp) wstdcall;
 WIN_FUNC_DECL(IoPassIrpDown,2);
@@ -1088,16 +988,10 @@ NTSTATUS IoAsyncForwardIrp(struct device_object *dev_obj,
 NTSTATUS IoInvalidDeviceRequest(struct device_object *dev_obj,
 				struct irp *irp) wstdcall;
 
-KIRQL KeGetCurrentIrql(void) wstdcall;
 void KeInitializeSpinLock(NT_SPIN_LOCK *lock) wstdcall;
-void KeAcquireSpinLock(NT_SPIN_LOCK *lock, KIRQL *irql) wstdcall;
-void KeReleaseSpinLock(NT_SPIN_LOCK *lock, KIRQL oldirql) wstdcall;
-KIRQL KeAcquireSpinLockRaiseToDpc(NT_SPIN_LOCK *lock) wstdcall;
-
 void IoAcquireCancelSpinLock(KIRQL *irql) wstdcall;
 void IoReleaseCancelSpinLock(KIRQL irql) wstdcall;
 
-void RtlCopyMemory(void *dst, const void *src, SIZE_T length) wstdcall;
 NTSTATUS RtlUnicodeStringToAnsiString
 	(struct ansi_string *dst, const struct unicode_string *src,
 	 BOOLEAN dup) wstdcall;
@@ -1105,7 +999,6 @@ NTSTATUS RtlAnsiStringToUnicodeString
 	(struct unicode_string *dst, const struct ansi_string *src,
 	 BOOLEAN dup) wstdcall;
 void RtlInitAnsiString(struct ansi_string *dst, const char *src) wstdcall;
-void RtlInitString(struct ansi_string *dst, const char *src) wstdcall;
 void RtlInitUnicodeString(struct unicode_string *dest,
 			  const wchar_t *src) wstdcall;
 void RtlFreeUnicodeString(struct unicode_string *string) wstdcall;
@@ -1113,30 +1006,16 @@ void RtlFreeAnsiString(struct ansi_string *string) wstdcall;
 LONG RtlCompareUnicodeString(const struct unicode_string *s1,
 			     const struct unicode_string *s2,
 			     BOOLEAN case_insensitive) wstdcall;
-void RtlCopyUnicodeString(struct unicode_string *dst,
-			  struct unicode_string *src) wstdcall;
 NTSTATUS RtlUpcaseUnicodeString(struct unicode_string *dst,
 				struct unicode_string *src,
 				BOOLEAN alloc) wstdcall;
-void KeInitializeTimer(struct nt_timer *nt_timer) wstdcall;
-void KeInitializeTimerEx(struct nt_timer *nt_timer,
-			 enum timer_type type) wstdcall;
-BOOLEAN KeSetTimerEx(struct nt_timer *nt_timer, LARGE_INTEGER duetime_ticks,
-		     LONG period_ms, struct kdpc *kdpc) wstdcall;
-BOOLEAN KeSetTimer(struct nt_timer *nt_timer, LARGE_INTEGER duetime_ticks,
-		   struct kdpc *kdpc) wstdcall;
 BOOLEAN KeCancelTimer(struct nt_timer *nt_timer) wstdcall;
 void KeInitializeDpc(struct kdpc *kdpc, void *func, void *ctx) wstdcall;
-struct nt_thread *KeGetCurrentThread(void) wstdcall;
-NTSTATUS ObReferenceObjectByHandle(void *handle, ACCESS_MASK desired_access,
-				   void *obj_type, KPROCESSOR_MODE access_mode,
-				   void **object, void *handle_info) wstdcall;
-
-void adjust_user_shared_data_addr(char *driver, unsigned long length);
 
 extern spinlock_t ntoskernel_lock;
 extern spinlock_t irp_cancel_lock;
 extern struct nt_list object_list;
+extern CCHAR cpu_count;
 #ifdef CONFIG_X86_64
 extern struct kuser_shared_data kuser_shared_data;
 #endif
