@@ -18,20 +18,17 @@
 #include "wrapndis.h"
 #include "usb.h"
 #include "loader.h"
-
-extern spinlock_t ntoskernel_lock;
-extern spinlock_t irp_cancel_lock;
-extern struct nt_list object_list;
+#include "ntoskernel_io_exports.h"
 
 wstdcall void WIN_FUNC(IoAcquireCancelSpinLock,1)
-	(KIRQL *irql)
+	(KIRQL *irql) __acquires(irql)
 {
 	spin_lock_bh(&irp_cancel_lock);
 	*irql = 0;
 }
 
 wstdcall void WIN_FUNC(IoReleaseCancelSpinLock,1)
-	(KIRQL irql)
+	(KIRQL irql) __releases(irql)
 {
 	spin_unlock_bh(&irp_cancel_lock);
 }
@@ -117,11 +114,18 @@ wstdcall BOOLEAN WIN_FUNC(IoCancelIrp,1)
 	IOTRACE("%p", cancel_routine);
 	irp->cancel = TRUE;
 	if (cancel_routine) {
-		struct io_stack_location *irp_sl;
-		irp_sl = IoGetCurrentIrpStackLocation(irp);
-		IOTRACE("%p, %p", irp_sl, irp_sl->dev_obj);
+		struct device_object *dev_obj;
+
+		if (irp->current_location >= 0 &&
+		    irp->current_location < irp->stack_count)
+			dev_obj = IoGetCurrentIrpStackLocation(irp)->dev_obj;
+		else
+			dev_obj = NULL;
+		IOTRACE("current_location: %d, dev_obj: %p",
+			irp->current_location, dev_obj);
 		/* cancel_routine will release the spin lock */
-		LIN2WIN2(cancel_routine, irp_sl->dev_obj, irp);
+		__release(irp->cancel_irql);
+		LIN2WIN2(cancel_routine, dev_obj, irp);
 		/* in usb's cancel, irp->cancel is set to indicate
 		 * status of cancel */
 		IOEXIT(return xchg(&irp->cancel, TRUE));
@@ -166,8 +170,14 @@ wstdcall void WIN_FUNC(IoFreeIrp,1)
 	(struct irp *irp)
 {
 	IOENTER("irp = %p", irp);
+	if (!irp) {
+		WARNING("irp is NULL");
+		return;
+	}
+
 	if (irp->flags & IRP_SYNCHRONOUS_API)
 		IoDequeueThreadIrp(irp);
+	IoCancelIrp(irp);
 	kfree(irp);
 
 	IOEXIT(return);
@@ -524,6 +534,10 @@ static irqreturn_t io_irq_isr(int irq, void *data ISR_PT_REGS_PARAM_DECL)
 	struct kinterrupt *interrupt = data;
 	BOOLEAN ret;
 
+#ifdef CONFIG_DEBUG_SHIRQ
+	if (!interrupt->u.enabled)
+		EXIT1(return IRQ_NONE);
+#endif
 	TRACE6("%p", interrupt);
 	nt_spin_lock(interrupt->actual_lock);
 	ret = LIN2WIN2(interrupt->isr, interrupt, interrupt->isr_ctx);
@@ -561,18 +575,24 @@ wstdcall NTSTATUS WIN_FUNC(IoConnectInterrupt,11)
 	interrupt->synch_irql = synch_irql;
 	interrupt->mode = mode;
 	if (request_irq(vector, io_irq_isr, shared ? IRQF_SHARED : 0,
-			"ndiswrapper", interrupt)) {
+			DRIVER_NAME, interrupt)) {
 		WARNING("request for irq %d failed", vector);
 		kfree(interrupt);
 		IOEXIT(return STATUS_INSUFFICIENT_RESOURCES);
 	}
 	*kinterrupt = interrupt;
+#ifdef CONFIG_DEBUG_SHIRQ
+	interrupt->u.enabled = 1;
+#endif
 	IOEXIT(return STATUS_SUCCESS);
 }
 
 wstdcall void WIN_FUNC(IoDisconnectInterrupt,1)
 	(struct kinterrupt *interrupt)
 {
+#ifdef CONFIG_DEBUG_SHIRQ
+	interrupt->u.enabled = 0;
+#endif
 	free_irq(interrupt->vector, interrupt);
 	kfree(interrupt);
 }
@@ -925,13 +945,12 @@ wstdcall NTSTATUS WIN_FUNC(IoGetDeviceProperty,5)
 
 wstdcall NTSTATUS WIN_FUNC(IoGetDeviceObjectPointer,4)
 	(struct unicode_string *name, ACCESS_MASK desired_access,
-	 void *file_obj, struct device_object *dev_obj)
+	 struct file_object *file_obj, struct device_object *dev_obj)
 {
 	struct common_object_header *coh;
 
 	dev_obj = NULL;
-	/* TODO: access is not checked and file_obj is set to NULL */
-	file_obj = NULL;
+	/* TODO: access is not checked and file_obj is filled with zeroes */
 	spin_lock_bh(&ntoskernel_lock);
 	nt_list_for_each_entry(coh, &object_list, list) {
 		TRACE5("header: %p, type: %d", coh, coh->type);
@@ -944,9 +963,11 @@ wstdcall NTSTATUS WIN_FUNC(IoGetDeviceObjectPointer,4)
 		}
 	}
 	spin_unlock_bh(&ntoskernel_lock);
-	if (dev_obj)
+	if (dev_obj) {
+		memset(file_obj, 0, sizeof(*file_obj));
+		WARNING("file_obj filled with zeroes");
 		IOEXIT(return STATUS_SUCCESS);
-	else
+	} else
 		IOEXIT(return STATUS_OBJECT_NAME_INVALID);
 }
 
@@ -1033,7 +1054,7 @@ wstdcall void WIN_FUNC(IoFreeErrorLogEntry,1)
 }
 
 wstdcall NTSTATUS WIN_FUNC(IoAcquireRemoveLockEx,5)
-	(struct io_remove_lock lock, void *tag, char *file, ULONG line,
+	(struct io_remove_lock *lock, void *tag, char *file, ULONG line,
 	 ULONG lock_size)
 {
 	TODO();
@@ -1041,7 +1062,7 @@ wstdcall NTSTATUS WIN_FUNC(IoAcquireRemoveLockEx,5)
 }
 
 wstdcall NTSTATUS WIN_FUNC(IoReleaseRemoveLockEx,3)
-	(struct io_remove_lock lock, void *tag, ULONG lock_size)
+	(struct io_remove_lock *lock, void *tag, ULONG lock_size)
 {
 	TODO();
 	IOEXIT(return STATUS_SUCCESS);
@@ -1097,4 +1118,21 @@ wstdcall void WIN_FUNC(IoInvalidateDeviceState,1)
 	TODO();
 }
 
-#include "ntoskernel_io_exports.h"
+wstdcall NTSTATUS WIN_FUNC(IoRegisterPlugPlayNotification,7)
+	(UINT category, ULONG flags, void *data, struct driver_object *object,
+	 void *callback, void *context, void **notification_entry)
+{
+	TRACE2("category: %d, flags 0x%x, data: %p, object: %p, callback: %p, "
+	       "context: %p, notification_entry: %p", category, flags, data,
+	       object, callback, context, notification_entry);
+	TODO();
+	IOEXIT(return STATUS_SUCCESS);
+}
+
+wstdcall NTSTATUS WIN_FUNC(IoUnregisterPlugPlayNotification,1)
+	(void *notification_entry)
+{
+	TRACE2("%p", notification_entry);
+	TODO();
+	IOEXIT(return STATUS_SUCCESS);
+}
