@@ -18,7 +18,11 @@
 #include "wrapndis.h"
 #include "loader.h"
 
-extern struct semaphore loader_mutex;
+/* Functions callable from the NDIS driver */
+wstdcall NTSTATUS pdoDispatchDeviceControl(struct device_object *pdo,
+					   struct irp *irp);
+wstdcall NTSTATUS pdoDispatchPnp(struct device_object *pdo, struct irp *irp);
+wstdcall NTSTATUS pdoDispatchPower(struct device_object *pdo, struct irp *irp);
 
 static NTSTATUS start_pdo(struct device_object *pdo)
 {
@@ -33,12 +37,10 @@ static NTSTATUS start_pdo(struct device_object *pdo)
 	if (ntoskernel_init_device(wd))
 		EXIT1(return STATUS_FAILURE);
 	if (wrap_is_usb_bus(wd->dev_bus)) {
-#ifdef CONFIG_USB
 		if (usb_init_device(wd)) {
 			ntoskernel_exit_device(wd);
 			EXIT1(return STATUS_FAILURE);
 		}
-#endif
 		EXIT1(return STATUS_SUCCESS);
 	}
 	if (!wrap_is_pci_bus(wd->dev_bus))
@@ -59,8 +61,8 @@ static NTSTATUS start_pdo(struct device_object *pdo)
 	/* 64-bit broadcom driver doesn't work if DMA is allocated
 	 * from over 1GB */
 	if (wd->vendor == 0x14e4) {
-		if (pci_set_dma_mask(pdev, DMA_30BIT_MASK) ||
-		    pci_set_consistent_dma_mask(pdev, DMA_30BIT_MASK))
+		if (pci_set_dma_mask(pdev, DMA_BIT_MASK(30)) ||
+		    pci_set_consistent_dma_mask(pdev, DMA_BIT_MASK(30)))
 			WARNING("couldn't set DMA mask; this driver "
 				"may not work with more than 1GB RAM");
 	}
@@ -176,18 +178,15 @@ static void remove_pdo(struct device_object *pdo)
 		wd->pci.pdev = NULL;
 		pci_set_drvdata(pdev, NULL);
 	} else if (wrap_is_usb_bus(wd->dev_bus)) {
-#ifdef CONFIG_USB
 		usb_exit_device(wd);
-#endif
 	}
-	if (wd->resource_list)
-		kfree(wd->resource_list);
+	kfree(wd->resource_list);
 	wd->resource_list = NULL;
 	return;
 }
 
-wstdcall NTSTATUS IoSendIrpTopDev(struct device_object *dev_obj, ULONG major_fn,
-				 ULONG minor_fn, struct io_stack_location *sl)
+static NTSTATUS IoSendIrpTopDev(struct device_object *dev_obj, ULONG major_fn,
+				ULONG minor_fn, struct io_stack_location *sl)
 {
 	NTSTATUS status;
 	struct nt_event event;
@@ -216,22 +215,22 @@ wstdcall NTSTATUS IoSendIrpTopDev(struct device_object *dev_obj, ULONG major_fn,
 }
 
 wstdcall NTSTATUS pdoDispatchDeviceControl(struct device_object *pdo,
-					   struct  irp *irp)
+					   struct irp *irp)
 {
-	struct io_stack_location *irp_sl;
 	NTSTATUS status;
+	struct wrap_device *wd = pdo->reserved;
 
 	DUMP_IRP(irp);
-	irp_sl = IoGetCurrentIrpStackLocation(irp);
-#ifdef CONFIG_USB
-	status = wrap_submit_irp(pdo, irp);
-	IOTRACE("status: %08X", status);
-	if (status != STATUS_PENDING)
+	(void)wd;
+	if (wrap_is_usb_bus(wd->dev_bus)) {
+		status = wrap_submit_irp(pdo, irp);
+		IOTRACE("status: %08X", status);
+		if (status != STATUS_PENDING)
+			IoCompleteRequest(irp, IO_NO_INCREMENT);
+	} else {
+		status = irp->io_status.status = STATUS_NOT_IMPLEMENTED;
 		IoCompleteRequest(irp, IO_NO_INCREMENT);
-#else
-	status = irp->io_status.status = STATUS_NOT_IMPLEMENTED;
-	IoCompleteRequest(irp, IO_NO_INCREMENT);
-#endif
+	}
 	IOEXIT(return status);
 }
 WIN_FUNC_DECL(pdoDispatchDeviceControl,2)
@@ -239,15 +238,11 @@ WIN_FUNC_DECL(pdoDispatchDeviceControl,2)
 wstdcall NTSTATUS pdoDispatchPnp(struct device_object *pdo, struct irp *irp)
 {
 	struct io_stack_location *irp_sl;
-	struct wrap_device *wd;
 	NTSTATUS status;
-#ifdef CONFIG_USB
-	struct usbd_bus_interface_usbdi *usb_intf;
-#endif
+	struct wrap_device *wd = pdo->reserved;
 
 	irp_sl = IoGetCurrentIrpStackLocation(irp);
 	TRACE2("%p %d:%d", pdo, irp_sl->major_fn, irp_sl->minor_fn);
-	wd = pdo->reserved;
 	switch (irp_sl->minor_fn) {
 	case IRP_MN_START_DEVICE:
 		status = start_pdo(pdo);
@@ -262,36 +257,10 @@ wstdcall NTSTATUS pdoDispatchPnp(struct device_object *pdo, struct irp *irp)
 		status = STATUS_SUCCESS;
 		break;
 	case IRP_MN_QUERY_INTERFACE:
-#ifdef CONFIG_USB
-		if (!wrap_is_usb_bus(wd->dev_bus)) {
+		if (wrap_is_usb_bus(wd->dev_bus))
+			status = usb_query_interface(wd, irp_sl);
+		else
 			status = STATUS_NOT_IMPLEMENTED;
-			break;
-		}
-		TRACE2("type: %x, size: %d, version: %d",
-		       irp_sl->params.query_intf.type->data1,
-		       irp_sl->params.query_intf.size,
-		       irp_sl->params.query_intf.version);
-		usb_intf = (struct usbd_bus_interface_usbdi *)
-			irp_sl->params.query_intf.intf;
-		usb_intf->Context = wd;
-		usb_intf->InterfaceReference = USBD_InterfaceReference;
-		usb_intf->InterfaceDereference = USBD_InterfaceDereference;
-		usb_intf->GetUSBDIVersion = USBD_InterfaceGetUSBDIVersion;
-		usb_intf->QueryBusTime = USBD_InterfaceQueryBusTime;
-		usb_intf->SubmitIsoOutUrb = USBD_InterfaceSubmitIsoOutUrb;
-		usb_intf->QueryBusInformation =
-			USBD_InterfaceQueryBusInformation;
-		if (irp_sl->params.query_intf.version >=
-		    USB_BUSIF_USBDI_VERSION_1)
-			usb_intf->IsDeviceHighSpeed =
-				USBD_InterfaceIsDeviceHighSpeed;
-		if (irp_sl->params.query_intf.version >=
-		    USB_BUSIF_USBDI_VERSION_2)
-			usb_intf->LogEntry = USBD_InterfaceLogEntry;
-		status = STATUS_SUCCESS;
-#else
-		status = STATUS_NOT_IMPLEMENTED;
-#endif
 		break;
 	default:
 		TRACE2("fn %d not implemented", irp_sl->minor_fn);
@@ -332,11 +301,7 @@ wstdcall NTSTATUS pdoDispatchPower(struct device_object *pdo, struct irp *irp)
 			TRACE2("resuming %p", wd);
 			if (wrap_is_pci_bus(wd->dev_bus)) {
 				pdev = wd->pci.pdev;
-#if LINUX_VERSION_CODE > KERNEL_VERSION(2,6,9)
 				pci_restore_state(pdev);
-#else
-				pci_restore_state(pdev, wd->pci.pci_state);
-#endif
 				if (wd->pci.wake_state == PowerDeviceD3) {
 					pci_enable_wake(wd->pci.pdev,
 							PCI_D3hot, 0);
@@ -344,20 +309,14 @@ wstdcall NTSTATUS pdoDispatchPower(struct device_object *pdo, struct irp *irp)
 							PCI_D3cold, 0);
 				}
 				pci_set_power_state(pdev, PCI_D0);
-			} else { // usb device
-#ifdef CONFIG_USB
+			} else if (wrap_is_usb_bus(wd->dev_bus)) {
 				wrap_resume_urbs(wd);
-#endif
 			}
 		} else {
 			TRACE2("suspending device %p", wd);
 			if (wrap_is_pci_bus(wd->dev_bus)) {
 				pdev = wd->pci.pdev;
-#if LINUX_VERSION_CODE > KERNEL_VERSION(2,6,9)
 				pci_save_state(pdev);
-#else
-				pci_save_state(pdev, wd->pci.pci_state);
-#endif
 				TRACE2("%d", wd->pci.wake_state);
 				if (wd->pci.wake_state == PowerDeviceD3) {
 					pci_enable_wake(wd->pci.pdev,
@@ -366,10 +325,8 @@ wstdcall NTSTATUS pdoDispatchPower(struct device_object *pdo, struct irp *irp)
 							PCI_D3cold, 1);
 				}
 				pci_set_power_state(pdev, PCI_D3hot);
-			} else { // usb device
-#ifdef CONFIG_USB
+			} else if (wrap_is_usb_bus(wd->dev_bus)) {
 				wrap_suspend_urbs(wd);
-#endif
 			}
 		}
 		status = STATUS_SUCCESS;
@@ -388,8 +345,8 @@ wstdcall NTSTATUS pdoDispatchPower(struct device_object *pdo, struct irp *irp)
 }
 WIN_FUNC_DECL(pdoDispatchPower,2)
 
-NTSTATUS pnp_set_device_power_state(struct wrap_device *wd,
-				    enum device_power_state state)
+static NTSTATUS pnp_set_device_power_state(struct wrap_device *wd,
+					   enum device_power_state state)
 {
 	NTSTATUS status;
 	struct device_object *pdo;
@@ -415,7 +372,7 @@ NTSTATUS pnp_set_device_power_state(struct wrap_device *wd,
 	EXIT1(return status);
 }
 
-NTSTATUS pnp_start_device(struct wrap_device *wd)
+static NTSTATUS pnp_start_device(struct wrap_device *wd)
 {
 	struct device_object *fdo;
 	struct device_object *pdo;
@@ -432,15 +389,15 @@ NTSTATUS pnp_start_device(struct wrap_device *wd)
 		wd->resource_list;
 	status = IoSendIrpTopDev(pdo, IRP_MJ_PNP, IRP_MN_START_DEVICE, &irp_sl);
 	fdo = IoGetAttachedDevice(pdo);
-	if (status == STATUS_SUCCESS)
-		fdo->drv_obj->drv_ext->count++;
-	else
+	fdo->drv_obj->drv_ext->count++;
+	if (status != STATUS_SUCCESS)
 		WARNING("Windows driver couldn't initialize the device (%08X)",
 			status);
 	EXIT1(return status);
 }
 
-NTSTATUS pnp_stop_device(struct wrap_device *wd)
+#if 0
+static NTSTATUS pnp_stop_device(struct wrap_device *wd)
 {
 	struct device_object *pdo;
 	NTSTATUS status;
@@ -458,8 +415,9 @@ NTSTATUS pnp_stop_device(struct wrap_device *wd)
 		WARNING("status: %08X", status);
 	EXIT2(return status);
 }
+#endif
 
-NTSTATUS pnp_remove_device(struct wrap_device *wd)
+static NTSTATUS pnp_remove_device(struct wrap_device *wd)
 {
 	struct device_object *pdo, *fdo;
 	struct driver_object *fdo_drv_obj;
@@ -483,7 +441,7 @@ NTSTATUS pnp_remove_device(struct wrap_device *wd)
 	if (status == STATUS_SUCCESS)
 		fdo_drv_obj->drv_ext->count--;
 	TRACE1("count: %d", fdo_drv_obj->drv_ext->count);
-	if (fdo_drv_obj->drv_ext->count < 0)
+	if ((LONG)fdo_drv_obj->drv_ext->count < 0)
 		WARNING("wrong count: %d", fdo_drv_obj->drv_ext->count);
 	if (fdo_drv_obj->drv_ext->count == 0) {
 		struct wrap_driver *wrap_driver;
@@ -494,10 +452,9 @@ NTSTATUS pnp_remove_device(struct wrap_device *wd)
 		if (fdo_drv_obj->unload)
 			LIN2WIN1(fdo_drv_obj->unload, fdo_drv_obj);
 		if (wrap_driver) {
-			if (down_interruptible(&loader_mutex))
-				WARNING("couldn't obtain loader_mutex");
+			mutex_lock(&loader_mutex);
 			unload_wrap_driver(wrap_driver);
-			up(&loader_mutex);
+			mutex_unlock(&loader_mutex);
 		} else
 			ERROR("couldn't get wrap_driver");
 		ObDereferenceObject(fdo_drv_obj);
@@ -512,7 +469,7 @@ WIN_FUNC_DECL(IoInvalidDeviceRequest,2)
 static struct device_object *alloc_pdo(struct driver_object *drv_obj)
 {
 	struct device_object *pdo;
-	NTSTATUS status ;
+	NTSTATUS status;
 	int i;
 	struct ansi_string ansi_name;
 	struct unicode_string unicode_name;
@@ -654,23 +611,13 @@ int wrap_pnp_resume_pci_device(struct pci_dev *pdev)
 	return pnp_set_device_power_state(wd, PowerDeviceD0);
 }
 
-#ifdef CONFIG_USB
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
+#ifdef ENABLE_USB
 int wrap_pnp_start_usb_device(struct usb_interface *intf,
 			      const struct usb_device_id *usb_id)
-#else
-void *wrap_pnp_start_usb_device(struct usb_device *udev,
-				unsigned int ifnum,
-				const struct usb_device_id *usb_id)
-#endif
 {
 	struct wrap_device *wd;
 	int ret;
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
 	struct usb_device *udev = interface_to_usbdev(intf);
-#else
-	struct usb_interface *intf = usb_ifnum_to_if(udev, ifnum);
-#endif
 	ENTER1("%04x, %04x, %04x", udev->descriptor.idVendor,
 	       udev->descriptor.idProduct, udev->descriptor.bDeviceClass);
 
@@ -686,8 +633,8 @@ void *wrap_pnp_start_usb_device(struct usb_device *udev,
 		struct load_device load_device;
 
 		load_device.bus = WRAP_USB_BUS;
-		load_device.vendor = udev->descriptor.idVendor;
-		load_device.device = udev->descriptor.idProduct;
+		load_device.vendor = le16_to_cpu(udev->descriptor.idVendor);
+		load_device.device = le16_to_cpu(udev->descriptor.idProduct);
 		load_device.subvendor = 0;
 		load_device.subdevice = 0;
 		wd = load_wrap_device(&load_device);
@@ -707,20 +654,12 @@ void *wrap_pnp_start_usb_device(struct usb_device *udev,
 	}
 
 	TRACE2("ret: %d", ret);
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
 	if (ret)
 		EXIT1(return ret);
 	else
 		return 0;
-#else
-	if (ret)
-		return NULL;
-	else
-		return wd;
-#endif
 }
 
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
 void __devexit wrap_pnp_remove_usb_device(struct usb_interface *intf)
 {
 	struct wrap_device *wd;
@@ -737,13 +676,11 @@ void __devexit wrap_pnp_remove_usb_device(struct usb_interface *intf)
 int wrap_pnp_suspend_usb_device(struct usb_interface *intf, pm_message_t state)
 {
 	struct wrap_device *wd;
-	struct device_object *pdo;
 
 	wd = usb_get_intfdata(intf);
 	ENTER1("%p, %p", intf, wd);
 	if (!wd)
 		EXIT1(return 0);
-	pdo = wd->pdo;
 	if (pnp_set_device_power_state(wd, PowerDeviceD3))
 		return -1;
 	return 0;
@@ -761,20 +698,4 @@ int wrap_pnp_resume_usb_device(struct usb_interface *intf)
 	return 0;
 }
 
-#else
-
-void __devexit wrap_pnp_remove_usb_device(struct usb_device *udev, void *ptr)
-{
-	struct wrap_device *wd = ptr;
-	struct usb_interface *intf;
-
-	ENTER1("%p, %p", udev, wd);
-	if (wd == NULL)
-		EXIT1(return);
-	intf = wd->usb.intf;
-	wd->usb.intf = NULL;
-	pnp_remove_device(wd);
-}
-#endif
-
 #endif // USB
