@@ -47,25 +47,25 @@ static int procfs_read_ndis_stats(char *page, char **start, off_t off,
 	res = mp_query(wnd, OID_802_11_STATISTICS, &stats, sizeof(stats));
 	if (!res) {
 
-		p += sprintf(p, "tx_frames=%Lu\n", stats.tx_frag);
-		p += sprintf(p, "tx_multicast_frames=%Lu\n",
+		p += sprintf(p, "tx_frames=%llu\n", stats.tx_frag);
+		p += sprintf(p, "tx_multicast_frames=%llu\n",
 			     stats.tx_multi_frag);
-		p += sprintf(p, "tx_failed=%Lu\n", stats.failed);
-		p += sprintf(p, "tx_retry=%Lu\n", stats.retry);
-		p += sprintf(p, "tx_multi_rerty=%Lu\n", stats.multi_retry);
-		p += sprintf(p, "tx_rtss_success=%Lu\n", stats.rtss_succ);
-		p += sprintf(p, "tx_rtss_fail=%Lu\n", stats.rtss_fail);
-		p += sprintf(p, "ack_fail=%Lu\n", stats.ack_fail);
-		p += sprintf(p, "frame_duplicates=%Lu\n", stats.frame_dup);
-		p += sprintf(p, "rx_frames=%Lu\n", stats.rx_frag);
-		p += sprintf(p, "rx_multicast_frames=%Lu\n",
+		p += sprintf(p, "tx_failed=%llu\n", stats.failed);
+		p += sprintf(p, "tx_retry=%llu\n", stats.retry);
+		p += sprintf(p, "tx_multi_retry=%llu\n", stats.multi_retry);
+		p += sprintf(p, "tx_rtss_success=%llu\n", stats.rtss_succ);
+		p += sprintf(p, "tx_rtss_fail=%llu\n", stats.rtss_fail);
+		p += sprintf(p, "ack_fail=%llu\n", stats.ack_fail);
+		p += sprintf(p, "frame_duplicates=%llu\n", stats.frame_dup);
+		p += sprintf(p, "rx_frames=%llu\n", stats.rx_frag);
+		p += sprintf(p, "rx_multicast_frames=%llu\n",
 			     stats.rx_multi_frag);
-		p += sprintf(p, "fcs_errors=%Lu\n", stats.fcs_err);
+		p += sprintf(p, "fcs_errors=%llu\n", stats.fcs_err);
 	}
 
 	if (p - page > count) {
-		ERROR("wrote %lu bytes (limit is %u)\n",
-		      (unsigned long)(p - page), count);
+		ERROR("wrote %td bytes (limit is %u)\n",
+		      p - page, count);
 		*eof = 1;
 	}
 
@@ -92,7 +92,7 @@ static int procfs_read_ndis_encr(char *page, char **start, off_t off,
 	if (res)
 		memset(ap_address, 0, ETH_ALEN);
 	p += sprintf(p, "ap_address=%2.2X", ap_address[0]);
-	for (i = 1 ; i < ETH_ALEN ; i++)
+	for (i = 1; i < ETH_ALEN; i++)
 		p += sprintf(p, ":%2.2X", ap_address[i]);
 	p += sprintf(p, "\n");
 
@@ -122,8 +122,8 @@ static int procfs_read_ndis_encr(char *page, char **start, off_t off,
 		     "adhoc" : (infra_mode == Ndis802_11Infrastructure) ?
 		     "managed" : "auto");
 	if (p - page > count) {
-		WARNING("wrote %lu bytes (limit is %u)",
-			(unsigned long)(p - page), count);
+		WARNING("wrote %td bytes (limit is %u)",
+			p - page, count);
 		*eof = 1;
 	}
 
@@ -136,7 +136,7 @@ static int procfs_read_ndis_hw(char *page, char **start, off_t off,
 	char *p = page;
 	struct ndis_device *wnd = (struct ndis_device *)data;
 	struct ndis_configuration config;
-	unsigned int power_mode;
+	enum ndis_power power_mode;
 	NDIS_STATUS res;
 	ndis_tx_power_level tx_power;
 	ULONG bit_rate;
@@ -155,8 +155,7 @@ static int procfs_read_ndis_hw(char *page, char **start, off_t off,
 	}
 
 	res = mp_query_int(wnd, OID_GEN_HARDWARE_STATUS, &n);
-	if (res == NDIS_STATUS_SUCCESS &&
-	    n >= 0 && n < sizeof(hw_status) / sizeof(hw_status[0]))
+	if (res == NDIS_STATUS_SUCCESS && n >= 0 && n < ARRAY_SIZE(hw_status))
 		p += sprintf(p, "status=%s\n", hw_status[n]);
 
 	res = mp_query(wnd, OID_802_3_CURRENT_ADDRESS, mac, sizeof(mac));
@@ -167,7 +166,7 @@ static int procfs_read_ndis_hw(char *page, char **start, off_t off,
 		p += sprintf(p, "beacon_period=%u msec\n",
 			     config.beacon_period);
 		p += sprintf(p, "atim_window=%u msec\n", config.atim_window);
-		p += sprintf(p, "frequency=%u kHZ\n", config.ds_config);
+		p += sprintf(p, "frequency=%u kHz\n", config.ds_config);
 		p += sprintf(p, "hop_pattern=%u\n",
 			     config.fh_config.hop_pattern);
 		p += sprintf(p, "hop_set=%u\n",
@@ -243,8 +242,8 @@ static int procfs_read_ndis_hw(char *page, char **start, off_t off,
 		p += sprintf(p, "packet_filter: 0x%08x\n", packet_filter);
 	}
 	if (p - page > count) {
-		WARNING("wrote %lu bytes (limit is %u)",
-			(unsigned long)(p - page), count);
+		WARNING("wrote %td bytes (limit is %u)",
+			p - page, count);
 		*eof = 1;
 	}
 
@@ -265,7 +264,7 @@ static int procfs_read_ndis_settings(char *page, char **start, off_t off,
 
 	p += sprintf(p, "hangcheck_interval=%d\n",
 		     hangcheck_interval == 0 ?
-		     (int)(wnd->hangcheck_interval / HZ) : -1);
+		     (wnd->hangcheck_interval / HZ) : -1);
 
 	list_for_each_entry(setting, &wnd->wd->settings, list) {
 		p += sprintf(p, "%s=%s\n", setting->name, setting->value);
@@ -316,27 +315,21 @@ static int procfs_write_ndis_settings(struct file *file, const char __user *buf,
 		i = simple_strtol(p, NULL, 10);
 		if (i <= 0 || i > 3)
 			return -EINVAL;
+		i = -1;
 		if (wrap_is_pci_bus(wnd->wd->dev_bus))
 			i = wrap_pnp_suspend_pci_device(wnd->wd->pci.pdev,
 							PMSG_SUSPEND);
-		else
-#ifdef ENABLE_USB
+		else if (wrap_is_usb_bus(wnd->wd->dev_bus))
 			i = wrap_pnp_suspend_usb_device(wnd->wd->usb.intf,
 							PMSG_SUSPEND);
-#else
-		i = -1;
-#endif
 		if (i)
 			return -EINVAL;
 	} else if (!strcmp(setting, "resume")) {
+		i = -1;
 		if (wrap_is_pci_bus(wnd->wd->dev_bus))
 			i = wrap_pnp_resume_pci_device(wnd->wd->pci.pdev);
-		else
-#ifdef ENABLE_USB
+		else if (wrap_is_usb_bus(wnd->wd->dev_bus))
 			i = wrap_pnp_resume_usb_device(wnd->wd->usb.intf);
-#else
-		i = -1;
-#endif
 		if (i)
 			return -EINVAL;
 	} else if (!strcmp(setting, "stats_enabled")) {
@@ -395,10 +388,10 @@ int wrap_procfs_add_ndis_device(struct ndis_device *wnd)
 		return -ENOMEM;
 
 	if (wnd->procfs_iface) {
-		ERROR("%s already registered?", wnd->netdev_name);
+		ERROR("%s already registered?", wnd->procfs_iface->name);
 		return -EINVAL;
 	}
-	wnd->procfs_iface = proc_mkdir(wnd->netdev_name, wrap_procfs_entry);
+	wnd->procfs_iface = proc_mkdir(wnd->net_dev->name, wrap_procfs_entry);
 	if (wnd->procfs_iface == NULL) {
 		ERROR("couldn't create proc directory");
 		return -ENOMEM;
@@ -464,7 +457,7 @@ err_encr:
 err_stats:
 	remove_proc_entry("hw", wnd->procfs_iface);
 err_hw:
-	remove_proc_entry(wnd->netdev_name, wrap_procfs_entry);
+	remove_proc_entry(wnd->procfs_iface->name, wrap_procfs_entry);
 	wnd->procfs_iface = NULL;
 	return -ENOMEM;
 }
@@ -480,25 +473,26 @@ void wrap_procfs_remove_ndis_device(struct ndis_device *wnd)
 	remove_proc_entry("encr", procfs_iface);
 	remove_proc_entry("settings", procfs_iface);
 	if (wrap_procfs_entry)
-		remove_proc_entry(wnd->netdev_name, wrap_procfs_entry);
+		remove_proc_entry(procfs_iface->name, wrap_procfs_entry);
 }
 
 static int procfs_read_debug(char *page, char **start, off_t off,
 			     int count, int *eof, void *data)
 {
 	char *p = page;
+#if ALLOC_DEBUG
 	enum alloc_type type;
+#endif
 
 	if (off != 0) {
 		*eof = 1;
 		return 0;
 	}
 	p += sprintf(p, "%d\n", debug);
-	type = 0;
-#ifdef ALLOC_DEBUG
+#if ALLOC_DEBUG
 	for (type = 0; type < ALLOC_TYPE_MAX; type++)
-		p += sprintf(p, "total size of allocations in %d: %d\n",
-			     type, alloc_size(type));
+		p += sprintf(p, "total size of allocations in %s: %d\n",
+			     alloc_type_name[type], alloc_size(type));
 #endif
 	return p - page;
 }
@@ -550,7 +544,7 @@ int wrap_procfs_init(void)
 	} else {
 		procfs_entry->uid = proc_uid;
 		procfs_entry->gid = proc_gid;
-		procfs_entry->read_proc  = procfs_read_debug;
+		procfs_entry->read_proc = procfs_read_debug;
 		procfs_entry->write_proc = procfs_write_debug;
 	}
 	return 0;
