@@ -15,17 +15,38 @@
 
 #include "ntoskernel.h"
 
+struct workqueue_thread {
+	spinlock_t lock;
+	struct task_struct *task;
+	struct completion *completion;
+	char name[16];
+	int pid;
+	/* whether any work_structs pending? <0 implies quit */
+	s8 pending;
+	/* list of work_structs pending */
+	struct list_head work_list;
+};
+
 struct workq_thread_data {
-	workqueue_struct_t *workq;
+	struct workqueue_struct *workq;
 	int index;
 };
 
+struct wrap_workqueue_struct {
+	u8 singlethread;
+	u8 qon;
+	int num_cpus;
+	struct workqueue_thread threads[0];
+};
+
+static void wrap_destroy_wq_on(struct workqueue_struct *workq, int cpu);
+
 static int workq_thread(void *data)
 {
 	struct workq_thread_data *thread_data = data;
 	struct workqueue_thread *thread;
-	workqueue_struct_t *workq;
-	work_struct_t *work;
+	struct workqueue_struct *workq;
+	struct work_struct *work;
 
 	workq = thread_data->workq;
 	thread = &workq->threads[thread_data->index];
@@ -70,7 +91,7 @@ static int workq_thread(void *data)
 				break;
 			}
 			entry = thread->work_list.next;
-			work = list_entry(entry, work_struct_t, list);
+			work = list_entry(entry, struct work_struct, list);
 			if (xchg(&work->thread, NULL))
 				list_del(entry);
 			else
@@ -90,8 +111,8 @@ out:
 	return 0;
 }
 
-int wrap_queue_work_on(workqueue_struct_t *workq, work_struct_t *work,
-		       int cpu)
+static int wrap_queue_work_on(struct workqueue_struct *workq,
+			      struct work_struct *work, int cpu)
 {
 	struct workqueue_thread *thread = &workq->threads[cpu];
 	unsigned long flags;
@@ -115,13 +136,13 @@ int wrap_queue_work_on(workqueue_struct_t *workq, work_struct_t *work,
 	return ret;
 }
 
-int wrap_queue_work(workqueue_struct_t *workq, work_struct_t *work)
+int wrap_queue_work(struct workqueue_struct *workq, struct work_struct *work)
 {
-	if (NR_CPUS == 1 || workq->singlethread)
+	if (num_online_cpus() == 1 || workq->singlethread)
 		return wrap_queue_work_on(workq, work, 0);
 	else {
 		typeof(workq->qon) qon;
-		/* work is queued on threads in a round-robbin fashion */
+		/* work is queued on threads in a round-robin fashion */
 		do {
 			qon = workq->qon % workq->num_cpus;
 			atomic_inc_var(workq->qon);
@@ -130,7 +151,7 @@ int wrap_queue_work(workqueue_struct_t *workq, work_struct_t *work)
 	}
 }
 
-void wrap_cancel_work(work_struct_t *work)
+void wrap_cancel_work(struct work_struct *work)
 {
 	struct workqueue_thread *thread;
 	unsigned long flags;
@@ -144,23 +165,23 @@ void wrap_cancel_work(work_struct_t *work)
 	}
 }
 
-workqueue_struct_t *wrap_create_wq(const char *name, u8 singlethread, u8 freeze)
+struct workqueue_struct *wrap_create_wq(const char *name, u8 singlethread,
+					u8 freeze)
 {
 	struct completion started;
-	workqueue_struct_t *workq;
+	struct workqueue_struct *workq;
 	int i, n;
 
 	if (singlethread)
 		n = 1;
 	else
-		n = NR_CPUS;
-	workq = kmalloc(sizeof(*workq) + n * sizeof(workq->threads[0]),
+		n = num_online_cpus();
+	workq = kzalloc(sizeof(*workq) + n * sizeof(workq->threads[0]),
 			GFP_KERNEL);
 	if (!workq) {
 		WARNING("couldn't allocate memory");
 		return NULL;
 	}
-	memset(workq, 0, sizeof(*workq) + n * sizeof(workq->threads[0]));
 	WORKTRACE("%p", workq);
 	workq->singlethread = singlethread;
 	init_completion(&started);
@@ -201,7 +222,7 @@ workqueue_struct_t *wrap_create_wq(const char *name, u8 singlethread, u8 freeze)
 	return workq;
 }
 
-void wrap_flush_wq_on(workqueue_struct_t *workq, int cpu)
+static void wrap_flush_wq_on(struct workqueue_struct *workq, int cpu)
 {
 	struct workqueue_thread *thread = &workq->threads[cpu];
 	struct completion done;
@@ -215,7 +236,7 @@ void wrap_flush_wq_on(workqueue_struct_t *workq, int cpu)
 	return;
 }
 
-void wrap_flush_wq(workqueue_struct_t *workq)
+void wrap_flush_wq(struct workqueue_struct *workq)
 {
 	int i, n;
 
@@ -223,12 +244,12 @@ void wrap_flush_wq(workqueue_struct_t *workq)
 	if (workq->singlethread)
 		n = 1;
 	else
-		n = NR_CPUS;
+		n = num_online_cpus();
 	for (i = 0; i < n; i++)
 		wrap_flush_wq_on(workq, i);
 }
 
-void wrap_destroy_wq_on(workqueue_struct_t *workq, int cpu)
+static void wrap_destroy_wq_on(struct workqueue_struct *workq, int cpu)
 {
 	struct workqueue_thread *thread = &workq->threads[cpu];
 
@@ -243,7 +264,7 @@ void wrap_destroy_wq_on(workqueue_struct_t *workq, int cpu)
 	}
 }
 
-void wrap_destroy_wq(workqueue_struct_t *workq)
+void wrap_destroy_wq(struct workqueue_struct *workq)
 {
 	int i, n;
 
@@ -251,7 +272,7 @@ void wrap_destroy_wq(workqueue_struct_t *workq)
 	if (workq->singlethread)
 		n = 1;
 	else
-		n = NR_CPUS;
+		n = num_online_cpus();
 	for (i = 0; i < n; i++)
 		wrap_destroy_wq_on(workq, i);
 	kfree(workq);
