@@ -13,16 +13,15 @@
  *
  */
 
+#include <linux/inetdevice.h>
+#include <linux/ip.h>
+#include <linux/in.h>
+#include <linux/proc_fs.h>
 #include "ndis.h"
 #include "iw_ndis.h"
 #include "pnp.h"
 #include "loader.h"
 #include "wrapndis.h"
-#include <linux/inetdevice.h>
-#include <linux/ip.h>
-#include <linux/tcp.h>
-#include <linux/udp.h>
-#include <linux/in.h>
 #include "wrapper.h"
 
 /* Functions callable from the NDIS driver */
@@ -31,8 +30,7 @@ wstdcall NTSTATUS NdisDispatchDeviceControl(struct device_object *fdo,
 wstdcall NTSTATUS NdisDispatchPnp(struct device_object *fdo, struct irp *irp);
 wstdcall NTSTATUS NdisDispatchPower(struct device_object *fdo, struct irp *irp);
 
-workqueue_struct_t *wrapndis_wq;
-static struct nt_thread *wrapndis_worker_thread;
+struct workqueue_struct *wrapndis_wq;
 
 static int set_packet_filter(struct ndis_device *wnd,
 			     ULONG packet_filter);
@@ -53,12 +51,8 @@ NDIS_STATUS mp_reset(struct ndis_device *wnd)
 	KIRQL irql;
 
 	ENTER2("wnd: %p", wnd);
-	if (down_interruptible(&wnd->tx_ring_mutex))
-		EXIT3(return NDIS_STATUS_FAILURE);
-	if (down_interruptible(&wnd->ndis_req_mutex)) {
-		up(&wnd->tx_ring_mutex);
-		EXIT3(return NDIS_STATUS_FAILURE);
-	}
+	mutex_lock(&wnd->tx_ring_mutex);
+	mutex_lock(&wnd->ndis_req_mutex);
 	mp = &wnd->wd->driver->ndis_driver->mp;
 	prepare_wait_condition(wnd->ndis_req_task, wnd->ndis_req_done, 0);
 	WARNING("%s is being reset", wnd->net_dev->name);
@@ -79,12 +73,12 @@ NDIS_STATUS mp_reset(struct ndis_device *wnd)
 		}
 		TRACE2("%08X, %08X", res, reset_address);
 	}
-	up(&wnd->ndis_req_mutex);
+	mutex_unlock(&wnd->ndis_req_mutex);
 	if (res == NDIS_STATUS_SUCCESS && reset_address) {
 		set_packet_filter(wnd, wnd->packet_filter);
 		set_multicast_list(wnd);
 	}
-	up(&wnd->tx_ring_mutex);
+	mutex_unlock(&wnd->tx_ring_mutex);
 	EXIT3(return res);
 }
 
@@ -98,8 +92,7 @@ NDIS_STATUS mp_request(enum ndis_request_type request,
 	struct miniport *mp;
 	KIRQL irql;
 
-	if (down_interruptible(&wnd->ndis_req_mutex))
-		EXIT3(return NDIS_STATUS_FAILURE);
+	mutex_lock(&wnd->ndis_req_mutex);
 	if (!written)
 		written = &w;
 	if (!needed)
@@ -135,7 +128,7 @@ NDIS_STATUS mp_request(enum ndis_request_type request,
 			res = wnd->ndis_req_status;
 		TRACE2("%08X, %08X", res, oid);
 	}
-	up(&wnd->ndis_req_mutex);
+	mutex_unlock(&wnd->ndis_req_mutex);
 	DBG_BLOCK(2) {
 		if (res || needed)
 			TRACE2("%08X, %d, %d, %d", res, buflen, *written,
@@ -168,7 +161,7 @@ static NDIS_STATUS mp_pnp_event(struct ndis_device *wnd,
 		       (wnd->attributes & NDIS_ATTRIBUTE_SURPRISE_REMOVE_OK),
 		       mp->pnp_event_notify);
 		if ((wnd->attributes & NDIS_ATTRIBUTE_SURPRISE_REMOVE_OK) &&
-		    !test_bit(HW_PRESENT, &wnd->wd->hw_status) &&
+		    !test_bit(HW_DISABLED, &wnd->wd->hw_status) &&
 		    mp->pnp_event_notify) {
 			TRACE1("calling surprise_removed");
 			LIN2WIN4(mp->pnp_event_notify, wnd->nmb->mp_ctx,
@@ -183,7 +176,7 @@ static NDIS_STATUS mp_pnp_event(struct ndis_device *wnd,
 			power_profile = NdisPowerProfileAcOnLine;
 		LIN2WIN4(mp->pnp_event_notify, wnd->nmb->mp_ctx,
 			 NdisDevicePnPEventPowerProfileChanged,
-			 &power_profile, (ULONG)sizeof(power_profile));
+			 &power_profile, sizeof(power_profile));
 		return NDIS_STATUS_SUCCESS;
 	default:
 		WARNING("event %d not yet implemented", event);
@@ -212,8 +205,7 @@ static NDIS_STATUS mp_init(struct ndis_device *wnd)
 	}
 	mp = &wnd->wd->driver->ndis_driver->mp;
 	status = LIN2WIN6(mp->init, &error_status, &medium_index, medium_array,
-			  sizeof(medium_array) / sizeof(medium_array[0]),
-			  wnd->nmb, wnd->nmb);
+			  ARRAY_SIZE(medium_array), wnd->nmb, wnd->nmb);
 	TRACE1("init returns: %08X, irql: %d", status, current_irql());
 	if (status != NDIS_STATUS_SUCCESS) {
 		WARNING("couldn't initialize device: %08X", status);
@@ -266,10 +258,9 @@ static void mp_halt(struct ndis_device *wnd)
 	del_iw_stats_timer(wnd);
 	if (wnd->physical_medium == NdisPhysicalMediumWirelessLan &&
 	    wrap_is_pci_bus(wnd->wd->dev_bus)) {
-		up(&wnd->ndis_req_mutex);
+		mutex_unlock(&wnd->ndis_req_mutex);
 		disassociate(wnd, 0);
-		if (down_interruptible(&wnd->ndis_req_mutex))
-			WARNING("couldn't obtain ndis_req_mutex");
+		mutex_lock(&wnd->ndis_req_mutex);
 	}
 	mp = &wnd->wd->driver->ndis_driver->mp;
 	TRACE1("halt: %p", mp->mp_halt);
@@ -278,7 +269,7 @@ static void mp_halt(struct ndis_device *wnd)
 	 * halt, deregister it now */
 	if (wnd->mp_interrupt)
 		NdisMDeregisterInterrupt(wnd->mp_interrupt);
-	/* cancel any timers left by bugyy windows driver; also free
+	/* cancel any timers left by buggy windows driver; also free
 	 * the memory for timers */
 	while (1) {
 		struct nt_slist *slist;
@@ -313,7 +304,7 @@ static NDIS_STATUS mp_set_power_state(struct ndis_device *wnd,
 	TRACE1("%d", state);
 	if (state == NdisDeviceStateD0) {
 		status = NDIS_STATUS_SUCCESS;
-		up(&wnd->ndis_req_mutex);
+		mutex_unlock(&wnd->ndis_req_mutex);
 		if (test_and_clear_bit(HW_HALTED, &wnd->wd->hw_status)) {
 			status = mp_init(wnd);
 			if (status == NDIS_STATUS_SUCCESS) {
@@ -335,7 +326,7 @@ static NDIS_STATUS mp_set_power_state(struct ndis_device *wnd,
 			pci_enable_wake(wnd->wd->pci.pdev, PCI_D3cold, 0);
 		}
 		if (status == NDIS_STATUS_SUCCESS) {
-			up(&wnd->tx_ring_mutex);
+			mutex_unlock(&wnd->tx_ring_mutex);
 			netif_device_attach(wnd->net_dev);
 			hangcheck_add(wnd);
 			add_iw_stats_timer(wnd);
@@ -344,8 +335,7 @@ static NDIS_STATUS mp_set_power_state(struct ndis_device *wnd,
 				" resumed", wnd->net_dev->name, state);
 		EXIT1(return status);
 	} else {
-		if (down_interruptible(&wnd->tx_ring_mutex))
-			EXIT1(return NDIS_STATUS_FAILURE);
+		mutex_lock(&wnd->tx_ring_mutex);
 		netif_device_detach(wnd->net_dev);
 		hangcheck_del(wnd);
 		del_iw_stats_timer(wnd);
@@ -354,7 +344,8 @@ static NDIS_STATUS mp_set_power_state(struct ndis_device *wnd,
 			status = mp_set_int(wnd, OID_PNP_ENABLE_WAKE_UP,
 					    wnd->ndis_wolopts);
 			TRACE2("0x%x, 0x%x", status, wnd->ndis_wolopts);
-			if (status == NDIS_STATUS_SUCCESS) {
+			if (status == NDIS_STATUS_SUCCESS &&
+			    wrap_is_pci_bus(wnd->wd->dev_bus)) {
 				if (wnd->ndis_wolopts)
 					wnd->wd->pci.wake_state =
 						PowerDeviceD3;
@@ -377,8 +368,7 @@ static NDIS_STATUS mp_set_power_state(struct ndis_device *wnd,
 			set_bit(HW_HALTED, &wnd->wd->hw_status);
 			status = STATUS_SUCCESS;
 		}
-		if (down_interruptible(&wnd->ndis_req_mutex))
-			WARNING("couldn't lock ndis_req_mutex");
+		mutex_lock(&wnd->ndis_req_mutex);
 		EXIT1(return status);
 	}
 }
@@ -447,7 +437,7 @@ static int setup_tx_sg_list(struct ndis_device *wnd, struct sk_buff *skb,
 		oob_data->wrap_tx_sg_list.nent = 1;
 		oob_data->ext.info[ScatterGatherListPacketInfo] =
 			&oob_data->wrap_tx_sg_list;
-		TRACE3("%Lx, %u", sg_element->address, sg_element->length);
+		TRACE3("%llx, %u", sg_element->address, sg_element->length);
 		return 0;
 	}
 	sg_list = kmalloc(sizeof(*sg_list) +
@@ -467,10 +457,10 @@ static int setup_tx_sg_list(struct ndis_device *wnd, struct sk_buff *skb,
 		sg_element++;
 		sg_element->length = frag->size;
 		sg_element->address =
-			pci_map_page(wnd->wd->pci.pdev, frag->page,
+			pci_map_page(wnd->wd->pci.pdev, skb_frag_page(frag),
 				     frag->page_offset, frag->size,
 				     PCI_DMA_TODEVICE);
-		TRACE3("%Lx, %u", sg_element->address, sg_element->length);
+		TRACE3("%llx, %u", sg_element->address, sg_element->length);
 	}
 	oob_data->ext.info[ScatterGatherListPacketInfo] = sg_list;
 	return 0;
@@ -490,7 +480,7 @@ static void free_tx_sg_list(struct ndis_device *wnd,
 	if (sg_list->nent == 1)
 		EXIT3(return);
 	for (i = 1; i < sg_list->nent; i++, sg_element++) {
-		TRACE3("%Lx, %u", sg_element->address, sg_element->length);
+		TRACE3("%llx, %u", sg_element->address, sg_element->length);
 		pci_unmap_page(wnd->wd->pci.pdev, sg_element->address,
 			       sg_element->length, PCI_DMA_TODEVICE);
 	}
@@ -584,7 +574,7 @@ void free_tx_packet(struct ndis_device *wnd, struct ndis_packet *packet,
 	    ((pool->max_descr - pool->num_used_descr) >=
 	     (wnd->max_tx_packets / 4))) {
 		set_bit(NETIF_WAKEQ, &wnd->ndis_pending_work);
-		schedule_wrapndis_work(&wnd->ndis_work);
+		queue_work(wrapndis_wq, &wnd->ndis_work);
 	}
 	EXIT4(return);
 }
@@ -683,16 +673,15 @@ static u8 mp_tx_packets(struct ndis_device *wnd, u8 start, u8 n)
 	EXIT3(return sent);
 }
 
-static void tx_worker(worker_param_t param)
+static void tx_worker(struct work_struct *work)
 {
 	struct ndis_device *wnd;
 	s8 n;
 
-	wnd = worker_param_data(param, struct ndis_device, tx_work);
+	wnd = container_of(work, struct ndis_device, tx_work);
 	ENTER3("tx_ok %d", wnd->tx_ok);
 	while (wnd->tx_ok) {
-		if (down_interruptible(&wnd->tx_ring_mutex))
-			break;
+		mutex_lock(&wnd->tx_ring_mutex);
 		spin_lock_bh(&wnd->tx_ring_lock);
 		n = wnd->tx_ring_end - wnd->tx_ring_start;
 		TRACE3("%d, %d, %d", wnd->tx_ring_start, wnd->tx_ring_end, n);
@@ -703,7 +692,7 @@ static void tx_worker(worker_param_t param)
 				n = TX_RING_SIZE - wnd->tx_ring_start;
 			else {
 				spin_unlock_bh(&wnd->tx_ring_lock);
-				up(&wnd->tx_ring_mutex);
+				mutex_unlock(&wnd->tx_ring_mutex);
 				break;
 			}
 		} else if (n < 0)
@@ -718,7 +707,7 @@ static void tx_worker(worker_param_t param)
 				(wnd->tx_ring_start + n) % TX_RING_SIZE;
 			wnd->is_tx_ring_full = 0;
 		}
-		up(&wnd->tx_ring_mutex);
+		mutex_unlock(&wnd->tx_ring_mutex);
 		TRACE3("%d, %d, %d", wnd->tx_ring_start, wnd->tx_ring_end, n);
 	}
 	EXIT3(return);
@@ -749,7 +738,7 @@ static int tx_skbuff(struct sk_buff *skb, struct net_device *dev)
 	}
 	spin_unlock(&wnd->tx_ring_lock);
 	TRACE4("ring: %d, %d", wnd->tx_ring_start, wnd->tx_ring_end);
-	schedule_wrapndis_work(&wnd->tx_work);
+	queue_work(wrapndis_wq, &wnd->tx_work);
 	return NETDEV_TX_OK;
 }
 
@@ -803,33 +792,68 @@ static int set_packet_filter(struct ndis_device *wnd, ULONG packet_filter)
 
 void set_media_state(struct ndis_device *wnd, enum ndis_media_state state)
 {
-	ENTER2("state: 0x%x", state);
-	if (state == NdisMediaStateConnected) {
-		netif_carrier_on(wnd->net_dev);
+	struct net_device *net_dev = wnd->net_dev;
+
+	ENTER2("state: 0x%x, carrier %d", state, netif_carrier_ok(net_dev));
+	switch (state) {
+	case NdisMediaStateConnected:
+		if (netif_carrier_ok(net_dev))
+			return;
+		netif_carrier_on(net_dev);
 		wnd->tx_ok = 1;
-		if (netif_queue_stopped(wnd->net_dev))
-			netif_wake_queue(wnd->net_dev);
+		if (netif_queue_stopped(net_dev))
+			netif_wake_queue(net_dev);
 		if (wnd->physical_medium == NdisPhysicalMediumWirelessLan) {
 			set_bit(LINK_STATUS_ON, &wnd->ndis_pending_work);
-			schedule_wrapndis_work(&wnd->ndis_work);
+			queue_work(wrapndis_wq, &wnd->ndis_work);
 		}
-	} else if (state == NdisMediaStateDisconnected) {
-		netif_carrier_off(wnd->net_dev);
-		netif_stop_queue(wnd->net_dev);
+		break;
+	case NdisMediaStateDisconnected:
+		if (!netif_carrier_ok(net_dev))
+			return;
+		netif_carrier_off(net_dev);
+		netif_stop_queue(net_dev);
 		wnd->tx_ok = 0;
 		if (wnd->physical_medium == NdisPhysicalMediumWirelessLan) {
 			memset(&wnd->essid, 0, sizeof(wnd->essid));
 			set_bit(LINK_STATUS_OFF, &wnd->ndis_pending_work);
-			schedule_wrapndis_work(&wnd->ndis_work);
+			queue_work(wrapndis_wq, &wnd->ndis_work);
 		}
-	} else {
+		break;
+	default:
 		WARNING("invalid media state: 0x%x", state);
+		break;
 	}
 }
 
+static int ndis_net_dev_init(struct net_device *net_dev)
+{
+	struct ndis_device *wnd = netdev_priv(net_dev);
+
+	ENTER1("%p", wnd);
+	wrap_procfs_add_ndis_device(wnd);
+	EXIT1(return 0);
+}
+
+static void ndis_net_dev_uninit(struct net_device *net_dev)
+{
+	struct ndis_device *wnd = netdev_priv(net_dev);
+
+	ENTER1("%p", wnd);
+	wrap_procfs_remove_ndis_device(wnd);
+	EXIT1(return);
+}
+
 static int ndis_net_dev_open(struct net_device *net_dev)
 {
-	ENTER1("%p", netdev_priv(net_dev));
+	int status, res;
+	struct ndis_device *wnd = netdev_priv(net_dev);
+
+	ENTER1("%p", wnd);
+	res = mp_query_int(wnd, OID_GEN_MEDIA_CONNECT_STATUS, &status);
+	if (res == NDIS_STATUS_SUCCESS && status >= NdisMediaStateConnected &&
+	    status <= NdisMediaStateDisconnected)
+		set_media_state(wnd, status);
 	netif_start_queue(net_dev);
 	netif_poll_enable(net_dev);
 	EXIT1(return 0);
@@ -886,7 +910,7 @@ static void ndis_set_multicast_list(struct net_device *dev)
 {
 	struct ndis_device *wnd = netdev_priv(dev);
 	set_bit(SET_MULTICAST_LIST, &wnd->ndis_pending_work);
-	schedule_wrapndis_work(&wnd->ndis_work);
+	queue_work(wrapndis_wq, &wnd->ndis_work);
 }
 
 /* called from BH context */
@@ -920,7 +944,7 @@ static void update_iw_stats(struct ndis_device *wnd)
 		qual = 100;
 
 	iw_stats->qual.noise = WL_NOISE;
-	iw_stats->qual.qual  = qual;
+	iw_stats->qual.qual = qual;
 
 	res = mp_query(wnd, OID_802_11_STATISTICS,
 		       &ndis_stats, sizeof(ndis_stats));
@@ -950,20 +974,34 @@ static void set_multicast_list(struct ndis_device *wnd)
 		packet_filter |= NDIS_PACKET_TYPE_PROMISCUOUS |
 			NDIS_PACKET_TYPE_ALL_LOCAL;
 	} else if (net_dev->flags & IFF_ALLMULTI ||
-		   net_dev->mc_count > wnd->multicast_size) {
+		   netdev_mc_count(net_dev) > wnd->multicast_size) {
 		packet_filter |= NDIS_PACKET_TYPE_ALL_MULTICAST;
 		TRACE2("0x%08x", packet_filter);
-	} else if (net_dev->mc_count > 0) {
+	} else if (netdev_mc_count(net_dev) > 0) {
 		int i, size;
 		char *buf;
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,35)
+		struct netdev_hw_addr *ha;
+#else
 		struct dev_mc_list *mclist;
-		size = min(wnd->multicast_size, net_dev->mc_count);
-		TRACE2("%d, %d", wnd->multicast_size, net_dev->mc_count);
+#endif
+		size = min(wnd->multicast_size, netdev_mc_count(net_dev));
+		TRACE2("%d, %d", wnd->multicast_size, netdev_mc_count(net_dev));
 		buf = kmalloc(size * ETH_ALEN, GFP_KERNEL);
 		if (!buf) {
 			WARNING("couldn't allocate memory");
 			EXIT2(return);
 		}
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,35)
+		i = 0;
+		netdev_for_each_mc_addr(ha, net_dev) {
+			if (i >= size)
+				break;
+			memcpy(buf + i * ETH_ALEN, ha->addr, ETH_ALEN);
+			TRACE2(MACSTRSEP, MAC2STR(ha->addr));
+			i++;
+		}
+#else
 		mclist = net_dev->mc_list;
 		for (i = 0; i < size && mclist; mclist = mclist->next) {
 			if (mclist->dmi_addrlen != ETH_ALEN)
@@ -972,6 +1010,7 @@ static void set_multicast_list(struct ndis_device *wnd)
 			TRACE2(MACSTRSEP, MAC2STR(mclist->dmi_addr));
 			i++;
 		}
+#endif
 		res = mp_set(wnd, OID_802_3_MULTICAST_LIST, buf, i * ETH_ALEN);
 		if (res == NDIS_STATUS_SUCCESS && i > 0)
 			packet_filter |= NDIS_PACKET_TYPE_MULTICAST;
@@ -1055,7 +1094,7 @@ static void iw_stats_timer_proc(unsigned long data)
 	ENTER2("%d", wnd->iw_stats_interval);
 	if (wnd->iw_stats_interval > 0) {
 		set_bit(COLLECT_IW_STATS, &wnd->ndis_pending_work);
-		schedule_wrapndis_work(&wnd->ndis_work);
+		queue_work(wrapndis_wq, &wnd->ndis_work);
 	}
 	mod_timer(&wnd->iw_stats_timer, jiffies + wnd->iw_stats_interval);
 }
@@ -1086,7 +1125,7 @@ static void hangcheck_proc(unsigned long data)
 	ENTER3("%d", wnd->hangcheck_interval);
 	if (wnd->hangcheck_interval > 0) {
 		set_bit(HANGCHECK, &wnd->ndis_pending_work);
-		schedule_wrapndis_work(&wnd->ndis_work);
+		queue_work(wrapndis_wq, &wnd->ndis_work);
 	}
 	mod_timer(&wnd->hangcheck_timer, jiffies + wnd->hangcheck_interval);
 	EXIT3(return);
@@ -1118,11 +1157,11 @@ void hangcheck_del(struct ndis_device *wnd)
 }
 
 /* worker procedure to take care of setting/checking various states */
-static void ndis_worker(worker_param_t param)
+static void wrapndis_worker(struct work_struct *work)
 {
 	struct ndis_device *wnd;
 
-	wnd = worker_param_data(param, struct ndis_device, ndis_work);
+	wnd = container_of(work, struct ndis_device, ndis_work);
 	WORKTRACE("0x%lx", wnd->ndis_pending_work);
 
 	if (test_and_clear_bit(NETIF_WAKEQ, &wnd->ndis_pending_work)) {
@@ -1199,11 +1238,11 @@ static void get_encryption_capa(struct ndis_device *wnd, char *buf,
 			TRACE2("%d", net_types->types[i]);
 			set_bit(net_types->types[i], &types);
 		}
-		if (types & Ndis802_11OFDM24)
+		if (test_bit(Ndis802_11OFDM24, &types))
 			mode = Ndis802_11OFDM24;
-		else if (types & Ndis802_11DS)
+		else if (test_bit(Ndis802_11DS, &types))
 			mode = Ndis802_11DS;
-		else if (types & Ndis802_11OFDM5)
+		else if (test_bit(Ndis802_11OFDM5, &types))
 			mode = Ndis802_11OFDM5;
 		else
 			mode = Ndis802_11DS;
@@ -1258,7 +1297,7 @@ static void get_encryption_capa(struct ndis_device *wnd, char *buf,
 	ndis_key.index = 0xC0000001;
 	ndis_key.struct_size = sizeof(ndis_key);
 	res = mp_set(wnd, OID_802_11_ADD_KEY, &ndis_key, ndis_key.struct_size);
-	TRACE2("%08X, %lu", res, (unsigned long)sizeof(ndis_key));
+	TRACE2("%08X, %zu", res, sizeof(ndis_key));
 	if (res && res != NDIS_STATUS_INVALID_DATA)
 		EXIT1(return);
 	res = mp_query(wnd, OID_802_11_ASSOCIATION_INFORMATION,
@@ -1324,7 +1363,7 @@ wstdcall NTSTATUS NdisDispatchDeviceControl(struct device_object *fdo,
 	struct ndis_device *wnd;
 
 	TRACE3("fdo: %p", fdo);
-	/* for now, we don't have anything intresting here, so pass it
+	/* for now, we don't have anything interesting here, so pass it
 	 * down to bus driver */
 	wnd = fdo->reserved;
 	return IoPassIrpDown(wnd->nmb->pdo, irp);
@@ -1478,7 +1517,7 @@ static void set_task_offload(struct ndis_device *wnd, void *buf,
 			task_offload_header->offset_first_task);
 	while (1) {
 		TRACE1("%d, %d", task_offload->version, task_offload->task);
-		switch(task_offload->task) {
+		switch (task_offload->task) {
 		case TcpIpChecksumNdisTask:
 			csum = (void *)task_offload->task_buf;
 			break;
@@ -1719,13 +1758,15 @@ static int notifier_event(struct notifier_block *notifier, unsigned long event,
 	if (net_dev->ethtool_ops == &ndis_ethtool_ops
 	    && event == NETDEV_CHANGENAME) {
 		struct ndis_device *wnd = netdev_priv(net_dev);
+
 		/* called with rtnl lock held, so no need to lock */
-		wrap_procfs_remove_ndis_device(wnd);
-		printk(KERN_INFO "%s: changing interface name from '%s' to "
-		       "'%s'\n", DRIVER_NAME, wnd->netdev_name, net_dev->name);
-		memcpy(wnd->netdev_name, net_dev->name,
-		       sizeof(wnd->netdev_name));
-		wrap_procfs_add_ndis_device(wnd);
+		if (likely(wnd->procfs_iface)) {
+			printk(KERN_INFO "%s: changing interface name from "
+			       "'%s' to '%s'\n", DRIVER_NAME,
+			       wnd->procfs_iface->name, net_dev->name);
+			wrap_procfs_remove_ndis_device(wnd);
+			wrap_procfs_add_ndis_device(wnd);
+		}
 	}
 	return NOTIFY_DONE;
 }
@@ -1736,11 +1777,17 @@ static struct notifier_block netdev_notifier = {
 
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 29)
 static const struct net_device_ops ndis_netdev_ops = {
+	.ndo_init = ndis_net_dev_init,
+	.ndo_uninit = ndis_net_dev_uninit,
 	.ndo_open = ndis_net_dev_open,
 	.ndo_stop = ndis_net_dev_close,
 	.ndo_start_xmit = tx_skbuff,
 	.ndo_change_mtu = ndis_change_mtu,
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 2, 0)
+	.ndo_set_rx_mode = ndis_set_multicast_list,
+#else
 	.ndo_set_multicast_list = ndis_set_multicast_list,
+#endif
 	.ndo_set_mac_address = ndis_set_mac_address,
 	.ndo_get_stats = ndis_get_stats,
 #ifdef CONFIG_NET_POLL_CONTROLLER
@@ -1791,6 +1838,8 @@ static NDIS_STATUS ndis_start_device(struct ndis_device *wnd)
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 29)
 	net_dev->netdev_ops = &ndis_netdev_ops;
 #else
+	net_dev->init = ndis_net_dev_init;
+	net_dev->uninit = ndis_net_dev_uninit;
 	net_dev->open = ndis_net_dev_open;
 	net_dev->hard_start_xmit = tx_skbuff;
 	net_dev->stop = ndis_net_dev_close;
@@ -1802,9 +1851,11 @@ static NDIS_STATUS ndis_start_device(struct ndis_device *wnd)
 	net_dev->poll_controller = ndis_poll_controller;
 #endif
 #endif
+#ifdef CONFIG_WIRELESS_EXT
 	if (wnd->physical_medium == NdisPhysicalMediumWirelessLan) {
 		net_dev->wireless_handlers = &ndis_handler_def;
 	}
+#endif
 	net_dev->ethtool_ops = &ndis_ethtool_ops;
 	if (wnd->mp_interrupt)
 		net_dev->irq = wnd->mp_interrupt->irq;
@@ -1834,7 +1885,6 @@ static NDIS_STATUS ndis_start_device(struct ndis_device *wnd)
 		ERROR("cannot register net device %s", net_dev->name);
 		goto err_register;
 	}
-	memcpy(wnd->netdev_name, net_dev->name, sizeof(wnd->netdev_name));
 	memset(buf, 0, buf_len);
 	status = mp_query(wnd, OID_GEN_VENDOR_DESCRIPTION, buf, buf_len);
 	if (status != NDIS_STATUS_SUCCESS) {
@@ -1849,8 +1899,8 @@ static NDIS_STATUS ndis_start_device(struct ndis_device *wnd)
 	       "driver: %s, version: 0x%x, NDIS version: 0x%x, vendor: '%s', "
 	       "%s\n", net_dev->name, MAC2STR(net_dev->dev_addr),
 	       deserialized_driver(wnd) ? "" : "serialized ",
-	       wnd->wd->driver->name, n, wnd->drv_ndis_version, buf,
-	       wnd->wd->conf_file_name);
+	       wd->driver->name, n, wnd->drv_ndis_version, buf,
+	       wd->conf_file_name);
 
 	if (deserialized_driver(wnd)) {
 		/* deserialized drivers don't have a limit, but we
@@ -1902,7 +1952,7 @@ static NDIS_STATUS ndis_start_device(struct ndis_device *wnd)
 	if (wnd->physical_medium == NdisPhysicalMediumWirelessLan) {
 		mp_set_int(wnd, OID_802_11_POWER_MODE, NDIS_POWER_OFF);
 		get_encryption_capa(wnd, buf, buf_len);
-		TRACE1("capbilities = %ld", wnd->capa.encr);
+		TRACE1("capabilities = %ld", wnd->capa.encr);
 		printk(KERN_INFO "%s: encryption modes supported: "
 		       "%s%s%s%s%s%s%s\n", net_dev->name,
 		       test_bit(Ndis802_11Encryption1Enabled, &wnd->capa.encr) ?
@@ -1913,22 +1963,18 @@ static NDIS_STATUS ndis_start_device(struct ndis_device *wnd)
 		       test_bit(Ndis802_11AuthModeWPA2, &wnd->capa.auth) ?
 		       ", WPA2" : "",
 		       test_bit(Ndis802_11AuthModeWPA2PSK, &wnd->capa.auth) ?
-		       ", WPA2PSK" : "",
+		       ", WPA2-PSK" : "",
 
 		       test_bit(Ndis802_11Encryption3Enabled, &wnd->capa.encr) ?
 		       "; AES/CCMP with WPA" : "",
 		       test_bit(Ndis802_11AuthModeWPA2, &wnd->capa.auth) ?
 		       ", WPA2" : "",
 		       test_bit(Ndis802_11AuthModeWPA2PSK, &wnd->capa.auth) ?
-		       ", WPA2PSK" : "");
+		       ", WPA2-PSK" : "");
 
 		set_default_iw_params(wnd);
 	}
-	status = mp_query_int(wnd, OID_GEN_MEDIA_CONNECT_STATUS, (int *)buf);
-	if (status == NDIS_STATUS_SUCCESS)
-		set_media_state(wnd, *((int *)buf));
 	kfree(buf);
-	wrap_procfs_add_ndis_device(wnd);
 	hangcheck_add(wnd);
 	add_iw_stats_timer(wnd);
 	EXIT1(return NDIS_STATUS_SUCCESS);
@@ -1940,26 +1986,30 @@ buffer_pool_err:
 		wnd->tx_packet_pool = NULL;
 	}
 packet_pool_err:
+	unregister_netdev(net_dev);
+	wnd->max_tx_packets = 0;
 err_register:
 	kfree(buf);
 err_start:
-	ndis_remove_device(wnd);
+	mp_halt(wnd);
 	EXIT1(return NDIS_STATUS_FAILURE);
 }
 
 static int ndis_remove_device(struct ndis_device *wnd)
 {
 	s8 tx_pending;
+	int our_mutex;
 
 	/* prevent setting essid during disassociation */
 	memset(&wnd->essid, 0, sizeof(wnd->essid));
 	wnd->tx_ok = 0;
+	netif_carrier_off(wnd->net_dev);
 	if (wnd->max_tx_packets)
 		unregister_netdev(wnd->net_dev);
-	netif_carrier_off(wnd->net_dev);
 	/* if device is suspended, but resume failed, tx_ring_mutex
 	 * may already be locked */
-	if (down_trylock(&wnd->tx_ring_mutex))
+	our_mutex = mutex_trylock(&wnd->tx_ring_mutex);
+	if (!our_mutex)
 		WARNING("couldn't obtain tx_ring_mutex");
 	spin_lock_bh(&wnd->tx_ring_lock);
 	tx_pending = wnd->tx_ring_end - wnd->tx_ring_start;
@@ -1977,8 +2027,8 @@ static int ndis_remove_device(struct ndis_device *wnd)
 		wnd->tx_ring_start = (wnd->tx_ring_start + 1) % TX_RING_SIZE;
 	}
 	spin_unlock_bh(&wnd->tx_ring_lock);
-	up(&wnd->tx_ring_mutex);
-	wrap_procfs_remove_ndis_device(wnd);
+	if (our_mutex)
+		mutex_unlock(&wnd->tx_ring_mutex);
 	mp_halt(wnd);
 	ndis_exit_device(wnd);
 
@@ -1990,8 +2040,7 @@ static int ndis_remove_device(struct ndis_device *wnd)
 		NdisFreeBufferPool(wnd->tx_buffer_pool);
 		wnd->tx_buffer_pool = NULL;
 	}
-	if (wnd->pmkids)
-		kfree(wnd->pmkids);
+	kfree(wnd->pmkids);
 	printk(KERN_INFO "%s: device %s removed\n", DRIVER_NAME,
 	       wnd->net_dev->name);
 	kfree(wnd->nmb);
@@ -1999,8 +2048,8 @@ static int ndis_remove_device(struct ndis_device *wnd)
 	EXIT2(return 0);
 }
 
-static wstdcall NTSTATUS NdisAddDevice(struct driver_object *drv_obj,
-				       struct device_object *pdo)
+static NTSTATUS ndis_add_device(struct driver_object *drv_obj,
+				struct device_object *pdo)
 {
 	struct device_object *fdo;
 	struct ndis_mp_block *nmb;
@@ -2041,7 +2090,7 @@ static wstdcall NTSTATUS NdisAddDevice(struct driver_object *drv_obj,
 		free_netdev(net_dev);
 		return STATUS_RESOURCES;
 	}
-#if defined(DEBUG) && DEBUG >= 6
+#if DEBUG >= 6
 	/* poison nmb so if a driver accesses uninitialized pointers, we
 	 * know what it is */
 	for (i = 0; i < sizeof(*nmb) / sizeof(unsigned long); i++)
@@ -2051,7 +2100,6 @@ static wstdcall NTSTATUS NdisAddDevice(struct driver_object *drv_obj,
 	wnd->nmb = nmb;
 	nmb->wnd = wnd;
 	nmb->pdo = pdo;
-	wd->wnd = wnd;
 	wnd->wd = wd;
 	wnd->net_dev = net_dev;
 	fdo->reserved = wnd;
@@ -2064,10 +2112,10 @@ static wstdcall NTSTATUS NdisAddDevice(struct driver_object *drv_obj,
 	}
 	nmb->next_device = IoAttachDeviceToDeviceStack(fdo, pdo);
 	spin_lock_init(&wnd->tx_ring_lock);
-	sema_init(&wnd->tx_ring_mutex, 1);
-	sema_init(&wnd->ndis_req_mutex, 1);
+	mutex_init(&wnd->tx_ring_mutex);
+	mutex_init(&wnd->ndis_req_mutex);
 	wnd->ndis_req_done = 0;
-	initialize_work(&wnd->tx_work, tx_worker, wnd);
+	INIT_WORK(&wnd->tx_work, tx_worker);
 	wnd->tx_ring_start = 0;
 	wnd->tx_ring_end = 0;
 	wnd->is_tx_ring_full = 0;
@@ -2085,7 +2133,7 @@ static wstdcall NTSTATUS NdisAddDevice(struct driver_object *drv_obj,
 	memset(&wnd->essid, 0, sizeof(wnd->essid));
 	memset(&wnd->encr_info, 0, sizeof(wnd->encr_info));
 	wnd->infrastructure_mode = Ndis802_11Infrastructure;
-	initialize_work(&wnd->ndis_work, ndis_worker, wnd);
+	INIT_WORK(&wnd->ndis_work, wrapndis_worker);
 	wnd->iw_stats_enabled = TRUE;
 
 	TRACE1("nmb: %p, pdo: %p, fdo: %p, attached: %p, next: %p",
@@ -2107,7 +2155,7 @@ static wstdcall NTSTATUS NdisAddDevice(struct driver_object *drv_obj,
 int init_ndis_driver(struct driver_object *drv_obj)
 {
 	ENTER1("%p", drv_obj);
-	drv_obj->drv_ext->add_device = NdisAddDevice;
+	drv_obj->drv_ext->add_device = ndis_add_device;
 	return 0;
 }
 
@@ -2116,8 +2164,7 @@ int wrapndis_init(void)
 	wrapndis_wq = create_singlethread_workqueue("wrapndis_wq");
 	if (!wrapndis_wq)
 		EXIT1(return -ENOMEM);
-	wrapndis_worker_thread = wrap_worker_init(wrapndis_wq);
-	TRACE1("%p", wrapndis_worker_thread);
+	TRACE1("wrapndis_wq: %p", wrapndis_wq);
 	register_netdevice_notifier(&netdev_notifier);
 	return 0;
 }
@@ -2127,7 +2174,4 @@ void wrapndis_exit(void)
 	unregister_netdevice_notifier(&netdev_notifier);
 	if (wrapndis_wq)
 		destroy_workqueue(wrapndis_wq);
-	TRACE1("%p", wrapndis_worker_thread);
-	if (wrapndis_worker_thread)
-		ObDereferenceObject(wrapndis_worker_thread);
 }
