@@ -17,39 +17,35 @@
 #include "iw_ndis.h"
 #include "loader.h"
 #include "pnp.h"
+#include "wrapper.h"
 
 char *if_name = "wlan%d";
 int proc_uid, proc_gid;
 int hangcheck_interval;
-char *utils_version = UTILS_VERSION;
-
-#if defined(DEBUG) && (DEBUG > 0)
+static char *utils_version = UTILS_VERSION;
 int debug = DEBUG;
-#else
-int debug = 0;
-#endif
 
-WRAP_MODULE_PARM_STRING(if_name, 0400);
+module_param(if_name, charp, 0400);
 MODULE_PARM_DESC(if_name, "Network interface name or template "
 		 "(default: wlan%d)");
-WRAP_MODULE_PARM_INT(proc_uid, 0600);
+module_param(proc_uid, int, 0600);
 MODULE_PARM_DESC(proc_uid, "The uid of the files created in /proc "
 		 "(default: 0).");
-WRAP_MODULE_PARM_INT(proc_gid, 0600);
+module_param(proc_gid, int, 0600);
 MODULE_PARM_DESC(proc_gid, "The gid of the files created in /proc "
 		 "(default: 0).");
-WRAP_MODULE_PARM_INT(debug, 0600);
+module_param(debug, int, 0600);
 MODULE_PARM_DESC(debug, "debug level");
 
 /* 0 - default value provided by NDIS driver,
  * positive value - force hangcheck interval to that many seconds
  * negative value - disable hangcheck
  */
-WRAP_MODULE_PARM_INT(hangcheck_interval, 0600);
+module_param(hangcheck_interval, int, 0600);
 MODULE_PARM_DESC(hangcheck_interval, "The interval, in seconds, for checking"
 		 " if driver is hung. (default: 0)");
 
-WRAP_MODULE_PARM_STRING(utils_version, 0400);
+module_param(utils_version, charp, 0400);
 MODULE_PARM_DESC(utils_version, "Compatible version of utils "
 		 "(read only: " UTILS_VERSION ")");
 
@@ -65,21 +61,19 @@ MODULE_LICENSE("GPL");
 static void module_cleanup(void)
 {
 	loader_exit();
-#ifdef CONFIG_USB
 	usb_exit();
-#endif
-
 	wrap_procfs_remove();
 	wrapndis_exit();
 	ndis_exit();
-	rtl_exit();
-	crt_exit();
 	ntoskernel_exit();
 	wrapmem_exit();
 }
 
 static int __init wrapper_init(void)
 {
+#ifdef TAINT_OOT_MODULE
+	add_taint(TAINT_OOT_MODULE);
+#endif
 	printk(KERN_INFO "%s version %s loaded (smp=%s, preempt=%s)\n",
 	       DRIVER_NAME, DRIVER_VERSION,
 #ifdef CONFIG_SMP
@@ -97,12 +91,9 @@ static int __init wrapper_init(void)
 #endif
 		);
 
-	if (wrapmem_init() || ntoskernel_init() || crt_init() ||
-	    rtl_init() || ndis_init() || wrapndis_init() ||
-#ifdef CONFIG_USB
-	    usb_init() ||
-#endif
-	    wrap_procfs_init() || loader_init()) {
+	if (wrapmem_init() || ntoskernel_init() || ndis_init() ||
+	    wrapndis_init() || usb_init() || wrap_procfs_init() ||
+	    loader_init()) {
 		module_cleanup();
 		ERROR("%s: initialization failed", DRIVER_NAME);
 		return -EINVAL;
